classdef Beamformer < handle
    % BEAMFORMER Class to beamform visual gamma, compute virtual sources.
    %
    % Beamformer Properties:
    %   cfg                 = A config struct as created by fillConfigs().
    %   subject             = A subject struct with subject-specific info and
    %                         already computed analysis results (eg beamformer
    %                         filters, gamma peaks, etc.).
    %   data                = Cleaned, preprocessed, sensorlevel MEG data.
    %   mri                 = The subject-specific 1mm-resolution MPRAGE.
    %   sensors             = Sensor-positions from the CTF275.
    %   headmodel           = The subject-specific headmodel, created from
    %                         the MRI.
    %   grid                = The subject-specific dipole grid, warped from
    %                         an MNI-compatible template grid.
    %   mni_coordinates     = MNI coordinates of each dipole
    %   gammapeak_sensors   = Subject-specific gamma frequency peak on the
    %                         occipital sensor data.
    %   gammasources        = Gamma power increase at stimulation at each
    %                         dipole location - the results of the DICS
    %                         beamformer. Split into gratingsources and
    %                         colorsources.
    %   filters             = LCMV sensors->source filter matrix for each
    %                         dipole.
    %
    % Beamformer Methods:
    %   Beamformer(cfg, subj_nr) = Class constructor.
    %   loadData()               = Load MEG and MRI data from disk.
    %   createSourcemodel()      = Create subject-specific dipole grid.
    
    
    properties
        cfg
        subject
        data
        mri
        sensors
        headmodel
        grid
        mni_coordinates
        subject_coordinates
        color_gammapeak_sensors
        grating_gammapeak_sensors
        color_gammapeak_V1
        grating_gammapeak_V1
        gammasources
        filters
    end
    
    methods
        function obj = Beamformer(subj_nr)
            [obj.cfg, obj.subject] = setup(subj_nr);
        end
        
        function doEverything(obj) % Run the whole beamforming pipeline
            loadData(obj);
            createSourcemodel(obj);
            findGammaPeak(obj);
            findGammasourceDICS(obj);
            computeLCMV(obj);
        end
        
        function loadData(obj) % Load MRI, sensors and MEG data. Align MRI to CTF.
            %% Load MRI and data
            fprintf('Loading data for subject %s.\n', obj.subject.subj_nr);
            obj.mri = ft_read_mri(obj.subject.mridata);
            ESIload(fullfile(obj.subject.preproc_data, 'data_cleaned'));
            obj.data = data_cleaned;
            
            try % attempt to load existing transfer matrices
                ESIload(fullfile(obj.subject.preproc_data, ...
                    'transform_orig2vox'));
                ESIload(fullfile(obj.subject.preproc_data, ...
                    'transform_vox2ctf'));
                ESIload(fullfile(obj.subject.preproc_data, ...
                    'transform_vox2acpc'));
                transMatricesExist = true;
            catch
                transMatricesExist = false;
            end
            
            %% Realign MRI to CTF
            % Take existing MRI realignment to CTF coordinates or redo manually
            if transMatricesExist && ~obj.cfg.beam.mri_realign_by_hand
                disp('Using saved MRI realignment to ACPC to CTF space.');
                obj.mri.transform = transform_orig2vox;
                conf = [];
                conf.resolution = 1;
                conf.dim = [256 256 256];
                obj.mri = ft_volumereslice(conf, obj.mri);
                obj.mri.transform = transform_vox2ctf;
                obj.mri.coordsys = 'ctf';
            else
                % Realign to ctf coordinates by marking fiducials, save
                % results.
                disp('Realigning MRI by hand to CTF space.');
                obj.alignHCP()
            end
        end
        
        function createSourcemodel(obj)
            switch obj.cfg.general.HCP_or_grid_sources
                case 'grid'
                    disp('Creating MNI-aligned regular grid sourcemodel.');
                    createGridSourcemodel(obj);
                case 'HCP'
                    disp(['Creating HCP-Atlas-based cortical midsurface '...
                        'sourcemodel.']);
                    createHCPSourcemodel(obj);
                otherwise
                    error(['Invalid source model type requested. '...
                        'Must be ''HCP'' or ''grid''.'])
            end
        end
        
        function createGridSourcemodel(obj)
            %% Create subject headmodel
            % Segment subject mri.
            mri_segm = ft_volumesegment(obj.cfg.beam.segment, obj.mri);
            
            % Create subject headmodel and convert to cm.
            obj.headmodel = ft_prepare_headmodel(obj.cfg.beam.headmodel, ...
                mri_segm);
            obj.headmodel = ft_convert_units(obj.headmodel, 'cm');
            
            %% Create MNI template grid.
            % Segment template brain and create headmodel
            template = ft_read_mri(obj.cfg.beam.template.file);
            template.coordsys  = obj.cfg.beam.template.coordsystem;
            template_seg = ft_volumesegment(obj.cfg.beam.template, template);
            template_headmodel = ft_prepare_headmodel(obj.cfg.beam.headmodel, template_seg);
            % convert the headmodel to cm
            template_headmodel = ft_convert_units(template_headmodel, 'cm');
            % construct the dipole grid in the template brain coordinates
            obj.cfg.beam.template.sourcemodel.headmodel = template_headmodel;
            template_grid = ft_prepare_sourcemodel(...
                obj.cfg.beam.template.sourcemodel);
            obj.mni_coordinates = template_grid.pos;
            
            %% Create subject-specific dipole grid
            obj.cfg.beam.sourcemodel.grid.template  = template_grid;
            obj.cfg.beam.sourcemodel.mri            = obj.mri;
            obj.grid = ft_prepare_sourcemodel(obj.cfg.beam.sourcemodel);
            
            % Create quality control plots
            makeGridplots(obj, template_headmodel, template_grid);
        end
        
        function createHCPSourcemodel(obj) % Load the 32k_fs_LR midsurface
            % grid created with freesurfer/connectome workbench, use them
            % as source models.
            
            % Segment subject mri.
            % make brain mask a bit bigger to get all dipoles inside
            obj.cfg.beam.segment.brainthreshold = 0.3;
            mri_segm = ft_volumesegment(obj.cfg.beam.segment, obj.mri);
            
            % Create subject headmodel and convert to cm.
            obj.headmodel = ft_prepare_headmodel(obj.cfg.beam.headmodel, ...
                mri_segm);
            obj.headmodel = ft_convert_units(obj.headmodel, 'cm');
            
            % Load labeled cortex surfaces
            left_hem    = ft_read_headshape(obj.subject.HCPLsurfpath);
            right_hem   = ft_read_headshape(obj.subject.HCPRsurfpath);
            atlas       = ft_read_cifti(obj.cfg.beam.hcp.atlas, ...
                'mapname', 'array', 'readdata', true);
            
            % Load and apply ACPC->CTF transfer matrices. Subject grids are
            % in ACPC coordinates and mm.
            ESIload(fullfile(obj.subject.preproc_data, ...
                'transform_acpc2ctf'));
            left_hem = ft_transform_geometry(transform_acpc2ctf, left_hem);
            right_hem = ft_transform_geometry(transform_acpc2ctf, right_hem);
            left_hem = ft_convert_units(left_hem, 'cm');
            right_hem = ft_convert_units(right_hem, 'cm');
            
            % Create quality control plots
            disp('Creating source model QC plot.');
            makeHCPsurfplots(obj, left_hem, right_hem, atlas);
            
            % Merge left and right hemisphere into one sourcemodel
            conf = [];
            conf.sourcemodel.pos    = [left_hem.pos; right_hem.pos];
            conf.sourcemodel.inside = ones(size(conf.sourcemodel.pos ,1),1);
            conf.sourcemodel.unit   = 'cm';
            conf.grad               = obj.data.grad;
            conf.headmodel          = obj.headmodel;
            obj.grid = ft_prepare_sourcemodel(conf);
        end
        
        function findGammaPeak(obj) % Find gamma peak in the raw data
            %% Find the gamma spectral peak
            assert(~isempty(obj.data), 'Load data first via .loadData!');
            disp('Calling the FOOOF pipeline on sensor data.')
            [obj.color_gammapeak_sensors, obj.grating_gammapeak_sensors] = ...
                FOOOFpipeline(obj, obj.data, 'sensor');
            
            obj.subject.color_gammapeak_sensors = ...
                obj.color_gammapeak_sensors;
            obj.subject.grating_gammapeak_sensors = ...
                obj.grating_gammapeak_sensors;
            
            if obj.cfg.gammapeak.computeSourceLevel
                disp('Getting source level data.')
                freqs = Freqs(obj.subject.subj_nr);
                freqs.loadData();
                sourceDat = freqs.computeAreaSources('V1');
                disp('Calling the FOOOF pipeline.')
                [obj.color_gammapeak_V1,...
                    obj.grating_gammapeak_V1] = ...
                    FOOOFpipeline(obj, sourceDat, 'V1');
                
                obj.subject.color_gammapeak_V1 = ...
                    obj.color_gammapeak_V1;
                obj.subject.grating_gammapeak_V1 = ...
                    obj.grating_gammapeak_V1;
            end
            
            if obj.cfg.general.save_partial_results
                subject = obj.subject;
                display('Saving gamma frequency peak to subject struct.');
                ESIsave(fullfile(obj.cfg.paths.data, obj.subject.subj_nr,...
                    'Preprocessed', 'subject.mat'), 'subject');
            end
        end
        
        function findGammasourceDICS(obj) % Run DICS, find max. gamma dipole
            %% DICS beamforming
            % redefine trials into baseline and stimulus (step 1)
            % do frequency analysis (step 2)
            
            dat_whole = ft_redefinetrial(obj.cfg.beam.defall, obj.data);
            dat_base  = ft_redefinetrial(obj.cfg.beam.defbase, obj.data);
            dat_stim  = ft_redefinetrial(obj.cfg.beam.defstim, obj.data);
            
            if obj.cfg.beam.slidingwindow
                disp('Cutting data into overlapping windows');
                conf         = [];
                conf.length  = obj.cfg.beam.window_length;
                conf.overlap = obj.cfg.beam.window_overlap;
                dat_whole    = ft_redefinetrial(conf, dat_whole);
                dat_base     = ft_redefinetrial(conf, dat_base);
                dat_stim     = ft_redefinetrial(conf, dat_stim);
            end
            
            freq_whole = ft_freqanalysis(obj.cfg.beam.freq, dat_whole);
            freq_base = ft_freqanalysis(obj.cfg.beam.freq, dat_base);
            clear dat_whole dat_base
            
            % Split data into color, contrast and grating trials
            stimtypes = {'color', 'contrast', 'grating'}; %TODO
            [~,~,~,stimclass] = getStimtypes(obj.subject.behavior);
            
            for iStimtype = 1:length(stimtypes)
                conf = [];
                iTrials = find(strcmp(stimclass, ...
                    stimtypes{iStimtype}));
                conf.trials = find(ismember(dat_stim.trialinfo, iTrials));
                iDat = ft_redefinetrial(conf, dat_stim);
                freq_stim = ft_freqanalysis(obj.cfg.beam.freq, iDat);
                switch stimtypes{iStimtype}
                    case {'color', 'contrast'}
                        peaks = obj.subject.color_gammapeak_sensors;
                    case 'grating'
                        peaks = obj.subject.grating_gammapeak_sensors;
                end
                
                for iPeak = 1:length(peaks)
                    % run sourceanalysis on whole trial (step 3)
                    obj.cfg.beam.beamdics.sourcemodel  = obj.grid;
                    obj.cfg.beam.beamdics.headmodel    = obj.headmodel;
                    obj.cfg.beam.beamdics.frequency    = round(peaks(iPeak));
                    source_whole = ft_sourceanalysis(obj.cfg.beam.beamdics, freq_whole);
                    
                    % run sourceanalysis on baseline and stimulus presentation (step 4)
                    obj.cfg.beam.beamdics.sourcemodel.filter = source_whole.avg.filter;
                    source_base = ft_sourceanalysis(obj.cfg.beam.beamdics, freq_base);
                    source_stim = ft_sourceanalysis(obj.cfg.beam.beamdics, freq_stim);
                    
                    % create difference variable
                    source_diff = source_stim;
                    source_diff.avg.pow = source_stim.avg.pow ./...
                        source_base.avg.pow;
                    source_diff.mni_pos = obj.mni_coordinates;
                    %TODO: remove next line after debbugging
                    fprintf('Adding result %i, %i to object.\n', iStimtype, iPeak);
                    try
                        obj.gammasources{iStimtype, iPeak} = source_diff;
                    catch
                        disp('It failed. Moving on.')
                    end
                    fprintf('Adding result %i, %i to subject.\n', iStimtype, iPeak);
                    obj.subject.gammasources{iStimtype, iPeak} = source_diff;
                    clear source_stim source_base
                    
                    switch obj.cfg.general.HCP_or_grid_sources
                        case 'HCP'
                            
                            % plot dics results
                            template = ft_read_headshape(obj.cfg.beam.hcp.plot_template);
                            atlas = ft_read_cifti(obj.cfg.beam.hcp.atlas, ...
                                'mapname', 'array', 'readdata', true);
                            template.labels = label_HCP_areas([], atlas, ...
                                'V1', 'V2', 'V3', 'V4');
                            
                            figure('units', 'centimeters','position', [1 1 20 9], ...
                                'Visible', 'on', 'Color', 'w');
                            subplot(1,2,1, 'Units', 'normalized', ...
                                'Position', [.015, .00, .4, 1.0]);
                            HCP_source_plot([], template, source_diff.avg.pow);
                            view([40, -5]);
                            set_colormap('seq');
                            camlight('headlight')
                            subplot(1,2,2, 'Units', 'normalized', ...
                                'Position', [.45, .00, .4, 1.0]);
                            HCP_source_plot([], template, source_diff.avg.pow);
                            view([-40, -5]);
                            set_colormap('seq');
                            camlight('headlight')
                            hbar = colorbar('manual', 'Position', [0.89 0.2 0.03 0.6]);
                            ylabel(hbar, ['Induced gamma power change (times)']);
                            
                            filepath = [obj.subject.saveplots.HCPdics '_' ...
                                stimtypes{iStimtype} '_' ...
                                num2str(round(peaks(iPeak)))];
                            export_fig(filepath, '-png', '-nocrop', ...
                                '-q101', '-r300')
                            
                        case 'grid'
                            
                            source_plot = source_diff;
                            % interpolate sources on MNI-normalised MRI (step 5)
                            source_plot.pos = obj.mni_coordinates;
                            
                            norm_mri = ft_volumenormalise([], obj.mri);
                            norm_mri.unit = 'cm';
                            source_plot_interp = ft_sourceinterpolate(obj.cfg.beam.int, ...
                                source_plot , norm_mri);
                            
                            % plot dics results
                            atlas = ft_read_atlas(obj.cfg.beam.mask.atlasname);
                            conf               = [];
                            conf.method        = 'ortho';%obj.cfg.beam.plot.method;
                            conf.funparameter  = 'pow';
                            conf.maskparameter = conf.funparameter;
                            conf.atlas         = atlas;
                            conf.visible       = 'on';
                            conf.opacitylim    = 'zeromax';
                            conf.maskstyle     = 'colormix';
                            conf.slicedim = 2;
                            ft_sourceplot(conf, source_plot_interp);
                            
                            filepath = [obj.subject.saveplots.dics '_' ...
                                stimtypes{iStimtype} '_' ...
                                num2str(round(peaks(iPeak)))];
                            export_fig(filepath, ...
                                '-png', '-nocrop', '-q101', '-r600')
                            
                            
                            obj.subject.mni_pos = obj.mni_coordinates;
                            obj.subject.subject_pos = obj.subject_coordinates;
                    end
                end
            end
            
            
            if obj.cfg.general.save_partial_results
                subject = obj.subject;
                disp('Saving DICS result to subject struct.');
                ESIsave(fullfile(obj.cfg.paths.data, obj.subject.subj_nr,...
                    'Preprocessed', 'subject.mat'), 'subject');
                gammasources = obj.subject.gammasources;
                ESIsave(fullfile(obj.cfg.paths.data, obj.subject.subj_nr,...
                    'Preprocessed', 'gammasources.mat'), 'gammasources');
            end
        end
        
        function redoDICSplot(obj) % replot the DICS results. Useful if
            % the DICS analysis has been run on slurm, which has no proper
            % openGL implementation and therefore messes up the plots.
            
            disp('Loading DICS result from subject struct.');
            ESIload(fullfile(obj.cfg.paths.data, obj.subject.subj_nr,...
                'Preprocessed', 'subject.mat'));
            assert(isfield(subject, 'gammasources'), ...
                'No DICS results found.');
            
            stimtypes = {'color', 'grating'};
            for iStimtype = 1:length(stimtypes)
                switch stimtypes{iStimtype}
                    case 'color'
                        peaks = obj.subject.color_gammapeak_sensors;
                    case 'grating'
                        peaks = obj.subject.grating_gammapeak_sensors;
                end
                for iPeak = 1:length(peaks)
                    source_diff = subject.gammasources{iStimtype, iPeak};
                    switch obj.cfg.general.HCP_or_grid_sources
                        case 'HCP'
                            % plot dics results
                            template = ft_read_headshape(...
                                obj.cfg.beam.hcp.plot_template);
                            atlas = ft_read_cifti(obj.cfg.beam.hcp.atlas, ...
                                'mapname', 'array', 'readdata', true);
                            template.labels = label_HCP_areas([], atlas, ...
                                'V1', 'V2', 'V3', 'V4');
                            
                            figure('units', 'centimeters','position', ...
                                [1 1 20 9], ...
                                'Visible', 'on', 'Color', 'w');
                            subplot('position', [.015, .00, .4, 1.0],...
                                'Units', 'normalized');
                            HCP_source_plot([], template, ...
                                source_diff.avg.pow);
                            view([40, -5]);
                            set_colormap('seq');
                            camlight('headlight')
                            subplot('position', [.45, .00, .4, 1.0],...
                                'Units', 'normalized');
                            HCP_source_plot([], template, ...
                                source_diff.avg.pow);
                            view([-40, -5]);
                            set_colormap('seq');
                            camlight('headlight')
                            hbar = colorbar('manual', 'Position', ...
                                [0.89 0.2 0.03 0.6]);
                            ylabel(hbar, ['Induced gamma power change (times)']);
                            sgtitle(stimtypes{iStimtype})

                            filepath = [obj.subject.saveplots.HCPdics '_' ...
                                stimtypes{iStimtype} '_' ...
                                num2str(round(peaks(iPeak)))];
                            export_fig(filepath, '-png', '-nocrop', ...
                                '-q101', '-r300')
                            
                        case 'grid'
                            
                            source_plot = source_diff;
                            % interpolate sources on MNI-normalised MRI (step 5)
                            source_plot.pos = obj.mni_coordinates;
                            
                            norm_mri = ft_volumenormalise([], obj.mri);
                            norm_mri.unit = 'cm';
                            source_plot_interp = ft_sourceinterpolate(...
                                obj.cfg.beam.int, ...
                                source_plot , norm_mri);
                            
                            % plot dics results
                            atlas = ft_read_atlas(obj.cfg.beam.mask.atlasname);
                            conf               = [];
                            conf.method        = 'ortho';%obj.cfg.beam.plot.method;
                            conf.funparameter  = 'pow';
                            conf.maskparameter = conf.funparameter;
                            conf.atlas         = atlas;
                            conf.visible       = 'on';
                            conf.opacitylim    = 'zeromax';
                            conf.maskstyle     = 'colormix';
                            conf.slicedim = 2;
                            ft_sourceplot(conf, source_plot_interp);
                            
                            filepath = [obj.subject.saveplots.dics '_' ...
                                stimtypes{iStimtype} '_' ...
                                num2str(round(peaks(iPeak)))];
                            export_fig(filepath, ...
                                '-png', '-nocrop', '-q101', '-r600')
                            
                            
                            obj.subject.mni_pos = obj.mni_coordinates;
                            obj.subject.subject_pos = obj.subject_coordinates;
                    end
                end
            end
        end
        
        function computeLCMV(obj) % compute LCMV filters for each dipole
            conf = [];
            conf.toilim = obj.cfg.beam.lcmv.toilim;
            timelock = ft_redefinetrial(conf, obj.data);
            
            conf = [];
            conf.channel = 'meg';
            conf.covariance = 'yes';
            conf.latency = 'maxperiod';
            timelock = ft_timelockanalysis(conf, timelock);
            
            conf = [];
            conf.method = 'lcmv';
            conf.headmodel = obj.headmodel;
            conf.sourcemodel = obj.grid;
            conf.lcmv.keepfilter = 'yes';
            conf.lcmv.feedback = 'text';
            conf.lcmv.fixedori = 'yes';
            conf.lcmv.lambda = obj.cfg.beam.lambda;
            LCMVout = ft_sourceanalysis(conf, timelock);
            
            obj.filters = LCMVout.avg.filter;
            
            if strcmp(obj.cfg.general.HCP_or_grid_sources, 'grid')
                obj.subject.mni_pos = obj.mni_coordinates;
            end
            
            if obj.cfg.general.save_partial_results
                subject = obj.subject;
                disp('Saving MNI pos to subject struct.');
                ESIsave(fullfile(obj.cfg.paths.data, obj.subject.subj_nr,...
                    'Preprocessed', 'subject.mat'), 'subject');
                
                disp('Saving LCMV filters to file.');
                filters = obj.filters;
                ESIsave(fullfile(obj.cfg.paths.data, obj.subject.subj_nr,...
                    'Preprocessed', [obj.cfg.general.HCP_or_grid_sources ...
                    '_LCMV_filters.mat']), 'filters');
            end
        end
        
        function localizeERF(obj, timerangeInSec)
            % apply the LCMV general filters computed by .computeLCMV()
            % (needs to have been run first) to the stimulus baseline and
            % the ERF during the specified timerange relative to stimulus
            % onset. For the N70, set timerangeInSec to [0.07 0.09].
            conf = [];
            conf.toilim = obj.cfg.beam.lcmv.toilim;
            timelock = ft_redefinetrial(conf, obj.data);
            
            stimtypes = {'color', 'grating'};
            for iStimtype = 1:length(stimtypes)
                
                iTrials = obj.subject.behavior.trialNr(...
                    obj.subject.behavior.block < 3 & ... % exclude contrast
                    ismember(obj.subject.behavior.stimType, ...
                    stimtypes{iStimtype}));
                
                conf = [];
                conf.preproc.baselinewindow = [-1 0-1/1200];
                conf.preproc.demean = 'yes';
                conf.covariance = 'yes';
                conf.trials = ismember(obj.data.trialinfo(:,1), iTrials);
                conf.channel = 'meg';
                conf.covariance = 'yes';
                conf.latency = 'maxperiod';
                stimLock = ft_timelockanalysis(conf, timelock);
                
                conf = [];
                conf.method = 'lcmv';
                conf.headmodel = obj.headmodel;
                conf.sourcemodel = obj.grid;
                conf.sourcemodel.filter = obj.filters;
                conf.lcmv.keepfilter = 'yes';
                conf.lcmv.feedback = 'text';
                conf.lcmv.fixedori = 'yes';
                conf.lcmv.lambda = obj.cfg.beam.lambda;
                LCMVout = ft_sourceanalysis(conf, stimLock);
                
                conf = [];
                conf.operation = 'abs';
                conf.parameter = 'mom';
                LCMVout = ft_math(conf, LCMVout);
                
                conf = [];
                conf.latency = timerangeInSec;
                conf.avgovertime = 'yes';
                LCMVout = ft_selectdata(conf, LCMVout);
                
                ERFsources{iStimtype} = cat(1, LCMVout.mom{:});
            end
                
            
            ESIsave(fullfile(obj.cfg.paths.data, obj.subject.subj_nr,...
                'Preprocessed', 'ERFsources.mat'), 'ERFsources');
            fprintf('ERF sources for subject %s saved.\n', ...
                obj.subject.subj_nr)
            
        end
        
        function alignHCP(obj) % create a dipole grid according to the HCP atlas
            fprintf('Loading data for subject %s.\n', obj.subject.subj_nr);
            has_transforms = isfile(fullfile(obj.subject.preproc_data, ...
                'transform_acpc2ctf.mat'));
            
            obj.mri = ft_read_mri(obj.subject.mridata);
            % Realign to ACPC
            if has_transforms && ~obj.cfg.beam.hcp.redo_realign
                % apply existing transformation matrix
                disp('Using existing volume transformation matrices.');
                mri_acpc = obj.mri;
                ESIload(fullfile(obj.subject.preproc_data, ...
                    'transform_orig2vox'));
                mri_acpc.transform = transform_orig2vox;
                mri_acpc.coordsys = 'acpc';
            else % realign to ACPC manually
                conf = [];
                conf.method = 'interactive';
                conf.coordsys = 'acpc';
                mri_acpc = ft_volumerealign(conf, obj.mri);
                transform_orig2vox = mri_acpc.transform;
                ESIsave(fullfile(obj.subject.preproc_data, ...
                    'transform_orig2vox'), 'transform_orig2vox');
            end
            % Reslice
            conf = [];
            conf.resolution = 1;
            conf.dim = [256 256 256];
            mri_acpc_rs = ft_volumereslice(conf, mri_acpc);
            transform_vox2acpc = mri_acpc_rs.transform;
            ESIsave(fullfile(obj.subject.preproc_data, ...
                'transform_vox2acpc'), 'transform_vox2acpc');
            % Write MRI to disk
            conf = [];
            conf.filename    = fullfile(obj.cfg.paths.data, 'Freesurfed', ...
                [obj.subject.subj_nr '_T1']);
            conf.filetype    = 'mgz';
            conf.parameter   = 'anatomy';
            ft_volumewrite(conf, mri_acpc_rs);
            
            % Apply same transform to T2, if it exists
            if isfield(obj.subject, 'mridataT2')
                disp('Transforming T2 as T1 was transformed.')
                mriT2_acpc = ft_read_mri(obj.subject.mridataT2);
                ESIload(fullfile(obj.subject.preproc_data, ...
                    'transform_orig2vox'));
                mriT2_acpc.transform = transform_orig2vox;
                mriT2_acpc.coordsys = 'acpc';
                
                conf = [];
                conf.resolution = 1;
                conf.dim = [256 256 256];
                mriT2_acpc_rs = ft_volumereslice(conf, mriT2_acpc);
                
                conf = [];
                conf.filename    = fullfile(obj.cfg.paths.data, 'Freesurfed', ...
                    [obj.subject.subj_nr '_T2']);
                conf.filetype    = 'mgz';
                conf.parameter   = 'anatomy';
                ft_volumewrite(conf, mriT2_acpc_rs);
            end
            
            % Create and save transformation matrix to CTF and to ACPC
            if ~has_transforms || obj.cfg.beam.hcp.redo_realign
                conf          = [];
                conf.method   = 'interactive';
                conf.coordsys = 'ctf';
                mri_ctf_rs   = ft_volumerealign(conf, mri_acpc_rs);
                transform_vox2ctf = mri_ctf_rs.transform;
                ESIsave(fullfile(obj.subject.preproc_data, ...
                    'transform_vox2ctf'), 'transform_vox2ctf');
                transform_acpc2ctf = transform_vox2ctf/transform_vox2acpc;
                ESIsave(fullfile(obj.subject.preproc_data, ...
                    'transform_acpc2ctf'), 'transform_acpc2ctf');
            else
                mri_ctf_rs = mri_acpc_rs;
                ESIload(fullfile(obj.subject.preproc_data, ...
                    'transform_vox2ctf'));
                mri_ctf_rs.transform = transform_vox2ctf;
            end
        end
        
        function out = findFreqbandDICS(obj, sWhich)
            % Run DICS for any frequency band (sWhich)
            % which can be 'theta', 'alpha' or 'beta', 'grating_gamma' or
            % 'color_gamma.
            
            peakname_sensors = [sWhich 'peak_sensors'];
            peakname_source = [sWhich 'peak_source'];
            if isfield(obj.subject, peakname_source)
                disp(['Using source-localised ' sWhich ' peak.'])
                peak = round(obj.subject.(peakname_source));
            elseif isfield(obj.subject, peakname_sensors)
                disp(['Using sensor-level ' sWhich ' peak.'])
                peak = round(obj.subject.(peakname_sensors));
            else
                error([sWhich ' peak not found in subject struct!']);
            end
            
            %% DICS beamforming
            % redefine trials into baseline and stimulus (step 1)
            % do frequency analysis (step 2)
            
            % settings for data snippets
            conf = [];
            conf.base.toilim = [-1 0-1/obj.subject.meg_info.samplingMEG];
            conf.stim.toilim = [0.3 1.3-1/obj.subject.meg_info.samplingMEG];
            conf.channel = 'meg';
            switch sWhich
                case {'color_gamma', 'grating_gamma'}
                    conf.base.minlength = 0.5;
                    conf.stim.minlength = 0.5;
                    conf.whole.minlength = 1;
                case {'theta', 'alpha', 'beta'}
                    conf.base.minlength = 1;
                    conf.stim.minlength = 1;
                    conf.whole.minlength = 1;
                otherwise
                    error('Invalid sWhich!')
            end
            dat_base  = ft_redefinetrial(conf.base, obj.data);
            dat_stim  = ft_redefinetrial(conf.stim, obj.data);
            dat_whole  = ft_redefinetrial(conf.whole, obj.data);
            
            % settings for FFT
            conf = [];
            conf.channel            = 'meg';
            conf.method             = 'mtmfft';
            conf.output             = 'powandcsd';
            conf.foi                = peak;
            conf.pad                = 4;
            switch sWhich
                case {'color_gamma', 'grating_gamma'}
                    conf.taper = 'dpss';
                    conf.tapsmofrq = 4;
                case {'theta', 'alpha', 'beta'}
                    conf.taper = 'hanning';
            end
            
            freq_whole = ft_freqanalysis(conf, dat_whole);
            clear('dat_whole')
            freq_base = ft_freqanalysis(conf, dat_base);
            clear('dat_base')
            freq_stim = ft_freqanalysis(conf, dat_stim);
            clear('dat_stim')
            
            % run sourceanalysis on whole trial (step 3)
            conf = [];
            conf.sourcemodel  = obj.grid;
            conf.headmodel    = obj.headmodel;
            conf.frequency    = round(peak);
            conf.dics.realfilter = 'yes';
            conf.dics.keepfilter = 'yes';
            conf.dics.fixedori = 'yes';
            conf.dics.projectnoise = 'no';
            conf.dics.lambda = '5%';
            source.whole = ft_sourceanalysis(conf, freq_whole);
            
            % run sourceanalysis on baseline and stimulus presentation (step 4)
            conf.sourcemodel.filter = source.whole.avg.filter;
            source.base = ft_sourceanalysis(conf, freq_base);
            source.stim = ft_sourceanalysis(conf, freq_stim);
            
            switch sWhich
                case 'theta' % return NAI for the stimulation period
                    out = source.stim.avg.pow ./ source.base.avg.pow;
                case 'alpha' % return NAI for the baseline
                    out = source.stim.avg.pow ./ source.base.avg.pow;
                case 'beta' % return NAI for the baseline
                    out = source.stim.avg.pow ./ source.base.avg.pow;
                case {'color_gamma', 'grating_gamma'} % return base -> stim change
                    out = source.stim.avg.pow ./ source.base.avg.pow;
            end
            
            if obj.cfg.general.save_partial_results
                disp('Saving DICS filters to file.');
                filters = source.whole.avg.filter;
                ESIsave(fullfile(obj.cfg.paths.data, obj.subject.subj_nr,...
                    'Preprocessed', [obj.cfg.general.HCP_or_grid_sources ...
                    '_' sWhich '_DICS_filters.mat']), 'filters');
            end
            
            % Do per-subject plot
            source.stim.avg.pow = out;
            subjectDICSPlot(obj, source);
            export_fig([obj.subject.saveplots.dics 'color'], '-png', '-nocrop', '-q101', '-r200')
            export_fig([obj.subject.saveplots.dics 'grating'], '-png', '-nocrop', '-q101', '-r200')
            
        end
        
    end
end

