classdef ERFs < handle
    % ERFs Class to compute source timecourses, run ERF analyses on
    % a per-subject basis.
    %
    % ERFs Properties:
    %   cfg                 = A config struct as created by fillConfigs().
    %   subject             = A subject struct with subject-specific info and
    %                         already computed analysis results (eg beamformer
    %                         filters, gamma peaks, etc.).
    %   data                = The source-space dipole timecourses computed
    %                         from .data_sensors and .filters.
    %   data_sensors        = Cleaned, preprocessed, sensorlevel MEG data.
    %   filters             = LCMV sensors->source filter matrix for each
    %                         dipole.
    %
    % ERFs Methods:
    %   ERFs(cfg, subj_nr) = Class constructor.
    %   loadData()            = Load MEG data from disk.
    %   computeSources(area)  = Compute source timepoints for a specific
    %                           area (default = V1).
    
    properties
        cfg
        subject
        data
        data_sensors
        filters
        direction % estimated per-dipole direction relative to the generator
    end
    
    methods
        function obj = ERFs(subj_nr)
            % Constructor
            obj.cfg = setup(subj_nr);
            ESIload(fullfile(obj.cfg.paths.data, num2str(subj_nr, '%03d'), ...
                'Preprocessed', 'subject.mat'));
            obj.subject = subject;
        end
        
        function loadData(obj) % Load in raw MEG data and LCMV filters.
            fprintf('Loading raw MEG data for subject %s.\n', ...
                obj.subject.subj_nr);
            ESIload(fullfile(obj.subject.preproc_data, 'data_cleaned'));
            % The grad struct will not apply here, as we plan to project
            % the data into source space.
            data_cleaned = rmfield(data_cleaned, 'grad');
            conf = [];
            conf.channel = 'meg';
            obj.data_sensors = ft_selectdata(conf, data_cleaned);
            clear data_cleaned
            
            disp('Loading LCMV filters.')
            ESIload(fullfile(obj.cfg.paths.data, obj.subject.subj_nr,...
                'Preprocessed', [obj.cfg.general.HCP_or_grid_sources ...
                '_LCMV_filters.mat']));
            obj.filters = filters;
        end
        
        function lpfilterSensorData(obj)
            disp('Low-pass filtering.')
            conf = [];
            conf.lpfreq = 80;
            conf.method = 'gaussian';
            obj.data_sensors = ESI_lowpassfilter(conf, obj.data_sensors);
        end
        
        function computeAreaSources(obj, whichAreas) % Uses LCMV filters to get
            %  dipoles in pre-specified atlas labels.
            fprintf('Applying %s filters to subject %s.\n', whichAreas,...
                obj.subject.subj_nr);
            atlas = ft_read_cifti(obj.cfg.beam.hcp.atlas, ...
                'mapname', 'array', 'readdata', true);
            
            %subsampled = {};
            %[subsampled{1}, subsampled{2}] = reduce_HCP_surf(obj, 1/8);
            %subsampled = cat(1, subsampled{:});
            indices = select_HCP_parcel(whichAreas, atlas, 'both');
            %indices = intersect(indices, subsampled);
            
            areaFilters = cat(1, obj.filters{indices});
            obj.data = apply_filters(obj.data_sensors, ...
                areaFilters, indices);
        end
        
        function out = binnedERFs(obj)
            % Just compute and return binned (stimulus-wise)
            % ERFs, nothing else. Requires computeSources to have been run.
            disp('Computing average ERFs.')
            
            disp('Selecting full trials.')
            conf = [];
            conf.minlength = 'maxperlen';
            conf.toilim = [-0.2 0.6-1/1200];
            ERFdata = ft_redefinetrial(conf, obj.data);
            
            [stims, stimtypes] = getColoredStimtypes(obj.subject.behavior);
            
            % find ERFs of each stimtype
            disp('Computing stimulus ERFs.');
            for iStimtype = 1:length(stimtypes)
                select = find(strcmp(stims, stimtypes{iStimtype}));
                iTrials = ismember(ERFdata.trialinfo(:,1), select);
                
                % baseline on a per-trial level (important for SD
                % calculation for the z-score)
                conf = [];
                conf.trials = iTrials;
                conf.keeptrials = 'yes';
                timelock = ft_timelockanalysis(conf, ERFdata);
                
                base = timelock.time(timelock.time<0);
                conf = [];
                conf.baseline = [min(base), max(base)];
                conf.method = 'zscore';
                timelock = ESI_ft_timelockbaseline(conf, timelock);
                
                % then average over trials
                conf = [];
                timelock = ft_timelockanalysis(conf, timelock);
                
                disp('If necessary, flipping positive/negative.');
                conf = [];
                conf.latency = [0.055 0.095];
                iDirection = ft_selectdata(conf, timelock);
                direction(iStimtype,:) = sign(mean(iDirection.avg, 2));
                timelock.avg = timelock.avg .* -direction(iStimtype,:)';
                
                disp('Averaging dipoles.')
                conf = [];
                conf.avgoverchan = 'yes';
                timelock = ft_selectdata(conf, timelock);
                
                erf(:,iStimtype) = timelock.avg;
            end
            obj.direction = direction; % record per-dipole direction estimate
            time = timelock.time;
            
            out.erf = erf;
            out.time = time;
            out.stimtypes = stimtypes;
        end
        
        function makeERFtable(obj) % get a table of per-trial ERF sizes
            if isempty(obj.direction)
                obj.binnedERFs();
            end
            
            disp('Selecting full trials.')
            conf = [];
            conf.minlength = 'maxperlen';
            conf.toilim = [-0.2 0.6-1/1200];
            ERFdata = ft_redefinetrial(conf, obj.data);
            
            [stims, stimtypes] = getColoredStimtypes(obj.subject.behavior);
            
            C1size = table(obj.subject.behavior.trialNr, ...
                NaN(size(obj.subject.behavior.trialNr)), ...
                'VariableNames', {'trialNr', 'C1'});
            % find ERFs of each stimtype
            disp('Computing per-trial ERF.');
            for iStimtype = 1:length(stimtypes)
                select = find(strcmp(stims, stimtypes{iStimtype}));
                iTrials = ismember(ERFdata.trialinfo(:,1), select);
                
                conf = [];
                conf.trials = iTrials;
                conf.keeptrials = 'yes';
                timelock = ft_timelockanalysis(conf, ERFdata);
                
                base = timelock.time(timelock.time<0);
                conf = [];
                conf.baseline = [min(base), max(base)];
                conf.method = 'zscore';
                timelock = ESI_ft_timelockbaseline(conf, timelock);
                
                disp('If necessary, flipping positive/negative.');
                for iTrial = 1:size(timelock.trial,1)
                    timelock.trial(iTrial,:,:) = ...
                        squeeze(timelock.trial(iTrial,:,:)) .* ...
                        -obj.direction(iStimtype,:)';
                end
                
                disp('Averaging dipoles.')
                conf = [];
                conf.avgoverchan = 'yes';
                timelock = ft_selectdata(conf, timelock);
                
                disp('Getting per-trial C1 size.')
                conf = [];
                conf.latency = obj.cfg.general.ERFcomps;
                conf.avgovertime = 'yes';
                timelock = ft_selectdata(conf, timelock);
                
                for iTrial = 1:size(timelock.trialinfo,1)
                    % Put values into the trial table
                    iTrialNr = timelock.trialinfo(iTrial);
                    C1size.C1(iTrialNr) = timelock.trial(iTrial);
                end
            end
            
            outputfile = fullfile(obj.cfg.paths.data,...
                obj.subject.subj_nr, 'Preprocessed', 'C1size.csv');
            writetable(C1size, outputfile);
            
        end
        
        
    end
end

