classdef Eyedata < handle
    %Eye A class to analyze the eyetracking data.
    %   Detects (micro-)saccades using the Engbert&Kliegel algorithm, runs
    %   some sanity checks on them (+ plots) and adds their timing to the
    %   subject struct. Can also mark trials for rejection if participants
    %   left a (post-hoc) fixation window. Also fits slopes to the pupil
    %   dilation data on a trial-per-trial basis.
    
    properties
        cfg
        subject
        data_eyemov
        data_pupil
        saccades
    end
    
    methods
        function obj = Eyedata(subj_nr) % Class constructor
            obj.cfg = setup(subj_nr);
            % Load subject struct + data
            ESIload(fullfile(obj.cfg.paths.data, num2str(subj_nr, '%03d'), ...
                'Preprocessed', 'subject.mat'));
            obj.subject = subject;
        end      
        
        function doEverything(obj) % Run the whole eyedata pipeline
            obj.loadData();
            obj.saccadeDetectEK();
            obj.pupilSizeAnalysis();
            obj.qcPlots();
        end
           
        function loadData(obj)
            ESIload(fullfile(obj.subject.preproc_data, 'data_cleaned'));
            conf = [];
            conf.channel = obj.cfg.channels.eyemovement;
            obj.data_eyemov = ft_selectdata(conf, data_cleaned);
            conf.channel = obj.cfg.channels.pupilsize;
            obj.data_pupil = ft_selectdata(conf, data_cleaned);
            
            % Choose left eye if both were recorded.
            if length(obj.data_eyemov.label) >2
                disp('Only using left eye');
                conf.channel = obj.cfg.channels.eyemovementL;
                obj.data_eyemov = ft_selectdata(conf, data_cleaned);
            end
            
            % Fix inverted Y direction of the eyetracker.
            disp('Reversing Y-axis of eyemovements.');
            obj.data_eyemov.trial = cellfun(@(x) [x(1,:); x(2,:).*-1], ...
                obj.data_eyemov.trial, 'UniformOutput', false);
            
            % Transform voltage values of the eye position to degrees from
            % fixation.
            disp('Transforming eyetrace from voltage to degrees.');
            disp(['Assuming that eye x-position is on row 1 '... 
                  'and y-position on row 2.']);
            disp('Assuming fixation at screen middle.');
            obj.data_eyemov.trial = cellfun(@(x) ...
                voltToDegrees(obj.cfg.eyetracker, x), obj.data_eyemov.trial,...
                'UniformOutput', false);
            
        end
        
        function saccadeDetectEK(obj)
            % Detect saccades with the Engbert and Kliegel algorithm.
            % Requires ESI saccadedetection functions.
            disp('Detecting saccades.');
            conf = [];
            conf.debugplot = obj.cfg.ms.debugplot;
            conf.velThresh = obj.cfg.ms.velThresh;
            conf.MinDur = obj.cfg.ms.MinDur;
            conf.amplMax = obj.cfg.ms.amplMax;
            conf.speedAlgo = obj.cfg.ms.speedAlgo;
            conf.toilim = obj.cfg.ms.toilim;
            obj.saccades = bhv_microsaccade_detection(conf, obj.data_eyemov);
            
            if obj.cfg.general.save_partial_results
                [sac_per_trial, trialNr] = sumTrialSnippets(...
                    obj.saccades.nSac, obj.saccades.trialinfo');
                saccaderate = table(trialNr, sac_per_trial);
                
                outputfile = fullfile(obj.cfg.paths.data,...
                    obj.subject.subj_nr, 'Preprocessed', 'saccades.csv');
                writetable(saccaderate, outputfile);
            end
        end
        
        function out = pupilSizeAnalysis(obj)
            % Select only trials with complete data, remove ones with
            % blinks / muscle artefacts
            oneSample = 1/obj.subject.meg_info{1}.samplingMEG;
            
            conf = [];
            conf.toilim = [-1 1.3-oneSample];
            conf.minlength = 'maxperlen';
            dat_pupil = ft_redefinetrial(conf, obj.data_pupil);
            
            % Compute mean and std dev for pupil size, rune baseline
            % correction.
            conf = []; 
            conf.keeptrials = 'yes';
            timelocked = ft_timelockanalysis(conf, dat_pupil);
            conf = [];
            conf.baseline = [-0.1 0-oneSample];
            conf.parameter = 'trial';
            timelocked = ft_timelockbaseline(conf, timelocked);
            meanPupil = squeeze(ESI_nanmean(timelocked.trial, 2));
            avgpupil = squeeze(ESI_nanmean(meanPupil, 1));
            bounds = ESI_nanstd(meanPupil, [] ,1);
            
            % average pupil traces over stimuli
                        
            % Get different stimulus types
            [stimtype, stimtypes] = getColoredStimtypes(...
                obj.subject.behavior);
            for iStim = 1:length(stimtypes)
                iTrials = find(ismember(stimtype, stimtypes{iStim}));
                iTrials = ismember(dat_pupil.trialinfo, iTrials);
                
                conf = [];
                conf.trials = iTrials;
                conf.avgoverrpt = 'yes';
                conf.avgoverchan = 'yes';
                iMeanPupil = ft_selectdata(conf, timelocked);
                stimAvgPupil(iStim,:) = iMeanPupil.trial;
                time = iMeanPupil.time;
            end
            out.avgPupil = stimAvgPupil;
            out.time = time;            
            out.stimtypes = stimtypes;
            
            % Plot pupil size over all trials
            pupil_plot = figure('units', 'centimeters','position', ...
                [1 1 15 10], 'Color', 'w');
            plot(timelocked.time, meanPupil', 'Color', [0 0 0 0.05]);
            hold on
            boundedline(timelocked.time, avgpupil, bounds, 'alpha'); 
            title('Pupil size over time in trial');
            xlabel('Time (s)');
            ylabel('Pupil size, baseline corrected');
            % Save it into dedicated folder
            subj = num2str(obj.subject.subj_nr, '%03d');
            outputfile = fullfile(obj.cfg.paths.plot, 'QC_Pupil', subj);
            disp(['Saving QC_Pupil plot for subject ' subj '.']);
            export_fig(pupil_plot, outputfile, '-png', '-q101', '-r500');
            
            % Extract time means
            prestim_pupil = timelocked.time >= -0.1 & timelocked.time <0;
            prestim_pupil = squeeze(ESI_nanmean(meanPupil(:, prestim_pupil),2));
            
            poststim_pupil = timelocked.time >= 0.5 & timelocked.time <0.75;
            poststim_pupil = squeeze(ESI_nanmean(meanPupil(:, poststim_pupil),2)); 
            
            if obj.cfg.general.save_partial_results
                pupilsize = table(...,
                    timelocked.trialinfo, prestim_pupil, poststim_pupil,...
                    'VariableNames', ...
                    {'trialNr', 'prestim_pupil', 'poststim_pupil'});
                
                outputfile = fullfile(obj.cfg.paths.data,...
                    obj.subject.subj_nr, 'Preprocessed', 'pupilsize.csv');
                writetable(pupilsize, outputfile);
            end
        end
        
        function qcPlots(obj)
            % Takes computed saccade data, plots if (micro)saccades are
            % over-distributed in a specific direction, if the saccade
            % detection worked (as can be approximated from the main
            % sequence), and if saccade direction or probability changes
            % during the trial.
            
            % Get different stimulus types
            [stimtype, stimtypes] = getColoredStimtypes(...
                obj.subject.behavior);

            % Create big plot over all stimulus types
            nRows = 3;
            nCols = length(stimtypes);
            qc_plot = figure('units', 'centimeters','position', ...
                [1 1 12*nCols, 8*nRows], 'Visible', 'off');
            set(gcf,'color','w');
            
            for iStimtype = 1:length(stimtypes)
                % Find trials or trial snippets of this stim type - trial number might
                % not be the same as trial number during presentation if trials with
                % artifacts have been partially rejected.
                thisStimtype = stimtypes{iStimtype};
                stim_trials = find(strcmp(stimtype, thisStimtype));
                stim_trialparts = obj.saccades.trialinfo(...
                    ismember(obj.saccades.trialinfo, stim_trials));
                [~,~,iStim_trials] = intersect(stim_trialparts, ...
                    obj.saccades.trialinfo);
                
                % Select only saccades in this block
                iSaccades = vertcat(obj.saccades.micrDetails{iStim_trials});
                
                % Saccade plot
                subplot(nRows, nCols, iStimtype, ...
                    polaraxes);
                %stimTheta = deg2rad(sumDegrees(360-orientations{iStimtype}, 90));
                %stimTheta2 = deg2rad(sumDegrees(360-orientations{iStimtype}, 270));
                %polarplot([stimTheta, stimTheta2], [2 2], 'r');
                hold on
                for k = 1:length(iSaccades(:,6))
                    [theta, rho] = cart2pol(iSaccades(k,6), iSaccades(k,7));
                    polarplot([0 theta], [0 rho], 'b');
                end
                hold off
                title(sprintf('Block %i', iStimtype));
                
                % Saccade direction histogram.
                % Fix saccade angle ( is in -180 to 180, should be in 0 to
                % 360).
                saccadeAngle = iSaccades(:,5);
                saccadeAngle(saccadeAngle<0) = 360 + saccadeAngle(saccadeAngle<0);
                allAxs{iStimtype} = subplot(nRows, nCols, ....
                    iStimtype+length(stimtypes), polaraxes);
                AllHists{iStimtype} = CircHist(saccadeAngle, 20, 'ax', ...
                    allAxs{iStimtype});
                histoax = gca;
                histoax.ThetaDir = 'counterclockwise';
                histoax.ThetaZeroLocation = 'right';
            end
            
            % Align the Histogram axes.
            minRho = min(cellfun(@(x) min(rlim(x)), allAxs));
            maxRho = max(cellfun(@(x) max(rlim(x)), allAxs));
            cellfun(@(x) setRLim(x, [minRho maxRho]), AllHists);
            
            % Main sequence plot
            subplot(nRows, nCols, [iStimtype*2+1 iStimtype*2+2]);
            ms = vertcat(obj.saccades.micrDetails{:});
            corrMS = corr(ms(:,3), ms(:,4));
            corrMSlog = corr(log(ms(:,3)), log(ms(:,4)));
            scatter(ms(:,3), ms(:,4));
            set(gca,'xscale','log');
            set(gca,'yscale','log')
            title(sprintf('Main sequence. Corr_{log-log} = %1.2f, Corr_{lin} = %1.2f',...
                corrMSlog, corrMS));
            xlabel('Peak velocity (log)');
            ylabel('Amplitude (log)');
            
            % Microsaccades per trial over time plot
            subplot(nRows, nCols, [iStimtype*2+3 iStimtype*2+4]);
            % sum microsaccades over trial snippets
            [sac_per_trial, trial_nr] = sumTrialSnippets(...
                obj.saccades.sacRateTrl, obj.saccades.trialinfo');
            stim_nr = obj.subject.behavior.stimNr(trial_nr);
            
            nBlocks = max(obj.subject.behavior.block);
            
            blocks = zeros(nBlocks, 2);
            behav = obj.subject.behavior;
            
            for iBlock = 1:nBlocks
            	blocks(iBlock, 1) = min(...
                	behav.trialNr(behav.block == iBlock));
            	blocks(iBlock, 2) = max(...
                	behav.trialNr(behav.block == iBlock));
            end
            smoothedLinePlot(trial_nr, sac_per_trial, ...
                'fullX', 1:max(behav.trialNr), ...
                'blocks', blocks, 'span', obj.cfg.general.plotTrialBinsize, ...
                'legendloc', 'eastoutside', 'legendori', 'vertical');
            title(sprintf(['Microsaccades over time - '...
                'Corr_{stimNr - saccade rate} = %1.2f'],...
                corr(stim_nr, sac_per_trial)));
            xlabel('Trial');
            ylabel('Saccades per trial');
                        
            % Save it into dedicated folder.
            subj = num2str(obj.subject.subj_nr, '%03d');
            outputfile = fullfile(obj.cfg.paths.plot, 'QC_Saccades', subj);
            disp(['Saving QC_Saccades plot for subject ' subj '.']);
            export_fig(qc_plot, outputfile, '-png', '-q101', '-r300');
            
        end
    end
    
end

