classdef Freqs < handle
    % Freqs Class to compute source timecourses, run ERF analyses on
    % a per-subject basis.
    %
    % Freqs Properties:
    %   cfg                 = A config struct as created by fillConfigs().
    %   subject             = A subject struct with subject-specific info and
    %                         already computed analysis results (eg beamformer
    %                         filters, gamma peaks, etc.).
    %   data                = The source-space dipole timecourses computed
    %                         from .data_sensors and .filters.
    %   data_sensors        = Cleaned, preprocessed, sensorlevel MEG data.
    %   filters             = LCMV sensors->source filter matrix for each
    %                         dipole.
    %
    % Freqs Methods:
    %   Freqs(cfg, subj_nr) = Class constructor.
    %   loadData()            = Load MEG data from disk.
    %   computeSources(area)  = Compute source timepoints for a specific
    %                           area (default = V1).
    
    properties
        cfg
        subject
        data
        data_sensors
        filters
        TFR = [];
        spect = [];
        stimType = [];
        peaks = [];
    end
    
    methods
        function obj = Freqs(subj_nr)
            % Constructor
            obj.cfg = setup(subj_nr);
            ESIload(fullfile(obj.cfg.paths.data, num2str(subj_nr, '%03d'), ...
                'Preprocessed', 'subject.mat'));
            obj.subject = subject;
        end
        
        function runSpectraPipeline(obj, whichAreas, subsample)
            % whichAreas is an area name according to the HCP atlas, subsample
            % is the amount of subsampling for dipoles we want. 0.05 = 5%.
            if ~exist('whichAreas', 'var') || isempty(whichAreas)
                whichAreas = 'V1';
            end
            if ~exist('subsample', 'var') || isempty(subsample)
                subsample = obj.cfg.general.subsampleV1;
            end
            obj.loadData();
            obj.computeAreaSources(whichAreas, subsample);
            obj.quantifySpectra();
            obj.makeTrialTable();
        end
        
        function loadData(obj) % Load in raw MEG data and LCMV filters.
            fprintf('Loading raw MEG data for subject %s.\n', ...
                obj.subject.subj_nr);
            ESIload(fullfile(obj.cfg.paths.data, obj.subject.subj_nr, ...
                'Preprocessed', 'data_cleaned.mat'));
            % The grad struct will not apply here, as we plan to project
            % the data into source space.
            data_cleaned = rmfield(data_cleaned, 'grad');
            conf = [];
            conf.channel = 'meg';
            obj.data_sensors = ft_selectdata(conf, data_cleaned);
            clear data_cleaned
            
            disp('Loading LCMV filters.')
            ESIload(fullfile(obj.cfg.paths.data, obj.subject.subj_nr,...
                'Preprocessed', [obj.cfg.general.HCP_or_grid_sources ...
                '_LCMV_filters.mat']));
            obj.filters = filters;
            
        end
        
        function datOut = computeAreaSources(obj, whichAreas, subsample)
            % Uses LCMV filters to get dipoles in pre-specified atlas
            % label areas (whichAreas).
            % subsample determines the number of
            % dipoles: If set to 1, all dipoles in the reqeusted area are
            % used. If set to <1, that amount of dipoles (i.e. 0.2 = 20% of
            % dipoles) are subsampled from the bigger set and used.
            % Subsampling can be needed to decrease memory usage.
            if ~exist('whichAreas', 'var') || isempty(whichAreas)
                whichAreas = 'V1';
            end
            if ~exist('subsample', 'var') || isempty(subsample)
                subsample = obj.cfg.general.subsampleV1;
            end
            assert(subsample > 0 && subsample <= 1, ...
                'Invalid subsample amount. Must be 0 < subsample <= 1.');
            
            fprintf('Applying %s filters to subject %s.\n', whichAreas,...
                obj.subject.subj_nr);
            atlas = ft_read_cifti(obj.cfg.beam.hcp.atlas, ...
                'mapname', 'array', 'readdata', true);
            
            indices = select_HCP_parcel(whichAreas, atlas, 'both');
            if subsample < 1
                fprintf(['Subsampling is active: '...
                    'Selecting %3i%% of all dipoles.\n'], subsample*100);
                subsampled = {};
                [subsampled{1}, subsampled{2}] = ...
                    reduce_HCP_surf(obj, subsample);
                subsampled = cat(1, subsampled{:});
                indices = intersect(indices, subsampled);
            end
            
            areaFilters = cat(1, obj.filters{indices});
            obj.data = apply_filters(obj.data_sensors, ...
                areaFilters, indices);
            datOut = obj.data;
        end
        
        function [TFR, spect, stimType] = getSpectraAndTFR(obj,justSpectra)
            % Gets spectra and TFRs of each individual trial type by 
            % calling oneSpectrumAndTFR repeatedly. If justSpectra is true, 
            % TFR computation will be omitted.
            if ~exist('justSpectra', 'var')
                justSpectra = false;
            end
            
            disp('Running FFT for each stimulus type.')
            [~, stimtypes] = getColoredStimtypes(obj.subject.behavior);
            for iStimtype = 1:length(stimtypes)
                stimType{iStimtype} = stimtypes{iStimtype};
                [TFR{iStimtype}, spect{iStimtype}] = ...
                    obj.oneSpectrumAndTFR(stimType{iStimtype},...
                    justSpectra);
            end
            disp('All stimulus types ran succesfully.')
                        
            obj.TFR = TFR;
            obj.spect = spect;
            obj.stimType = stimType;
        end
        
        function [TFR, spect_stim] = oneSpectrumAndTFR(obj, stimType,...
                justSpectra)
            % Utility function that runs FFTs on baseline, stim-period
            % and as TFRs. Only uses trials of the requested stimulus type
            % to save memory. Use getSpectraAndTFR to run this for all stim
            % types. If justSpectra is true, TFR computation will be
            % omitted.
            if ~exist('justSpectra', 'var')
                justSpectra = false;
            end
            
            fprintf('Computing spectra for %s.\n', stimType);

            stims = getColoredStimtypes(obj.subject.behavior);
            select = find(strcmp(stims, stimType));
            iTrials = ismember(obj.data.trialinfo(:,1), select);
                       
            fprintf('Selecting only %s trials.\n', stimType);
            conf = [];
            conf.trials = iTrials;
            dataIn = ft_selectdata(conf, obj.data);
            
            % remove trials that have artifacts during stimulation
            % onset to prevent trials with delayed visual response
            % onsets from getting into the averages
            fullPeriod = cellfun(@(x) ...
                min(x) <= obj.cfg.freqs.minimum_trial_time(1) & ...
                max(x) >= obj.cfg.freqs.minimum_trial_time(2), ...
                dataIn.time);
            fprintf('%i trials have enough data.\n', sum(fullPeriod));
            
            conf = [];
            conf.trials = fullPeriod;
            dataIn = ft_selectdata(conf, dataIn);
                        
            % Select trials/snippets that are long enough
            conf = [];
            conf.minlength = obj.cfg.spectra.minlength;
            conf.toilim    = [obj.cfg.spectra.all.toilim(1) - 0.25,...
                obj.cfg.spectra.all.toilim(2) + 0.25];
            dipole_all     = ft_redefinetrial(conf, dataIn);
            
            conf.toilim    = obj.cfg.spectra.base.toilim;
            dipole_base    = ft_redefinetrial(conf, dataIn);
            
            conf.toilim    = obj.cfg.spectra.stim.toilim;
            dipole_stim    = ft_redefinetrial(conf, dataIn);
            clear dataIn
            
            % recut into sliding windows for Welch - lower frequencies
            if obj.cfg.freqs.sliding_window
                conf = [];
                conf.length = obj.cfg.spectra.window_length_long;
                conf.overlap = obj.cfg.spectra.window_overlap;
                dipole_base_long = ft_redefinetrial(conf, dipole_base);
                dipole_stim_long = ft_redefinetrial(conf, dipole_stim);
            end
            
            % recut into sliding windows for Welch - higher frequencies
            if obj.cfg.freqs.sliding_window
                conf = [];
                conf.length = obj.cfg.spectra.window_length_short;
                conf.overlap = obj.cfg.spectra.window_overlap;
                dipole_base_short = ft_redefinetrial(conf, dipole_base);
                dipole_stim_short = ft_redefinetrial(conf, dipole_stim);
            end
            clear dipole_base dipole_stim
            
            % Do frequency analysis - Hann for low frequencies
            conf = [];
            conf.foilim     = obj.cfg.spectra.foi_hanning;
            nDims = 3; % for snippet merging - freqs are two dimension
            conf.method         = 'mtmfft';
            conf.taper          = 'hann';
            conf.keeptrials     = 'yes';
            conf.output         = 'pow';
            conf.pad            = 1; %maxperlen
            conf.polyremoval    = obj.cfg.spectra.polyremoval;
            
            freq_base           = ft_freqanalysis(conf, dipole_base_long);
            freq_stim           = ft_freqanalysis(conf, dipole_stim_long);
            clear dipole_base_long dipole_stim_long
            
            spect_low.base = mergeFFTSnippets(freq_base, ...
                freq_base.trialinfo(:,1), nDims);
            spect_low.stim = mergeFFTSnippets(freq_stim, ...
                freq_stim.trialinfo(:,1), nDims);
            clear freq_base freq_stim
            
            % Do frequency analysis - DPSS for high frequencies
            conf.foilim         = obj.cfg.spectra.foi_dpss;
            conf.taper          = 'dpss';
            conf.tapsmofrq      = obj.cfg.spectra.tapsmofrq;
            conf.pad            = 1;%obj.cfg.spectra.window_length_short;
            freq_base           = ft_freqanalysis(conf, dipole_base_short);
            freq_stim           = ft_freqanalysis(conf, dipole_stim_short);
            clear dipole_base_short dipole_stim_short
            
            spect_high.base = mergeFFTSnippets(freq_base, ...
                freq_base.trialinfo(:,1), nDims);
            spect_high.stim = mergeFFTSnippets(freq_stim, ...
                freq_stim.trialinfo(:,1), nDims);
            clear freq_base freq_stim

            % Merge the two frequency analyses
            spect_base = ft_appendfreq([], spect_low.base, spect_high.base);
            spect_stim = ft_appendfreq([], spect_low.stim, spect_high.stim);
            clear spect_low spect_high
            
            % compute change spectra, then average over dipoles
            disp('Baselining per dipole, then averaging over dipoles.')
            for iBlock = 1:max(obj.subject.behavior.block)
                % compute avg. baseline per block
                iTrials = obj.subject.behavior.trialNr(...
                    obj.subject.behavior.block == iBlock);
                iTrials = ismember(spect_base.trialinfo, iTrials);
                avgBase{iBlock} = squeeze(nanmean(...
                    spect_base.powspctrm(iTrials,:,:) ,1));
            end
            clear spect_base
            
            change = nan(size(spect_stim.powspctrm));
            for iTrial = 1:size(spect_stim.powspctrm, 1)
                % divide by avg. baseline
                thisBlock = obj.subject.behavior.block(...
                    obj.subject.behavior.trialNr == ...
                    spect_stim.trialinfo(iTrial));
                blockBase = avgBase{thisBlock};
                for iChan = 1:size(spect_stim.powspctrm, 2)
                    change(iTrial,iChan,:) = ...
                        squeeze(spect_stim.powspctrm(iTrial,iChan,:)) ./...
                        blockBase(iChan,:)';
                end
            end
            clear avgBase
            % insert result into old spectrum structure
            spect_stim.powspctrm = change;
            
            % average over dipoles
            conf = [];
            conf.avgoverchan = 'yes';
            spect_stim = ft_selectdata(conf, spect_stim);
                        
            if justSpectra
                TFR = [];
                return 
            end
            
            % Run two TFRs
            ft_warning once 'FieldTrip:ft_freqanalysis:ft_specest_mtmconvol:line169'
            tconf = [];
            tconf.method     = 'mtmconvol';
            tconf.keeptrials = 'no';
            tconf.foi        = obj.cfg.spectra.foi_hanning(1):1:...
                    obj.cfg.spectra.foi_hanning(2);
            tconf.taper      = 'hanning';
            tconf.t_ftimwin  = repmat(0.5,length(tconf.foi),1);
            tconf.toi        = 'all';
            tconf.pad        = 4;
            tconf.polyremoval = obj.cfg.spectra.polyremoval;
            TFR_low         = ft_freqanalysis(tconf, dipole_all);
            
            % baseline the TFR
            conf = [];
            conf.baseline = [min(TFR_low.time) ...
                max(TFR_low.time(TFR_low.time<0))];
            conf.baselinetype = 'relative';
            TFR_low = ft_freqbaseline(conf, TFR_low);
            
            % average over dipoles
            conf = [];
            conf.avgoverchan = 'yes';
            TFR_low = ft_selectdata(conf, TFR_low);
            
            % compute multitaper TFR for higher frequencies
            tconf.foi    = obj.cfg.spectra.foi_dpss(1):...
                obj.cfg.spectra.foi_dpss(2);
            tconf.taper      = 'dpss';
            tconf.t_ftimwin  = repmat((1/3),length(tconf.foi),1);
            tconf.tapsmofrq  = obj.cfg.spectra.tapsmofrq;
            TFR_high        = ft_freqanalysis(tconf, dipole_all);
            ft_warning on
            
            % baseline the TFR
            conf = [];
            conf.baseline = [min(TFR_high.time) ...
                max(TFR_high.time(TFR_high.time<0))];
            conf.baselinetype = 'relative';
            TFR_high = ft_freqbaseline(conf, TFR_high);
            
            % average over dipoles
            conf = [];
            conf.avgoverchan = 'yes';
            TFR_high = ft_selectdata(conf, TFR_high);
            
            TFR = ft_appendfreq([], TFR_low, TFR_high);
            clear TFR_low TFR_high
                                    
            disp('Creating QC plot.')
            tfr_plot = figure('units', 'centimeters',...
                'position', [1 1 25 15], ...
                'Visible', 'on', 'Color', 'w');
            conf = [];
            conf.xlim = [-0.8, 1.6];
            %conf.baseline = [-1 0-1/1200];
            %conf.baselinetype = 'relative';
            conf.masknans = 'yes';
            ft_singleplotTFR(conf, TFR);
            title(sprintf('Induced changes at %s in dB', ...
                obj.cfg.freqs.doi_restriction));
            xlabel('Time (s)');
            ylabel('Freq (Hz)');
            h = yline(20.5, 'k', 'LineWidth', 5);
            h.Color = [0,0,0,1];
            cmocean('balance', 'pivot', 1);
            
            export_fig(tfr_plot, ...
                fullfile(obj.subject.saveplots.basepath,...
                'TFR', [stimType '_' obj.subject.subj_nr]), '-png',...
                '-q101', '-r400', '-nocrop');
            disp('TFR QC plot has been saved.')
        end
        
        function peaks = quantifySpectra(obj, QCplots)
            % Computes per-stimulus spectra, then attempty to quantify them
            % by fitting gaussians.
            if ~exist('QCplots', 'var') || isempty(QCplots)
                QCplots = true;
            end
            
            if isempty(obj.spect)
                disp('Computing average Spectra.')
                [~, obj.spect, obj.stimType] = obj.getSpectraAndTFR(true);
            end
            
            disp('Fitting trial-average spectra.')
            peaks = NaN(length(obj.stimType), 3, 2);
            qcf = figure('Unit', 'centimeters', 'Position', [0,0,15,23]);
            for iStim = 1:length(obj.stimType)
                conf = [];
                conf.frequency = [15, max(obj.spect{iStim}.freq)];
                avgSpectrum = ft_freqdescriptives(conf, obj.spect{iStim});
                iFreqs = avgSpectrum.freq;
                avgSpectrum = avgSpectrum.powspctrm -1;
                
                fSetting1 = fitoptions('Method','NonlinearLeastSquares', ...
                    'Lower', [-1, 0.05, 21, 3, 0], ...
                    'Upper', [1, Inf, max(iFreqs), 20, 0.1],...
                    'Start', [0, 1, 50, 5, 0], ...
                    'Weights', mat2gray(avgSpectrum),...
                    'MaxIter', 5000, 'MaxFunEvals', 5000);
                fSettings2 = fitoptions('Method','NonlinearLeastSquares', ...
                    'Lower', [-1, 0.05, 0.05, 21, 61, 3, 3, 0],...
                    'Upper', [1, Inf, Inf, max(iFreqs), max(iFreqs), 20, 20, 0.1],...
                    'Start', [0, 1, 1, 40, 100, 5, 5, 0],...
                    'Weights', mat2gray(avgSpectrum),...
                    'MaxIter', 5000, 'MaxFunEvals', 5000);
                fModel1 = fittype('a0*x + a1*exp(-((x-b1)/c1)^2)+d', ...
                    'options', fSetting1);
                fModel2 = fittype('a0*x + a1*exp(-((x-b1)/c1)^2) + a2*exp(-((x-b2)/c2)^2)+d', ...
                    'options', fSettings2);
                
                subplot(4,3,iStim);
                [f0, e0] = fit(iFreqs', avgSpectrum', 'poly1');

                [f1, e1] = fit(iFreqs', avgSpectrum', fModel1);
                [f2, e2] = fit(iFreqs', avgSpectrum', fModel2);
                
                if e0.adjrsquare > e1.adjrsquare && ...
                        e0.adjrsquare > e2.adjrsquare
                    disp('No peaks found.')
                    peaks(iStim,:,:) = NaN(3,2);
                    plot(f0, iFreqs, avgSpectrum);
                    title('No peaks.')
                elseif e2.adjrsquare > e1.adjrsquare
                    disp('Choosing two-peak model.')
                    if f2.b1 < f2.b2
                        peaks(iStim,:,:) = ...
                            [f2.a1, f2.a2; f2.b1, f2.b2; f2.c1, f2.c2];
                    else
                        peaks(iStim,:,:) = ...
                            [f2.a2, f2.a1; f2.b2, f2.b1; f2.c2, f2.c1];
                    end
                    
                    plot(f2, iFreqs, avgSpectrum);
                    title('2 peaks')
                else
                    disp('Choosinig one-peak model.')
                    if f1.b1 < 70
                        peaks(iStim,:,:) = ...
                            [f1.a1, NaN; f1.b1, NaN; f1.c1, NaN];
                    else
                        peaks(iStim,:,:) = ...
                            [NaN, f1.a1,; NaN, f1.b1; NaN, f1.c1];
                    end
                plot(f1, iFreqs, avgSpectrum);
                title('1 peak')
                end
                legend('hide');
                box off
            end
            
            obj.peaks = peaks;

            % Create table with peak frequencies
            [~, stimulus] = getColoredStimtypes(obj.subject.behavior);
            istimType = [repmat(string('color'),8,1); string('contrast'); ...
                string('grating')];
            ID = repmat(string(obj.subject.subj_nr), length(stimulus), 1);
            peaktable = table(ID, istimType, stimulus, ...
                peaks(:,2,1), peaks(:,2,2),...
                'VariableNames', {'ID', 'stimType', 'stimulus', ...
                'peak1', 'peak2'});
            disp('Writing gamma peak table to disc.')
            peakpath = fullfile(obj.cfg.paths.data, obj.subject.subj_nr,...
                'Preprocessed', 'gammapeaks.csv');
            writetable(peaktable, peakpath);
            
            if QCplots
                export_fig(qcf, fullfile(obj.cfg.paths.plot, 'Peakfits',...
                    obj.subject.subj_nr), '-png')
            end
        end
        
        function makeTrialTable(obj)
            % After the whole shebang has been run, get a per-trial table
            % with gamma power, color values, etc
            if isempty(obj.peaks)
                obj.quantifySpectra();
            end
            disp('Creating trial results table.')
            
            % Fill missing gamma peaks. For colored stimuli, take the
            % average of the other colored stimuli. For gratings, take the
            % (mean of the) surface peak(s).
            gammaPeakFound = true(size(obj.peaks,1),1);
            colorPeaks = obj.peaks(:,2,1);
            colorPeaks2 = obj.peaks(:,2,2);
            meanColorPeak = nanmean(colorPeaks(1:8));
            if isnan(meanColorPeak)
                meanColorPeak = mean(obj.subject.grating_gammapeak_sensors);
            end
            for iColor = 1:9
                if isnan(colorPeaks(iColor))
                    colorPeaks(iColor) = meanColorPeak;
                    gammaPeakFound(iColor) = false;
                end
            end
            if isnan(colorPeaks(10))
                colorPeaks(10) = mean(obj.subject.grating_gammapeak_sensors);
                gammaPeakFound(10) = false;
            end
            colorPeaks = round(colorPeaks);
            
            disp('Setting up skeleton trial table.')
            trialTable = obj.subject.behavior;
            trialTable.ID = repmat(obj.subject.subj_nr,size(trialTable,1),1);
            [trialTable.stimulus, ~, trialTable.repNr] = ...
                getColoredStimtypes(obj.subject.behavior);
            trialTable.stimType(...
                strcmp(trialTable.stimulus, string('contrast'))) = {'contrast'};
            trialTable.luminance = NaN(size(trialTable.trialNr));
            trialTable.LM = NaN(size(trialTable.trialNr));
            trialTable.SLM = NaN(size(trialTable.trialNr));
            trialTable.gammapower = NaN(size(trialTable.trialNr));
            trialTable.setGammapeak = NaN(size(trialTable.trialNr));
            trialTable.Gammapeak2 = NaN(size(trialTable.trialNr));
            trialTable.gammapeakFound = NaN(size(trialTable.trialNr));
            trialTable.correctReport = NaN(size(trialTable.trialNr));

            [RGBs, DKLs, LMSs] = stimtype2colors(obj.stimType, [], ...
                obj.subject.stim_cfg{1}.experiment.background);
            LABs = rgb2lab(RGBs, 'WhitePoint', 'd65', 'ColorSpace', 'linear-rgb');
            LABs(9:10,:) = NaN(size(LABs(9:10,:)));
            HSVs = rgb2hsv(lin2rgb(RGBs));
            HSVs(9:10,:) = NaN(size(HSVs(9:10,:)));
            % transform HSV_H into radians
            HSVs(:,1) = HSVs(:,1)*2;

            % Rescale RGB to [0 1]
            trialTable.colorR = trialTable.colorR ./ 255;
            trialTable.colorG = trialTable.colorG ./ 255;
            trialTable.colorB = trialTable.colorB ./ 255;

            disp('Filling trial table with DKL values & gamma power.')
            for iTrial = 1:max(trialTable.trialNr)
                iStim = strcmp(trialTable.stimulus(iTrial), obj.stimType);
                % add DKL infos
                trialTable.luminance(iTrial) = DKLs(iStim,1);
                trialTable.LM(iTrial) = DKLs(iStim,2);
                trialTable.SLM(iTrial) = DKLs(iStim,3);
                trialTable.L_cone(iTrial) = LMSs(iStim,1);
                trialTable.M_cone(iTrial) = LMSs(iStim,2);
                trialTable.S_cone(iTrial) = LMSs(iStim,3);
                trialTable.CIE_L(iTrial) = LABs(iStim,1);
                trialTable.CIE_a(iTrial) = LABs(iStim,2);
                trialTable.CIE_b(iTrial) = LABs(iStim,3);
                trialTable.HSV_h_rad(iTrial) = HSVs(iStim,1);
                trialTable.HSV_s(iTrial) = HSVs(iStim,2);
                trialTable.HSV_v(iTrial) = HSVs(iStim,3);
                trialTable.HSV_h_sin(iTrial) = sin(pi*HSVs(iStim,1));
                trialTable.HSV_h_cos(iTrial) = cos(pi*HSVs(iStim,1));

                % mark the defined per-stimulus gamma peak
                iFreq = colorPeaks(iStim) == obj.spect{iStim}.freq;
                trialTable.setGammapeak(iTrial) = colorPeaks(iStim);
                trialTable.Gammapeak2(iTrial) = colorPeaks2(iStim);
                trialTable.gammapeakFound(iTrial) = gammaPeakFound(iStim);
                % add power at the subject-stimulus gamma peak
                thisTrial = iTrial == obj.spect{iStim}.trialinfo;
                if sum(thisTrial) == 1
                    assert(sum(iFreq)==1, ...
                        'Frequency resolutions don''t match!');
                    trialTable.gammapower(iTrial) = ...
                        obj.spect{iStim}.powspctrm(thisTrial,:,iFreq);
                end
                
                % compute if answer was correct
                % make sure I'm using the correct key labels (1! and 4$)
                assert(obj.subject.stim_cfg{1}.experiment.leftKey == 11,...
                    'Wrong keylabels used to determine accuracy!');
                assert(obj.subject.stim_cfg{1}.experiment.rightKey == 14,...
                    'Wrong keylabels used to determine accuracy!');
                if strcmp(trialTable.answer{iTrial}, '1!') && ...
                        trialTable.changeX(iTrial) < 0
                    trialTable.correctReport(iTrial) = true;
                elseif strcmp(trialTable.answer{iTrial}, '4$') && ...
                        trialTable.changeX(iTrial) > 0
                    trialTable.correctReport(iTrial) = true;
                elseif strcmp(trialTable.answer{iTrial}, 'none') && ...
                        ~trialTable.wasCatch(iTrial)
                    trialTable.correctReport(iTrial) = false;
                elseif trialTable.wasCatch(iTrial) || ...
                        trialTable.changeX(iTrial) == 0
                    trialTable.correctReport(iTrial) = NaN;
                else
                    trialTable.correctReport(iTrial) = false;
                end
            end
            % check for pre-stimulus responses
            prepressed = trialTable.rt <0;
            trialTable.rt(prepressed) = NaN;
            trialTable.answer(prepressed) = {'prepress'};
            trialTable.correctReport(prepressed) = NaN;
            
            disp('If possible: load in eye data.')
            try
                saccadefile = fullfile(obj.cfg.paths.data,...
                    obj.subject.subj_nr, 'Preprocessed', 'saccades.csv');
                pupilfile = fullfile(obj.cfg.paths.data,...
                    obj.subject.subj_nr, 'Preprocessed', 'pupilsize.csv');
                eyedata = outerjoin(...
                    readtable(pupilfile), readtable(saccadefile), ...
                    'Keys', 'trialNr', 'MergeKeys', true);
                trialTable = outerjoin(eyedata, trialTable,...
                    'Keys', 'trialNr', 'MergeKeys', true);
            catch
                warning('No eyedata found.');
                trialTable.prestim_pupil = NaN(size(trialTable.trialNr));
                trialTable.poststim_pupil = NaN(size(trialTable.trialNr));
                trialTable.sac_per_trial = NaN(size(trialTable.trialNr));
            end
            
            % Compute pupil constiction
            trialTable.pupil_constriction = ...
                trialTable.prestim_pupil - trialTable.poststim_pupil;
            
            disp('If possible: load in ERF data.')
            try
                ERFfile = fullfile(obj.cfg.paths.data,...
                    obj.subject.subj_nr, 'Preprocessed', 'C1size.csv');
                trialTable = outerjoin(readtable(ERFfile), trialTable,...
                    'Keys', 'trialNr', 'MergeKeys', true);
            catch
                warning('No ERF data found.');
                trialTable.C1 = NaN(size(trialTable.trialNr));
            end
            
            disp('Reordering table, selecting only relevant variables.')
            trialTable = trialTable(:, {'ID', 'trialNr', 'repNr', ...
                'stimulus', 'stimType', 'orientation', ...
                'colorR', 'colorG', 'colorB', 'luminance', 'LM', 'SLM',...
                'L_cone', 'M_cone', 'S_cone', ...
                'CIE_L', 'CIE_a', 'CIE_b', ...
                'HSV_h_rad', 'HSV_s', 'HSV_v', 'HSV_h_sin', 'HSV_h_cos',...
                'tContrast', 'presLength', 'changeX', 'changeY', ...
                'correctReport', 'rt', 'answer', 'gammapower', ...
                'setGammapeak',  'gammapeakFound', 'C1', 'sac_per_trial', ...
                'pupil_constriction', 'prestim_pupil', 'poststim_pupil',...
                'block', 'wasCatch', 'wasFixated'});
            
            outpath = fullfile(obj.cfg.paths.data,...
                obj.subject.subj_nr, 'Preprocessed', 'gammapower.csv');
            writetable(trialTable, outpath);
            
        end
    end
end

