classdef GroupAnalysis < handle
    % GROUPANALYSIS Run analyses over all of our subjects. Requires individual
    % preprocessing, beamforming, etc to be finished.
    % Setup as obj = GroupAnalysis(subjects), whereby subjects is a vector
    % of subject numbers.
    
    properties
        subjects
        subject
        group_cfg
        data
        full_data
        rt % per-stimulus reaction time data
        sContrast % per-stimulus contrast data
        filename_gamma = 'DOIall_results.csv'
        filename_behav = 'subject.mat'
    end
    
    methods
        function obj = GroupAnalysis(subjects)
            obj.subjects = cellfun(@(x) num2str(x, '%03d'), ...
                num2cell(subjects), 'UniformOutput', false);
            obj.group_cfg = groupSetup(obj);
        end
        
        function loadSubjects(obj)
            % Load per-subject power spectra and behavioral data.
            for iSubj = 1:length(obj.subjects)
                this_subj = obj.subjects{iSubj};
                fprintf('Loading data for subject %s.\n', this_subj);
                subjpath = fullfile(obj.group_cfg.paths.data, this_subj, ...
                    'Preprocessed', 'subject');
                datpath = fullfile(obj.group_cfg.paths.data, this_subj, ...
                    'Preprocessed', 'gammapower.csv');
                ESIload(subjpath)
                obj.subject{iSubj} = subject;
                obj.data{iSubj} = readtable(datpath);
                
                assert(all(ismember(obj.data{iSubj}.answer, ...
                    {'1!', '4$', 'none', 'paused', 'prepress'})), ...
                    'Invalid answers found!')
                
                % check for trials with pre-pressed responses.
                prepressed = obj.data{iSubj}.rt <0;
                assert(sum(prepressed)==0, 'Prepressed responses found!');
            end
            
            obj.full_data = vertcat(obj.data{:});
        end
        
        function behaviorAnalysis(obj)
            % Correlate behavior, ERF and frequency results on a
            % per-subject basis over stimuli.
            
            % Load in data, if necessary
            if isempty(obj.data)
                disp('Loading in data.')
                obj.loadSubjects();
            end
            
            %% Compute per-stimulus behavior
            disp(['Computing per-subject, per-stimulus RT and final '...
                'change contrast.'])
            [~,stimtypes] = getColoredStimtypes(obj.data{1});
            obj.rt = NaN(length(obj.subjects), length(stimtypes));
            for iSubj = 1:length(obj.subjects)
                dat = obj.data{iSubj};
                iStims = getColoredStimtypes(dat);
                dat.correct = (ismember(dat.answer, '1!') & dat.changeX < 0) | ...
                    (ismember(dat.answer, '4$') & dat.changeX >  0);
                
                for iStimtype = 1:length(stimtypes)
                    iTrials = ismember(iStims ,stimtypes(iStimtype));
                    obj.rt(iSubj, iStimtype) = nanmean(dat.rt(iTrials));
                    ilastTrials = find(iTrials);
                    if isempty(ilastTrials)
                        lastTrials{iStimtype} = [];
                    else
                        lastTrials{iStimtype} = ilastTrials(end-9:end);
                    end
                    %lastTrial{iStimtype} = find(iTrials, 1 ,'last');
                    if isempty(lastTrials{iStimtype})
                        obj.sContrast(iSubj, iStimtype) = NaN;
                    else
                        obj.sContrast(iSubj, iStimtype) = mean(...
                            dat.tContrast(lastTrials{iStimtype}));
                    end
                end
            end
            
            %% Plot per-stimulus behavior
            %scaledRT = rescale(obj.rt, 'InputMin', min(obj.rt,[],2),...
            %    'InputMax', max(obj.rt,[],2));
            
            %obj.rt = 1-scaledRT;
            %obj.sContrast = log(obj.sContrast);
            obj.rt = obj.rt*1000; % to ms
            
            for iStim = 1:length(stimtypes)
                rtMean(iStim) = nanmean(obj.rt(:,iStim));
                rtCI(iStim,:) = bootci(10000, @nanmean, obj.rt(:,iStim));
                rtCI(iStim,:) = abs(rtCI(iStim,:) - rtMean(iStim));
                
                contrastMean(iStim) = nanmean(obj.sContrast(:,iStim));
                contrastCI(iStim,:) = ...
                    bootci(10000, @nanmean, obj.sContrast(:,iStim));
                contrastCI(iStim,:) = ...
                    abs(contrastCI(iStim,:) - contrastMean(iStim));
            end
            
            iColor = stimtype2colors(stimtypes);
            x = 1:10;
            cBars = figure(...
                'Unit', 'centimeter', 'Position', [0 0, 10, 5]);
            subplot(1,2,1)
            b = bar(x, rtMean, 'EdgeColor', 'none');
            b.FaceColor = 'flat';
            b.CData = iColor;
            hold on
            errorbar(x, rtMean, rtCI(:,1), rtCI(:,2),...
                'Color', 'k', 'LineStyle', 'none');
            box off
            title('Reaction time (ms)');
            ylim([0 600])
            
            subplot(1,2,2)
            b = bar(x, contrastMean, 'EdgeColor', 'none');
            b.FaceColor = 'flat';
            b.CData = iColor;
            hold on
            errorbar(x, contrastMean, contrastCI(:,1), contrastCI(:,2),...
                'Color', 'k', 'LineStyle', 'none');
            box off
            title('Final stimulus log-contrast');
            
            export_fig(cBars, ...
                fullfile(obj.group_cfg.paths.result, 'Behavior', ...
                'rt_and_contrast'), '-pdf');
            
            export_fig(cBars, ...
                fullfile(obj.group_cfg.paths.result, 'Behavior', ...
                'rt_and_contrast'), '-png', '-q101', '-r600');
            
        end
        
        function correlationBehaviorERF(obj, whichStimuli)
            % Calculates correlations between behavioral outcomes and ERF
            % size. Should be run after .ERFs() and .behaviorAnalysis()
            % have been run. whichStimuli
            % selects which stimuli are included in the calculation. 1-8
            % are the colors, 9 is color contrast, 10 is the grating.
            if isempty(obj.rt) || isempty(obj.sContrast)
                warning(['.behaviorAnalysis() needs to be run first '...
                    '- running now.'])
                obj.behaviorAnalysis();
            end
            
            if(~exist('whichStimuli', 'var') || isempty(whichStimuli))
                whichStimuli= 1:8;
            end
            
            warning(['Make sure you have run .ERFs() recently!']);
            ESIload(fullfile(obj.group_cfg.paths.data, 'Preprocessed', ...
                'ERFs'));
            
            % select only relevant ERF timepoints
            timewindow = time > -0.1 & time <= 0.4;
            time = time(timewindow);
            sERF = ERF(:,:,timewindow) .* obj.group_cfg.erfplot.scaling;
            
            %% Compute ERF-behavior correlation
            for iSubj = 1:length(obj.subjects)
                rtContrastCorr(iSubj) = ...
                    corr(obj.rt(iSubj,whichStimuli)', ...
                    obj.sContrast(iSubj,whichStimuli)', ...
                    'Type', 'Spearman');
                for iTime = 1:length(time)
                    rtERFCor(iSubj,iTime) = ...
                        corr(obj.rt(iSubj,whichStimuli)',...
                        sERF(iSubj,whichStimuli,iTime)',...
                        'Type', 'Spearman');
                    contrastERFCor(iSubj,iTime) = ...
                        corr(obj.sContrast(iSubj,whichStimuli)', ...
                        sERF(iSubj,whichStimuli,iTime)',...
                        'Type', 'Spearman');
                end
            end
            
            disp(['Testing the correlation between stimulus '...
                'change contrast and RT against zero'])
            [h, p, ci, stats] = ttest(rtContrastCorr)
            
            %% Plot ERF-behavior correlation
            % prevent rounding errors
            meancERF = mean(contrastERFCor);
            fprintf('Max ERF-contrast correlation: %f\n', max(meancERF));
            cicERF = bootci(10000, @nanmean, contrastERFCor);
            cicERF = abs(cicERF - meancERF);
            
            [~, statsCont] = permtest_t(contrastERFCor,[], 'rows', 'complete');
            hCont = bwconncomp(statsCont.h);
            hCont = hCont.PixelIdxList;
            disp('Significant ERF-staircase perf. correlations between:')
            for iCont = 1:length(hCont)
                fprintf('%3.1f-%3.1f ms.\n', time(min(hCont{iCont}))*1000, ...
                    time(max(hCont{iCont}))*1000)
                fprintf('Max. r = %f\n', max(abs(meancERF(hCont{iCont}))))
                fprintf('Max. p = %f\n', max(statsCont.p(hCont{iCont})))
            end
            
            iColor = myColors(3,'bright');
            
            figure('Unit', 'centimeter', 'position', [0 0 5 4],...
                'color', 'w')
            hold on
            yyaxis left
            boundedline(time, meancERF, cicERF', 'cmap', iColor(1,:));
            set(gca,'YColor','k');
            set(gca, 'YDir', 'reverse');
            
            yyaxis right
            plot(time, squeeze(mean(sERF, [1,2])), 'Color', [0.5 0.5 0.5]);
            set(gca,'YColor',[0.5 0.5 0.5]);
            ylabel('Average ERF (z-scored)');
            yyaxis left
            xlim([-0.1 0.4]);
            sigline_y = ylim;
            maxYlim = max(sigline_y);
            sigline_y = sigline_y(1);
            ylim([sigline_y + 0.1*sigline_y, maxYlim]);
            
            for iSig = 1:length(hCont)
                plot(time(hCont{iSig}), ...
                    repmat(sigline_y+0.01*sigline_y, length(hCont{iSig}),1),...
                    's', 'MarkerFaceColor', iColor(1,:), ...
                    'MarkerEdgeColor', 'none', 'MarkerSize', 1);
            end
            xlabel('Time (s)')
            ylabel('Corr(ERF, threshold')
            xline(0, ':');
            yline(0, ':');
            
            % Align zeros
            yyaxis left;
            yliml = ylim();
            ratio = yliml(1)/yliml(2);
            yyaxis right;
            ylimr = ylim;
            ylim([ylimr(1) ylimr(1)*ratio])
            
            plotpath = fullfile(obj.group_cfg.paths.result, ...
                'Correlations', 'ERF_vs_Stairc');
            export_fig(plotpath, '-pdf', '-painters')
            export_fig(plotpath, '-png', '-q101', '-r600')
        end
        
        function correlationBehaviorFreq(obj, whichStimuli)
            % Calculates correlations between behavioral outcomes and
            % spectral parameters of the gamma band. Should be run after
            % .Freqs() and .behaviorAnalysis() have been run. whichStimuli
            % selects which stimuli are included in the calculation. 1-8
            % are the colors, 9 is color contrast, 10 is the grating.
            
            if(~exist('whichStimuli', 'var') || isempty(whichStimuli))
                whichStimuli= 1:8;
            end
            
            if isempty(obj.rt) || isempty(obj.sContrast)
                warning(['.behaviorAnalysis() needs to be run first '...
                    '- running now.'])
                obj.behaviorAnalysis();
            end
            
            warning('Make sure you have run .Freqs() recently!')
            peaks = [];
            ESIload(obj.group_cfg.paths.freqData);
            
            mSpects = squeeze(nanmean(spects, 3));
            mSpects = mSpects(:,whichStimuli,:);
            %% Compute spectrum-behavior correlation
            for iSubj = 1:length(obj.subjects)
                gamma1 = squeeze(peaks(iSubj,:,:,1));
                gamma2 = squeeze(peaks(iSubj,:,:,2));
                
                gammaContrastCorr(iSubj,1:3) = ...
                    corr(gamma1(whichStimuli,:),...
                    obj.sContrast(iSubj,whichStimuli)', ...
                    'Type', 'Spearman', 'rows', 'complete');
                gammaContrastCorr(iSubj,4:6) = ...
                    corr(gamma2(whichStimuli,:),...
                    obj.sContrast(iSubj,whichStimuli)', ...
                    'Type', 'Spearman', 'rows', 'complete');
                gammaRTCorr(iSubj,1:3) = ...
                    corr(gamma1(whichStimuli,:),...
                    obj.rt(iSubj,whichStimuli)', ...
                    'Type', 'Spearman', 'rows', 'complete');
                gammaRTCorr(iSubj,4:6) = ...
                    corr(gamma2(whichStimuli,:),...
                    obj.rt(iSubj,whichStimuli)', ...
                    'Type', 'Spearman', 'rows', 'complete');
                
                for iFreq = 1:length(freqsAxis)
                    rtFreqCor(iSubj,iFreq) = ...
                        corr(obj.rt(iSubj,whichStimuli)', ...
                        mSpects(iSubj,:,iFreq)',...
                        'Type', 'Spearman');
                    contrastFreqCor(iSubj,iFreq) = ...
                        corr(obj.sContrast(iSubj,whichStimuli)', ...
                        mSpects(iSubj,:,iFreq)',...
                        'Type', 'Spearman');
                end
            end
            
            disp(['Testing the correlation between stimulus '...
                'RT and gamma properties against zero:'])
            [h, p, ci, stats] = ttest(gammaRTCorr)
            
            disp(['Testing the correlation between stimulus '...
                'change contrast and gamma properties against zero:'])
            [h, p, ci, stats] = ttest(gammaContrastCorr)
            
            %% Plot Spectrum-behavior correlation
            meanSpect = squeeze(nanmean(mSpects,2));
            
            meanCorr = mean(contrastFreqCor);
            ciCorr = bootci(10000, @mean, contrastFreqCor);
            ciCorr = abs(ciCorr-meanCorr);
            
            [~, statsCont] = permtest_t(contrastFreqCor,[], 'rows', 'complete');
            hCont = bwconncomp(statsCont.h);
            hCont = hCont.PixelIdxList;
            disp('Significant power-staircase perf. correlations between:')
            for iCont = 1:length(hCont)
                fprintf('%3.1f-%3.1f Hz.\n', freqsAxis(min(hCont{iCont})),...
                    freqsAxis(max(hCont{iCont})))
                %disp(statsCont.p(hCont{iCont}))
                fprintf('Max. r = %f\n', max(abs(meanCorr(hCont{iCont}))))
                fprintf('Max. p = %f\n', max(statsCont.p(hCont{iCont})))
            end
            
            iColor = myColors(3,'bright');
            
            figure('Unit', 'centimeter', 'position', [0 0 5 4],...
                'color', 'w')
            hold on
            yyaxis left
            boundedline(freqsAxis, mean(contrastFreqCor), ciCorr', ...
                'cmap', iColor(1,:));
            set(gca,'YColor','k');
            set(gca, 'YDir', 'reverse');
            
            yyaxis right
            plot(freqsAxis, mean(meanSpect), 'Color', [0.5 0.5 0.5]);
            ylabel('Power change (%)');
            set(gca,'YColor',[0.5 0.5 0.5]);
            yyaxis left
            sigline_y = ylim;
            maxYlim = max(sigline_y);
            sigline_y = sigline_y(1);
            ylim([sigline_y + 0.1*sigline_y, maxYlim]);
            xlim([min(freqsAxis) max(freqsAxis)]);
            
            for iSig = 1:length(hCont)
                yyaxis left
                plot(freqsAxis(hCont{iSig}), ...
                    repmat(sigline_y+0.01*sigline_y, length(hCont{iSig}),1),...
                    's', 'MarkerFaceColor', iColor(1,:), ...
                    'MarkerEdgeColor', 'none', 'MarkerSize', 1);
            end
            xlabel('Frequency (Hz.)')
            ylabel('Corr(Power, threshold)')
            xline(0, ':');
            yline(0, ':');
            yyaxis right
            ylim([-20 15])
            
            % Align zeros
            yyaxis left;
            yliml = ylim();
            ratio = yliml(1)/yliml(2);
            yyaxis right;
            ylimr = ylim;
            ylim([ylimr(1) ylimr(1)*ratio])
            
            plotpath = fullfile(obj.group_cfg.paths.result, ...
                'Correlations', 'Spect_vs_Stairc');
            export_fig(plotpath, '-pdf', '-painters')
            export_fig(plotpath, '-png', '-q101', '-r600')
            
        end
        
        function sigPos = correlationERFFreq(obj, whichStimuli)
            % calculates correlation coefficients between all ERF
            % timepoints and induced power frequencies
            if(~exist('whichStimuli', 'var') || isempty(whichStimuli))
                whichStimuli= 1:8;
            end
            
            warning(['Make sure you have run .ERFs() recently!']);
            ESIload(fullfile(obj.group_cfg.paths.data, 'Preprocessed', ...
                'ERFs'));
            warning('Make sure you have run .Freqs() recently!')
            peaks = [];
            ESIload(obj.group_cfg.paths.freqData);
            
            % select only relevant ERF timepoints
            timewindow = time > -0.1 & time <= 0.4;
            time = time(timewindow);
            sERF = ERF(:,whichStimuli,timewindow) .*...
                obj.group_cfg.erfplot.scaling;
            
            mSpects = squeeze(nanmean(spects, 3));
            mSpects = mSpects(:,whichStimuli,:);
            
            %% Compute ERF-behavior correlation
            spectERFcorr = NaN(length(obj.subjects), size(sERF,3), ...
                size(mSpects,3));
            disp('Computing ERF-Spectrum correlation matrices')
            for iSubj = 1:length(obj.subjects)
                fprintf('Computing subject %i/%i.\n', iSubj, ...
                    length(obj.subjects));
                for iTime = 1:length(time)
                    for iFreq = 1:length(freqsAxis)
                        spectERFcorr(iSubj, iTime, iFreq) = ...
                            corr(sERF(iSubj, :, iTime)',...
                            mSpects(iSubj, :, iFreq)',...
                            'Type', 'Spearman', 'rows', 'complete');
                    end
                end
            end
            
            mSpectERFcorr = squeeze(nanmean(spectERFcorr));
            
            % Create significance shading map
            disp('Running t-max significance test.')
            [~, hSpectERFcorr] = permtest_t(...
                reshape(spectERFcorr, size(spectERFcorr,1), ...
                size(spectERFcorr,2)*size(spectERFcorr,3)));
            pSpectERFcorr = reshape(hSpectERFcorr.p, size(mSpectERFcorr));
            hSpectERFcorr = reshape(hSpectERFcorr.h, size(mSpectERFcorr));
            
            transp = (1-hSpectERFcorr).*0.5;
            transp(transp==0) = 1;
            [sigPosX, sigPosY] = find(hSpectERFcorr);
            sigPos = [freqsAxis(sigPosY); time(sigPosX)];
            disp('Significant ERF-Power correlations at:');
            disp(sigPos);
            disp('max. p-values are:')
            disp(max(pSpectERFcorr(logical(hSpectERFcorr))'));
            disp('max. r-values are:')
            disp(max(mSpectERFcorr(logical(hSpectERFcorr))'));
            
            % Plot it
            figure('units', 'centimeters',...
                'position', [1 1 6 3.5], 'Color', 'w');
            hold on
            imagesc(time, freqsAxis, mSpectERFcorr');
            plot(time(sigPosX), freqsAxis(sigPosY), 'k.', 'MarkerSize',1);
            axis ij
            set(gca,'YDir','normal')
            set_colormap('-div', 0);
            %set_colormap('-seq');
            hbar = colorbar();
            ylabel(hbar, ['Cor(ERF, Power)']);
            box off
            ylim([min(freqsAxis), max(freqsAxis)]);
            axis square
            xlabel('Time (s)');
            ylabel('Frequency (Hz.)');
            
            plotpath = fullfile(obj.group_cfg.paths.result, ...
                'Correlations', 'ERF_vs_spect');
            
            export_fig(plotpath, '-pdf', '-painters')
            export_fig(plotpath, '-png', '-q101', '-r600')
            
        end
        
        function plotAnyVariable(obj, sWhich)
            
            if ~exist('sWhich', 'var') || isempty(sWhich)
                sWhich = {'tContrast', 'correctReport', 'rt', 'gammapower',...
                    'C1', 'sac_per_trial', 'prestim_pupil', ...
                    'poststim_pupil', 'pupil_constriction', 'wasFixated'};
            end
            
            if ischar(sWhich)
                sWhich = {sWhich};
            end
            
            %% Plots
            for iPlot = 1:length(sWhich)
                toPlot = sWhich{iPlot};
                
                effectplot_time = figure('units', 'centimeters',...
                    'position', [1 1 12 7], 'Color', 'w', ...
                    'Visible', 'on');
                hold on
                
                nBlocks = max(obj.full_data.block);
                iColors = myColors(nBlocks,'bright');
                blocks = zeros(nBlocks, 2);
                for iBlock = 1:nBlocks
                    blocks(iBlock, 1) = min(...
                        obj.full_data.trialNr(...
                        obj.full_data.block == iBlock));
                    blocks(iBlock, 2) = max(...
                        obj.full_data.trialNr(...
                        obj.full_data.block == iBlock));
                end
                first = min(obj.full_data.trialNr);
                last = max(obj.full_data.trialNr);
                data_cell = splitapply(@(x) {x}, obj.full_data{:, toPlot}, ...
                    findgroups(obj.full_data.ID))';
                trialnr_cell =  splitapply(@(x) {x}, obj.full_data.trialNr, ...
                    findgroups(obj.full_data.ID))';
                smoothedGroupLinePlot(trialnr_cell, data_cell, ...
                    'fullX', first:last, ...
                    'span', obj.group_cfg.trialplot.span, ...
                    'blocks', blocks, ...
                    'demean', false,...%obj.group_cfg.trialplot.demean, ...
                    'zscore', false,... %obj.group_cfg.trialplot.zscore
                    'color', iColors,...
                    'bootci', true);
                %title('Group results');
                xlabel('Trial');
                ylabel(sprintf('%s change', toPlot));
                
                plotpath = fullfile(obj.group_cfg.paths.result, 'OtherPlots', ...
                    ['trials_', toPlot]);
                
                export_fig(effectplot_time, plotpath, ...
                    '-png', '-q101', '-r600', '-nocrop');
                
                export_fig(effectplot_time, plotpath, '-pdf', '-painters');
            end
        end
        
        function powerModel(obj)
            % Generate a hierarchical model over all subjects of per-trial
            % gamma power based on stimulus, difficulty, stim-nr, trial-nr,
            % repetition, recency, etc. Output results and plots.
            %% Model
            
            %% Plots
            effectplot_time = figure('units', 'centimeters',...
                'position', [1 1 25 15], ...
                'Visible', 'off');
            set(effectplot_time,'color','w');
            hold on
            
            nBlocks = max(obj.data.block);
            iColors = lines(nBlocks);
            blocks = zeros(nBlocks, 2);
            for iBlock = 1:nBlocks
                blocks(iBlock, 1) = min(...
                    obj.data.trialNr(obj.data.block == iBlock));
                blocks(iBlock, 2) = max(...
                    obj.data.trialNr(obj.data.block == iBlock));
            end
            first = min(obj.data.trialNr);
            last = max(obj.data.trialNr);
            data_cell = splitapply(@(x) {x}, obj.data.gamma_inc, ...
                findgroups(obj.data.subjNr))';
            trialnr_cell =  splitapply(@(x) {x}, obj.data.trialNr, ...
                findgroups(obj.data.subjNr))';
            smoothedGroupLinePlot(trialnr_cell, data_cell, ...
                'fullX', first:last, ...
                'span', obj.group_cfg.trialplot.span, ...
                'blocks', blocks, ...
                'demean', obj.group_cfg.trialplot.demean, ...
                'zscore', obj.group_cfg.trialplot.zscore, ...
                'color', iColors);
            title('Group results');
            xlabel('Trial');
            ylabel('Mean induced gamma change');
            
            export_fig(effectplot_time, ...
                fullfile(obj.group_cfg.paths.result, 'Gammapower', 'all' ),...
                '-png', '-q101', '-r400', '-nocrop');
        end
        
        function averageSpectra(obj)
            % Generate per-bin average spectra (aligned or unaligned) over
            % all subjects. Plot them.
            
            % Find the columns which have spectrally resolved gamma power
            % data
            var_names = obj.data.Properties.VariableNames;
            spect_cols = ~cellfun(@isempty, ...
                regexp(var_names, 'gamma_inc_sp*'));
            
            freqs_axis = 1 : sum(double(spect_cols));
            freqs_axis = freqs_axis - mean(freqs_axis);
            
            bin_Nr = repelem(1:obj.group_cfg.spectplot.bins, ...
                max(obj.data.stimNr)/obj.group_cfg.spectplot.bins);
            n_Blocks = max(obj.data.block);
            effectplot_spect = figure('units', 'centimeters',...
                'position', [1 1 n_Blocks*14 12], ...
                'Visible', 'off');
            set(effectplot_spect,'color','w');
            i_Bin_Nr = bin_Nr(obj.data.stimNr)' + ...
                (obj.data.block*obj.group_cfg.spectplot.bins);
            hold on
            title('Induced gamma spectra over bins');
            for i_Block = 1:n_Blocks
                subplot(1, n_Blocks, i_Block);
                these_Bins = unique(i_Bin_Nr(obj.data.block==i_Block));
                colors = lines(length(these_Bins));
                set(gca, 'ColorOrder', colors);
                hold on
                for i_Bin = 1:length(these_Bins)
                    this_Bin = these_Bins(i_Bin);
                    i_Means = ESI_nanmean(obj.data{...
                        i_Bin_Nr == this_Bin, spect_cols})*100;
                    
                    plot(freqs_axis, i_Means);
                    
                    iLegend{i_Bin} = sprintf('Bin %i', i_Bin);
                end
                xlabel('Frequency (Hz)');
                ylabel('Mean induced change (%)');
                ylim([80 260]);
                title(sprintf('Block %i', i_Block));
                legend(iLegend, 'Location','southoutside',...
                    'Orientation','horizontal');
            end
            
            %non_axis = axes('Visible', 'off');
            %title('Induced gamma spectra over bins', 'Visible', 'on');
            
            export_fig(effectplot_spect, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', 'all' ),...
                '-png', '-q101', '-r400');
        end
        
        function gammaLocalisation(obj)
            % Load per-participant fs_LR aligned DICS results, then average and
            % plot.
            
            stimtypes = {'color', 'contrast', 'grating'};
            % size of the 2 hemisphere 32k fs_LR, 2nd dim = stimtype
            subj_pow = zeros(64984, 2, length(obj.subjects));
            for i_subj = 1:length(obj.subjects)
                this_subj = obj.subjects{i_subj};
                fprintf('Loading DICS results for subject %s.\n', this_subj);
                ESIload(fullfile(obj.group_cfg.paths.data, this_subj,...
                    'Preprocessed', 'gammasources.mat'));
                for iStimtype = 1:length(stimtypes)
                    peakHere = ~cellfun(@isempty, gammasources(iStimtype,:));
                    nPeaks = sum(peakHere);
                    if nPeaks > 1
                        out = zeros(64984,1,1);
                        for iPeak = 1:length(peakHere)
                            if peakHere(iPeak)
                                % iterative mean, to save RAM
                                out = out + 1/nPeaks .* ...
                                    gammasources{iStimtype, iPeak}.avg.pow;
                            end
                        end
                        subj_pow(:,iStimtype,i_subj) = out;
                    else
                        subj_pow(:,iStimtype,i_subj) = ...
                            gammasources{iStimtype, peakHere}.avg.pow;
                    end
                end
            end
            
            for iStimtype = 1:length(stimtypes)
                % Convert to % change
                subj_pow_pC = pow2pChange(subj_pow(:,iStimtype,:));
                % Take avg and FDR-corrected significance of induced gamma
                % power at each position.
                avg_pow_all = mean(subj_pow_pC, 3);
                %[~, permstat] = permtest_t(squeeze(subj_pow_pC)');
                %h = double(permstat.h)';
                [~, p] = ttest(squeeze(subj_pow_pC)');
                h = double(fdr_bh(p, 0.05, 'dep', 'yes'))';
                avg_pow = avg_pow_all .* h; % mask for significance
                
                % plot average dics results
                template = ft_read_headshape(obj.group_cfg.hcp.plot_template);
                atlas = ft_read_cifti(obj.group_cfg.hcp.atlas, ...
                    'mapname', 'array', 'readdata', true);
                template.labels = label_HCP_areas([], atlas, ...
                    'V1', 'V2', 'V3', 'V4', 'V3A');
                conf = [];
                conf.mask = 'no';
                
                dicsplot = figure('units', 'centimeters',...
                    'position', [1 1 7 3], ...
                    'Visible', 'on', 'Color', 'w');
                subplot('Position', [.015, .00, .4, 1.0]);
                HCP_source_plot(conf, template, avg_pow);
                view([40, -5]);
                set_colormap('div');
                camlight('headlight')
                subplot('Position', [.45, .00, .4, 1.0]);
                HCP_source_plot(conf, template, avg_pow);
                hbar = colorbar('manual', 'Position', [0.89 0.2 0.03 0.6]);
                ylabel(hbar, ['Induced gamma power change (%)']);
                view([-40, -5]);
                set_colormap('div');
                camlight('headlight')
                disp('Saving HCP DICS plot.')
                
                plotpath = fullfile(obj.group_cfg.paths.result, ...
                    'Gammapower', [stimtypes{iStimtype} '_DICS_HCP']);
                savefig(dicsplot, plotpath);
                export_fig(plotpath, '-png', '-nocrop', ...
                    '-q101', '-r600')
                
                % plot color bar only as vector graphic
                oldaxes = gca;
                figure('units', 'centimeters','position', [1 1 7 3], ...
                    'Visible', 'on', 'Color', 'w');
                bars2 = colorbar('manual', 'Position', [0.8 0.2 0.03 0.6]);
                ylabel(bars2, ['Gamma power change (%)']);
                set(gca, 'CLim', oldaxes.CLim);
                set_colormap('div');
                set(gca, 'Visible', 'off');
                bars2.Limits = hbar.Limits;
                export_fig(plotpath, '-pdf', '-painters', '-nocrop')
                
                % plot on retinotopy
                figure('units', 'centimeters','position', [1 1 5 4], ...
                    'Visible', 'on', 'Color', 'w');
                subplot('Position', [.00, .00, .8, 1]);
                gamma_retinotopy_plot(obj.group_cfg, atlas, avg_pow,...
                    {'V1'});
                set_colormap('seq');
                box off
                hbar = colorbar('manual', 'Position', [0.85 0.2 0.03 0.6]);
                ylabel(hbar, 'Induced gamma power change (%)');
                
                disp('Saving retinotopic DICS plot.')
                plotpath = fullfile(obj.group_cfg.paths.result, ...
                    'Gammapower', [stimtypes{iStimtype} '_DICS_retinotopic']);
                export_fig(plotpath, '-png', '-nocrop', ...
                    '-q101', '-r1000')
                export_fig(plotpath, '-pdf', '-painters')
                
                % Create a table of highest gamma power
                all_areas = unique(atlas.corticalareas_dil_final_final_areas_group_colors);
                all_areas = all_areas(~isnan(all_areas));
                all_area_names = ...
                    atlas.corticalareas_dil_final_final_areas_group_colorslabel(all_areas)';
                all_area_names = cellfun(@(x) strsplit(x, '_'), all_area_names(1:180),...
                    'UniformOutput', false);
                all_area_names = cellfun(@(x) x{2}, all_area_names,...
                    'UniformOutput', false);
                
                for iStim = 1:size(subj_pow, 2)
                    results = NaN(length(all_areas)/2, 4);
                    area_gamma = NaN(length(all_areas)/2,...
                        length(obj.subjects));
                    
                    for iArea = 1:length(all_areas)/2
                        fprintf('Creating statistics for area %i of %i.\n', iArea,...
                            length(all_areas)/2);
                        this_dipoles1 = ... % right
                            atlas.corticalareas_dil_final_final_areas_group_colors ==...
                            iArea;
                        this_dipoles2 =  ...% left
                            atlas.corticalareas_dil_final_final_areas_group_colors ==...
                            iArea + 180;
                        this_dipoles = this_dipoles1 | this_dipoles2;
                        iMean = ...
                            squeeze(mean(subj_pow(this_dipoles, iStim,:)));
                        iMean = (iMean - 1).*100; % in percent
                        results(iArea, 1) = mean(iMean);
                        results(iArea, 2:3) = bootci(1000, @mean, ...
                            iMean);
                        area_gamma(iArea, :) = ...
                            squeeze(mean(subj_pow(this_dipoles, iStim,:)));
                    end
                    [~,area_rise_p] = ...
                        permtest_t(squeeze(area_gamma)',[],...
                        'm', 1);
                    results(:,4) = area_rise_p.p;
                    
                    area_coefs = [table(all_area_names), array2table(results)];
                    area_coefs.Properties.VariableNames = {'Area', ...
                        'Gammapower', 'PowerCILow', 'PowerCIHigh', 'p'};
                    area_coefs = sortrows(area_coefs, 'Gammapower', 'descend');
                    
                    tablepath = fullfile(obj.group_cfg.paths.result, ...
                        'Gammapower', [stimtypes{iStim} '_DICS_HCP.csv']);
                    writetable(area_coefs, tablepath, 'Delimiter', ';')
                end
            end
        end
        
        function ERFLocalisation(obj)
            % Load per-participant fs_LR aligned ERFs, then average and
            % plot.
            
            % size of the 2 hemisphere 32k fs_LR, 2nd dim = stimtype
            subj_amp = zeros(64984, length(obj.subjects));
            stimtypes = {'color', 'grating'};
            
            for iStimtype = 1:length(stimtypes);
                for i_subj = 1:length(obj.subjects)
                    this_subj = obj.subjects{i_subj};
                    fprintf('Loading ERF localization for subject %s.\n', ...
                        this_subj);
                    ESIload(fullfile(obj.group_cfg.paths.data, this_subj,...
                        'Preprocessed', 'ERFsources.mat'));
                    subj_amp(:,iStimtype,i_subj) = ERFsources{iStimtype}';
                end
            end
            
            for iStimtype = 1:length(stimtypes)
                % Take avg and FDR-corrected significance of ERF amp
                % at each position.
                avg_amp_all = mean(subj_amp(:,iStimtype,:), 3);
                %[~, permstat] = permtest_t(squeeze(subj_pow_pC)');
                %h = double(permstat.h)';
                [~, p] = ttest(squeeze(subj_amp(:,iStimtype,:))');
                h = double(fdr_bh(p, 0.05, 'dep', 'yes'))';
                avg_pow = avg_amp_all%.* h; % mask for significance
                
                % plot average dics results
                template = ft_read_headshape(obj.group_cfg.hcp.plot_template);
                atlas = ft_read_cifti(obj.group_cfg.hcp.atlas, ...
                    'mapname', 'array', 'readdata', true);
                template.labels = label_HCP_areas([], atlas, ...
                    'V1', 'V2', 'V3', 'V4', 'V3A');
                conf = [];
                conf.mask = 'no';
                
                dicsplot = figure('units', 'centimeters',...
                    'position', [1 1 7 3], ...
                    'Visible', 'on', 'Color', 'w');
                subplot('Position', [.015, .00, .4, 1.0]);
                HCP_source_plot(conf, template, avg_pow);
                view([40, -5]);
                set_colormap('div');
                camlight('headlight')
                subplot('Position', [.45, .00, .4, 1.0]);
                HCP_source_plot(conf, template, avg_pow);
                hbar = colorbar('manual', 'Position', [0.89 0.2 0.03 0.6]);
                ylabel(hbar, ['Induced gamma power change (%)']);
                view([-40, -5]);
                set_colormap('div');
                camlight('headlight')
                disp('Saving HCP ERF plot.')
                
                plotpath = fullfile(obj.group_cfg.paths.result, ...
                    'ERFs', [stimtypes{iStimtype} '_HCP']);
                savefig(dicsplot, plotpath);
                export_fig(plotpath, '-png', '-nocrop', ...
                    '-q101', '-r600')
                
                % plot color bar only as vector graphic
                oldaxes = gca;
                figure('units', 'centimeters','position', [1 1 7 3], ...
                    'Visible', 'on', 'Color', 'w');
                bars2 = colorbar('manual', 'Position', [0.8 0.2 0.03 0.6]);
                ylabel(bars2, ['Gamma power change (%)']);
                set(gca, 'CLim', oldaxes.CLim);
                set_colormap('div');
                set(gca, 'Visible', 'off');
                bars2.Limits = hbar.Limits;
                export_fig(plotpath, '-pdf', '-painters', '-nocrop')
            end
        end
        
        function rerunPreprocessing(obj)
            % Rerun the non-manual parts of the preprocessing pipeline
            % using pre-defined artifacts according to the same configs for
            % every subject (if requested, as a Slurm job).
            switch obj.group_cfg.general.use_slurm
                case false
                    for iSubj = 1:length(obj.subjects)
                        subj_nr = str2num(obj.subjects{iSubj});
                        prep = Preprocessing(subj_nr);
                        prep.loadRaw();
                        prep.selectCleanChans();
                        %prep.savedArtefactDetection();
                        %prep.removeArtefacts();
                        %prep.QCplots();
                    end
                case true
                    disp('Running per-participant preprocessing on SLURM.')
                    out = slurmfun(@slurmRerunPreprocessing, obj.subjects,...
                        'partition', '24GBL', 'stopOnError', false);
            end
        end
        
        function rerunGammaPeak(obj)
            % Rerun the sensor- and sourcelevel pipeline to find
            % per-subject grating and gamma peaks.
            if obj.group_cfg.general.use_slurm
                warning(['rerunGammaPeak sadly does not work on slurm '...
                    'due to Python issues.'])
            end
            for iSubj = 1:length(obj.subjects)
                subj_nr = str2double(obj.subjects{iSubj});
                beamy = Beamformer(subj_nr);
                beamy.loadData();
                beamy.findGammaPeak();
            end
            disp('Done!');
        end
        
        function rerunBeamforming(obj)
            % Rerun the non-manual parts of the beamforming pipeline using
            % pre-defined alignment points according to the same configs for
            % every subject (if requested, as a Slurm job).
            switch obj.group_cfg.general.use_slurm
                case false
                    for iSubj = 1:length(obj.subjects)
                        subj_nr = obj.subjects(iSubj);
                        beamy = Beamformer(subj_nr);
                        %beamy.redoDICSplot();
                        beamy.doEverything;
                    end
                case true
                    disp('Running per-participant beamforming on SLURM.')
                    out = slurmfun(@slurmRerunBeamforming, obj.subjects,...
                        'partition', '48GBL', 'stopOnError', false);
            end
        end
        
        function rerunEffectQuantification(obj)
            % Rerun the individual effect quantification pipeline
            % according to the same configs for every subject
            % (if requested, as a Slurm job).
            switch obj.group_cfg.general.use_slurm
                case false
                    for iSubj = 1:length(obj.subjects)
                        subj_nr = obj.subjects(iSubj);
                        fprintf('Working on subject %s.\n', subj_nr);
                        freqIt = Freqs(subj_nr);
                        freqIt.runSpectraPipeline()
                    end
                case true
                    disp('Running per-participant FFTs on SLURM.')
                    out = slurmfun(@slurmMakeResults, obj.subjects,...
                        'partition', '16GB', 'stopOnError', true,...
                        'deleteFiles', false);
            end
        end
        
        function rerunERFQuantification(obj)
            % Rerun the individual ERF size quantification pipeline
            % according to the same configs for every subject
            % (if requested, as a Slurm job).
            switch obj.group_cfg.general.use_slurm
                case false
                    for iSubj = 1:length(obj.subjects)
                        subj_nr = obj.subjects(iSubj);
                        fprintf('Working on subject %s.\n', subj_nr);
                        herfderf = ERFs(subj_nr);
                        herfderf.loadData()
                        herfderf.lpfilterSensorData();
                        herfderf.computeAreaSources('V1');
                        herfderf.binnedERFs();
                        herfderf.makeERFtable();
                    end
                case true
                    disp('Running per-participant ERFs on SLURM.')
                    out = slurmfun(@slurmMakeERFtable, obj.subjects,...
                        'partition', '48GBL', 'stopOnError', true);
            end
        end
        
        function rerunERFLocalization(obj)
            % Rerun the individual ERF localization pipeline
            % according to the same configs for every subject
            % (if requested, as a Slurm job).
            switch obj.group_cfg.general.use_slurm
                case false
                    for iSubj = 1:length(obj.subjects)
                        subj_nr = obj.subjects(iSubj);
                        timerangeInSec = [0.07 0.09];
                        % time range of the ERF that we want to localize
                        fprintf('Working on subject %s.\n', subj_nr);
                        herf = ERFs(subj_nr);
                        herf.loadData();
                        herf.lpfilterSensorData()
                        herf.localizeERFtime(timerangeInSec);
                    end
                case true
                    disp('Running per-participant ERFs on SLURM.')
                    out = slurmfun(@slurmLocalizeERFs, obj.subjects,...
                        'partition', '32GBS', 'stopOnError', true);
            end
        end
        
        function rerunEyeAnalysis(obj)
            % Rerun the individual saccade detection and pupil size
            % analysis pipeline (if requested, as a Slurm job).
            switch obj.group_cfg.general.use_slurm
                case false
                    for iSubj = 1:length(obj.subjects)
                        subj_nr = obj.subjects(iSubj);
                        fprintf('Working on subject %s.\n', subj_nr);
                        eyes = Eyedata(subj_nr);
                        eyes.doEverything()
                    end
                case true
                    disp('Running per-participant eye analysis on SLURM.')
                    out = slurmfun(@slurmRerunEyeAnalysis, obj.subjects,...
                        'partition', '8GBS', 'stopOnError', true);
            end
        end
        
        function freqPlots(obj, whichStims, plotAllSigbars, plotSpectCIs)
            % Return stimulus-induced spectra and TFRs
            % plotAllSigbars determines if significances against zero are
            % plotted for all stimuli. spectCIs can also turn on CI shading
            % in the spectrum plot.
            if ~exist('plotAllSigbars', 'var')||isempty(plotAllSigbars)
                plotAllSigbars = false;
            end
            if ~exist('whichStims', 'var')||isempty(whichStims)
                whichStims = [1:8 10];
            end
            if ~exist('plotSpectCIs', 'var')||isempty(plotSpectCIs)
                plotSpectCIs = false;
            end
            
            if obj.group_cfg.general.rerunAnalyses
                disp('Running per-participant TFR analysis on SLURM.')
                out = slurmfun(@slurmGetFreq, obj.subjects,...
                    'partition', '24GBL', 'stopOnError', false, ...
                    'deleteFiles', false);
                
                nFreqs = length(out{1}.spect{1}.freq);
                freqsAxis = out{1}.spect{1}.freq;
                stimtypes = out{1}.stimType;
                nTrials = 60;
                spects = NaN(length(obj.subjects), length(stimtypes), ...
                    nTrials, nFreqs);
                peaks = NaN(length(obj.subjects), length(stimtypes), 3, 2);
                
                for iSubj = 1:length(obj.subjects)
                    peaks(iSubj,:,:,:) = out{iSubj}.peaks;
                    for iStimtype = 1:length(stimtypes)
                        for iTrial = 1:length(out{iSubj}.spect{iStimtype}.trialinfo(:,1)) % TODO implement rep nr properly
                            %thisT = out{iSubj}.spect{iStimtype}.trialinfo(iTrial,1);
                            spects(iSubj, iStimtype, iTrial,:) = ...
                                out{iSubj}.spect{iStimtype}.powspctrm(iTrial,:,:);
                        end
                        % Deal with TFRs
                        TFRs{iStimtype, iSubj} = out{iSubj}.TFR{iStimtype};
                        TFRs{iStimtype, iSubj}.label = {'source'};
                        
                    end
                end
                spects = pow2pChange(spects);
                ESIsave(obj.group_cfg.paths.freqData, ...
                    'TFRs',  'spects', 'peaks', 'freqsAxis', 'stimtypes');
                
            else
                disp('Loading in existing spectra results.')
                freqs = [];
                ESIload(obj.group_cfg.paths.freqData);
                freqsAxis = freqs;
                freqsAxis = 2:120; % TODO: make this more flexible
                clear freqs
            end
            
            %% Run significance tests of spectra against zero
            avgSpects = squeeze(nanmean(spects(:,:,:,:),3)); % average over trials
            avgSpects = avgSpects(:,whichStims,:);
            disp('Testing spectra against zero.')
            for iStim = 1:size(avgSpects,2)
                [~, tempStats] = permtest_t(squeeze(avgSpects(:,iStim,:)));
                tempStats = bwconncomp(tempStats.h); % select connected comps
                hVsZero{iStim} = tempStats.PixelIdxList;
                if plotSpectCIs
                    spectGrandavg = ...
                        squeeze(mean(avgSpects(:,iStim,:)));
                    spectci{iStim} = ...
                        bootci(10000, @mean, avgSpects(:,iStim,:));
                    spectci{iStim} = ...
                        abs((squeeze(spectci{iStim}) - spectGrandavg')');
                end
            end
            
            %% Spectrum plot.
            plotspects = spects(:,whichStims,:,:);
            iColors = stimtype2colors(stimtypes);
            spect_plot_bin = figure('units', 'centimeters',...
                'position', [1 1 4 4], ...
                'Visible', 'on', 'Color', 'w');
            xlabel('Frequency (Hz)');
            ylabel('Induced power change (%)');
            %ylim([-30 200]);
            box off
            hold on
            lines = size(plotspects,2);
            
            if lines > 1
                p = NaN(size(plotspects,4),1);
                for iFreq = 1:size(plotspects,4)
                    p(iFreq) = anova1(squeeze(...
                        nanmean(plotspects(:,:,:,iFreq),3)), ...
                        1:lines, 'off');
                end
            else
                p = ones(size(plotspects,4),1);
            end
            hAnova = fdr_bh(p, 0.05, 'pdep', 'yes');
            hAnova = bwconncomp(hAnova);
            hAnova = hAnova.PixelIdxList;
            
            for iLine = 1:lines
                iStimtype = whichStims(iLine);
                dat = squeeze(plotspects(:,iLine,:,:));
                meanTrial = squeeze(ESI_nanmean(dat, 2));
                spect = ESI_nanmean(meanTrial, 1);
                if ~plotSpectCIs
                    plot(freqsAxis, spect, 'Color', iColors(iStimtype,:),...
                        'LineWidth', 1);
                else
                    boundedline(freqsAxis, spect,...
                        spectci{iLine}, 'cmap', iColors(iStimtype,:))
                end
            end
            yline(0,':');
            
            linestep = -1.2;
            sigline_y = ylim;
            maxYlim = max(sigline_y);
            sigline_y = sigline_y(1);
            ylim([sigline_y + linestep, maxYlim]);
            % ANOVA significance bars
            for iSig = 1:length(hAnova)
                plot(freqsAxis(hAnova{iSig}), ...
                    repmat(sigline_y, length(hAnova{iSig}),1),...
                    's', 'MarkerFaceColor', 'k', ...
                    'MarkerEdgeColor', 'none', 'MarkerSize', 1);
            end
            
            if plotAllSigbars
                % significance bars against zero
                for iLine = 1:lines
                    thisH = hVsZero{iLine};
                    sigline_y = ylim;
                    maxYlim = max(sigline_y);
                    sigline_y = sigline_y(1);
                    ylim([sigline_y + linestep, maxYlim]);
                    for iSig = 1:length(thisH)
                        plot(freqsAxis(thisH{iSig}), ...
                            repmat(sigline_y, length(thisH{iSig}),1),...
                            's', 'MarkerFaceColor', iColors(iLine,:), ...
                            'MarkerEdgeColor', 'none', 'MarkerSize', 1);
                    end
                end
            end
            
            export_fig(spect_plot_bin, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                ['Stim_comp' num2str(whichStims)]),...
                '-png', '-q101', '-r600');
            
            export_fig(spect_plot_bin, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                ['Stim_comp' num2str(whichStims)] ),...
                '-pdf', '-painters');
            
            %% Per-subject spectra
            subj_spect_plot = figure('units', 'centimeters',...
                'position', [1 1 20 29], ...
                'Visible', 'on', 'Color', 'w');
            
            iSpects = squeeze(nanmean(spects(:,whichStims,:,:),3));
            iColors = stimtype2colors(stimtypes);
            for iSubj = 1:size(iSpects,1)
               subplot(6,5,iSubj)
               hold on
               for iLine = 1:size(iSpects,2)
                   iStim = whichStims(iLine);
                   plot(freqsAxis, squeeze(iSpects(iSubj,iLine,:)), ...
                       'Color', iColors(iStim,:))
               end
               yline(0, '.')
               xlabel('Frequency (Hz)');
               ylabel('Induced power change (%)');
               box off
            end
            
            export_fig(subj_spect_plot, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                ['all_subjs_' num2str(whichStims)]),...
                '-pdf', '-painters');
            

            subj_spect_plot2 = figure('units', 'centimeters',...
                'position', [1 1 18 10], ...
                'Visible', 'on', 'Color', 'w');
            for iLine = 1:size(iSpects,2)
                subplot(ceil(length(whichStims)/4),4,iLine)
                hold on
                iStim = whichStims(iLine);
                plot(freqsAxis, squeeze(iSpects(:,iLine,:)), ...
                    'Color', iColors(iStim,:))
                yline(0, '.')
                xlabel('Frequency (Hz)');
                ylabel('Induced power change (%)');
                box off
            end
            
            export_fig(subj_spect_plot2, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                ['all_subjs2_' num2str(whichStims)]),...
                '-pdf', '-painters');
            
            %% Spectrum plot - aligned to 1st gamma peak
            halfwindow = 10; % window size for frequency alignment
            for iSubj = 1:size(avgSpects,1)
                % find per-stimulus gamma peak
                gpeak = peaks(iSubj,whichStims,2,1);
                if any(isnan(gpeak)) && ~all(isnan(gpeak))
                    % average within subject
                    gpeak(isnan(gpeak)) = nanmean(gpeak);
                elseif all(isnan(gpeak))
                    % average over subjects
                    gpeak = nanmean(peaks(:,whichStims,2,1));
                end
                
                for iStim = 1:length(gpeak)
                    % get power around the peak
                    [~,closestFreq] = min(abs(gpeak(iStim)-freqsAxis));
                    lower = closestFreq - halfwindow;
                    upper = closestFreq + halfwindow;
                    alignedSpec(iSubj,iStim,:) =...
                        avgSpects(iSubj,iStim,lower:upper);
                end
            end
            
            iColors = stimtype2colors(stimtypes);
            spect_plot_aligned = figure('units', 'centimeters',...
                'position', [1 1 4 4], ...
                'Visible', 'on', 'Color', 'w');
            xlabel('Rel. frequency (Hz)');
            ylabel('Induced power change (%)');
            %ylim([-30 200]);
            box off
            hold on
            
            for iLine = 1:length(whichStims)
                iStimtype = whichStims(iLine);
                spect = squeeze(nanmean(alignedSpec(:,iLine,:), 1));
                if ~plotSpectCIs
                    plot(-halfwindow:halfwindow, spect, 'Color', iColors(iStimtype,:),...
                        'LineWidth', 1);
                else
                    spectci = ...
                        bootci(10000, @(x) squeeze(nanmean(x)),...
                        alignedSpec(:,iLine,:));
                    spectci = abs(spectci - spect.');
                    boundedline(-halfwindow:halfwindow, spect,...
                        spectci.', 'cmap', iColors(iStimtype,:))
                end
            end
            xline(0,':');
            
            export_fig(spect_plot_aligned, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                ['Stim_comp_aligned' num2str(whichStims)]),...
                '-png', '-q101', '-r600');
            
            export_fig(spect_plot_aligned, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                ['Stim_comp_aligned' num2str(whichStims)] ),...
                '-pdf', '-painters');
            
            %% Single stimulus spectra
            spect_singles = figure('units', 'centimeters',...
                'position', [1 1 4*10 4], ...
                'Visible', 'on', 'Color', 'w');
            for iStimtype = 1:length(stimtypes)
                
                thisStim = stimtypes{iStimtype};
                switch thisStim
                    case 'grating'
                        iColor = [0 0 0];
                        sTitle = 'Grating';
                    case 'contrast'
                        iColor = [100 100 100];
                        sTitle = 'Contrast';
                    otherwise
                        iColor = cellfun(@str2num, strsplit(thisStim, '_'));
                        sTitle = 'Color';
                end
                
                subplot(1,10,iStimtype)
                dat = squeeze(spects(:,iStimtype,:,:));
                meanTrial = squeeze(nanmean(dat, 2));
                spect = mean(meanTrial, 1);
                ci = bootci(1000, @mean, meanTrial);
                ci = abs(ci-spect);
                
                boundedline(freqsAxis, spect,...
                    ci', 'cmap', iColor./255)
                xlabel('Frequency (Hz)');
                ylabel('Power change (%)');
                title(sTitle)
                h = get(gca, 'Title');
                h.Color = iColor/255;
                %ylim([-30 200]);
                box off
                hold on
                yline(0,':');
            end
            
            export_fig(spect_singles, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                'Stim_comp_all'),...
                '-png', '-q101', '-r600');
            
            export_fig(spect_singles, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                'Stim_comp_all'),...
                '-pdf', '-painters');
            
            %% Single stimulus TFRs
            TFR_singles = figure('units', 'centimeters',...
                'position', [1 1 7*4 6*3], ...
                'Visible', 'on', 'Color', 'w');
            for iStimtype = 1:length(stimtypes)
                
                thisStim = stimtypes{iStimtype};
                switch thisStim
                    case 'grating'
                        iColor(iStimtype,:) = [0 0 0];
                        sTitle = 'Grating';
                    case 'contrast'
                        iColor(iStimtype,:) = [100 100 100];
                        sTitle = 'Contrast';
                    otherwise
                        iColor(iStimtype,:) = cellfun(@str2num, ...
                            strsplit(thisStim, '_'));
                        sTitle = 'Color';
                end
                
                conf = [];
                conf.toilim = [-1 1.3-1/1200];
                conf.foilim = [1 120];
                TFR = ft_freqgrandaverage(conf, TFRs{iStimtype,:});
                
                % The finer time and frequency axes:
                tim_interp = linspace(min(TFR.time), max(TFR.time), 1000);
                freq_interp = linspace(min(TFR.freq), max(TFR.freq), 1000);
                
                [tim_grid_orig, freq_grid_orig] = meshgrid(TFR.time, TFR.freq);
                [tim_grid_interp, freq_grid_interp] = meshgrid(tim_interp, freq_interp);
                
                % interpolate onto higher resolution
                TFR.powspctrm(isnan(TFR.powspctrm)) = 0;
                TFR_interp(:,:,iStimtype) = ...
                    interp2(tim_grid_orig, freq_grid_orig,...
                    squeeze(pow2db(TFR.powspctrm)),...
                    tim_grid_interp, freq_grid_interp, 'linear');
                TFR_interp(TFR_interp == -Inf) = NaN;
                
                subplot(3,4,iStimtype)
                
                imagesc(TFR.time, TFR.freq, TFR_interp(:,:,iStimtype));
                set(gca, 'YDir', 'normal');
                set(gca, 'TickDir', 'out');
                set_colormap('div');
                h = colorbar;
                ylabel(h, 'Induced power change (dB)');
                h = yline(20.5, 'k', 'LineWidth', 3);
                h.Color = [0,0,0,1];
                xlabel('Time (s)');
                
                ylabel('Frequency (Hz)');
                xlim([-0.3 1.3])
                box off
                title(sTitle)
                
                h = get(gca, 'Title');
                h.Color = iColor(iStimtype,:)/255;
            end
            
            export_fig(TFR_singles, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                'TFR_comp_all_full'),...
                '-png', '-q101', '-r600');
            
            export_fig(TFR_singles, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                'TFR_comp_all_full'),...
                '-pdf', '-painters');
            
            %% Stimulus-average TFRs
            figure('unit', 'centimeter', 'position', [0,0,6,4])
            imagesc(TFR.time, TFR.freq, mean(TFR_interp(:,:,1:8),3))
            set(gca, 'YDir', 'normal');
            set(gca, 'TickDir', 'out');
            set_colormap('div');
            h = colorbar;
            ylabel(h, 'Induced power change (dB)');
            h = yline(20.5, 'k', 'LineWidth', 3);
            h.Color = [0,0,0,1];
            xlabel('Time (s)');
            ylabel('Frequency (Hz)');
            xlim([-0.3 1.3])
            ylim([2 120]);
            yticks(0:20:120)
            box off
            export_fig(fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                'TFR_colormean'),...
                '-png', '-q101', '-r600');
            export_fig(fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                'TFR_colormean'),...
                '-pdf', '-painters');
            
            figure('unit', 'centimeter', 'position', [0,0,6,4])
            imagesc(TFR.time, TFR.freq, mean(TFR_interp(:,:,10),3))
            set(gca, 'YDir', 'normal');
            set(gca, 'TickDir', 'out');
            set_colormap('div');
            h = colorbar;
            ylabel(h, 'Induced power change (dB)');
            h = yline(20.5, 'k', 'LineWidth', 3);
            h.Color = [0,0,0,1];
            xlabel('Time (s)');
            ylabel('Frequency (Hz)');
            xlim([-0.3 1.3])
            ylim([2 120]);
            yticks(0:20:120)
            box off
            export_fig(fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                'TFR_gratingmean'),...
                '-png', '-q101', '-r600');
            export_fig(fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                'TFR_gratingmean'),...
                '-pdf', '-painters');
            
            %% Peak fitting results
            % transform data into plot compatible format
            peaks = reshape(peaks, size(peaks,1), size(peaks,2),6);
            ipeaks = peaks(:,whichStims,:);
            measures = {'Amplitude peak 1', 'Frequency peak 1',...
                'Spread peak 1', 'Amplitude peak 2', 'Frequency peak 2',...
                'Spread peak 2'};
            
            stimGroups = {[1,8], [4,5], [2,3,6,7]}; %RG, BY, off-axis
            for iMeasure = 1:length(measures)
                for iStim = 1:size(ipeaks,2)
                    stimMean(iStim,iMeasure) = ...
                        nanmean(ipeaks(:,iStim,iMeasure));
                    stimErr(iStim,iMeasure,:) = ...
                        bootci(500, {@nanmean, ipeaks(:,iStim,iMeasure)});
                    stimErr(iStim,iMeasure,:) = ...
                        abs(stimErr(iStim,iMeasure,:) - ...
                        stimMean(iStim,iMeasure));
                end
                for iGroup = 1:length(stimGroups)
                    thisDat = ipeaks(:,stimGroups{iGroup},iMeasure);
                    groupMean(iGroup,iMeasure) = ...
                        nanmean(thisDat(:));
                    groupErr(iGroup,iMeasure,:) = ...
                        bootci(500, {@nanmean, thisDat(:)});
                    groupErr(iGroup,iMeasure,:) = ...
                        abs(groupErr(iGroup,iMeasure,:) - ...
                        groupMean(iGroup,iMeasure));
                end
            end
            
            x = 1:length(whichStims);
            cBars = figure(...
                'Unit', 'centimeter', 'Position', [0 0, 25, 20]);
            for iMeasure =  1:length(measures)
                subplot(2,3,iMeasure)
                b = bar(x,squeeze(stimMean(:,iMeasure)), 'EdgeColor', 'none');
                b.FaceColor = 'flat';
                b.CData = iColor(whichStims,:)./255;
                hold on
                errorbar(x, stimMean(:,iMeasure), ...
                    stimErr(:,iMeasure,1), stimErr(:,iMeasure,2),...
                    'Color', 'k', 'LineStyle', 'none', 'CapSize', 0);
                box off
                title(measures{iMeasure});
            end
            
            export_fig(cBars, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                'fit_spectra'), '-pdf', '-painters');
            
            export_fig(cBars, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                'fit_spectra'), '-png', '-q101', '-r600');
            
            x = 1:3;
            cBars2 = figure(...
                'Unit', 'centimeter', 'Position', [0 0, 25, 20]);
            for iMeasure =  1:length(measures)
                subplot(2,3,iMeasure)
                b = bar(x,squeeze(groupMean(:,iMeasure)), 'EdgeColor', 'none');
                b.FaceColor = 'flat';
                xticklabels({'RG', 'BY', 'Off'});
                hold on
                errorbar(x, groupMean(:,iMeasure), ...
                    groupErr(:,iMeasure,1), groupErr(:,iMeasure,2),...
                    'Color', 'k', 'LineStyle', 'none', 'CapSize', 0);
                box off
                title(measures{iMeasure});
            end
            
            export_fig(cBars2, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                'fit_spectra_folded'), '-pdf', '-painters');
            
            export_fig(cBars2, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                'fit_spectra_folded'), '-png', '-q101', '-r600');
            
            %% Correlate first and second peaks
            colorStims = ~ismember(stimtypes, {'contrast', 'grating'});
            colorColors = iColor(colorStims,:)./255;
            %doublePeakSubj = contains(obj.subjects, '24');
            peak1 = peaks(:,colorStims,2);
            peak2 = peaks(:,colorStims,5);
            
            peakCorr = figure(...
                'Unit', 'centimeter', 'Position', [0 0, 4, 4]);
            hold on
            for iSubj = 1:size(peak1,1)
                assert(all(~(peak2(iSubj,:) - peak1(iSubj,:) <0)))
                scatter(peak1(iSubj,:), peak2(iSubj,:), 5, colorColors, 'filled', 'o')
            end
            line([25 70], [50 140], 'Color', [0.7 0.7 0.7], 'LineStyle', '-');
            xlim([25 70])
            ylim([50 140])
            xlabel('1st peak (Hz.)')
            ylabel('2nd peak (Hz.)')
            
            % compute correlation
            peak1c = peak1(~isnan(peak1) & ~isnan(peak2));
            peak2c = peak2(~isnan(peak1) & ~isnan(peak2));
            [r, p] = corr(peak1c, peak2c)
            
            export_fig(peakCorr, ...
                fullfile(obj.group_cfg.paths.result, 'Spectra', ...
                'first_vs_second_peak'), '-pdf');
            
        end
        
        function ERFs(obj, whichStims, plotAllSigbars)
            % Compute color and grating ERFs per subject, compare them.
            % whichStims determines which stimuli are plotted - chose 1:8
            % for colors, 9 for contrast, 10 for grating
            % plotAllSigbars determines if significances against zero are
            % plotted for all stimuli.
            if ~exist('plotAllSigbars', 'var')||isempty(plotAllSigbars)
                plotAllSigbars = false;
            end
            
            disp('Running per-participant ERF analysis on SLURM.')
            if ~obj.group_cfg.general.rerunAnalyses
                ESIload(obj.group_cfg.paths.ERFdata);
                if ~exist('whichStims', 'var')
                    whichStims = [1:8 10];
                end
                stimtypes = stimOrder(1,:);
            else
                out = slurmfun(@slurmGetERFs, obj.subjects,...
                    'partition', '48GBL', 'stopOnError', false);
                
                disp('Collecting results...')
                nSubj = length(obj.subjects);
                [nTimes, nStim] = size(out{1}.erf);
                if ~exist('whichStims', 'var'), whichStims = 1:nStim; end
                
                ERF = NaN(nSubj, nStim, nTimes);
                stimOrder = cell(nSubj, nStim);
                time = NaN(nSubj, nTimes);
                for iSubj = 1:length(obj.subjects)
                    ERF(iSubj,:,:) = out{iSubj}.erf';
                    stimOrder(iSubj,:) = out{iSubj}.stimtypes';
                    time(iSubj,:) = out{iSubj}.time;
                end
                assert(size(unique(...
                    double(cell2mat(stimOrder)), 'rows'), 1) == 1, ...
                    'stimOrder is mixed up over subjects!')
                stimtypes = stimOrder(1,:);
                assert(max(abs(diff(time)),[],'all') < 1e-15, ...
                    'Time vector of subjects is different!');
                time = median(time); % get rid of floating point error
                
                disp('Saving intermediary results to disk.')
                ESIsave(obj.group_cfg.paths.ERFdata,...
                    'ERF', 'stimOrder', 'time');
            end
            %% Average ERF for Fig. 1
            disp('Plotting avg. ERF.')
            avgERF = squeeze(mean(ERF(:,whichStims,:),2)); % avg. over stimuli
            ERFgrandavg = mean(avgERF);
            ERFci = bootci(10000, @mean, avgERF);
            ERFci = abs((ERFci - ERFgrandavg)');
            
            figure('units', 'centimeters','position', [1 1 5 4], ...
                'Visible', 'on', 'Color', 'w');
            boundedline(time, ERFgrandavg, ERFci,...
                'cmap', myColors(1, 'bright'));
            xlabel('Time (s)');
            ylabel('Dipole moment (z-scored)')
            xline(0, ':');
            yline(0, ':');
            %xlim([min(time)-0.001 max(time)]);
            xlim([-0.1 0.4]);
            xticks(-0.1 :0.1: 0.4);
            box off
            
            plotpath = fullfile(obj.group_cfg.paths.result, ...
                'ERFs', 'avg_ERF');
            export_fig(plotpath, '-pdf', '-painters')
            export_fig(plotpath, '-png', '-q101', '-r600')
            
            
            %% ERF per stimulus type
            disp('Plotting per-stimulus ERF.')
            figure('units', 'centimeters','position', [1 1 5 4], ...
                'Visible', 'on', 'Color', 'w');
            hold on
            iColors = stimtype2colors(stimtypes);
            
            for iStim = whichStims
                ERFgrandavg = mean(squeeze(ERF(:,iStim,:)));
                ERFci = bootci(1000, @mean, ...
                    squeeze(ERF(:,iStim,:)));
                ERFci = abs((ERFci - ERFgrandavg)');
                
                boundedline(time, ERFgrandavg, ERFci,...
                    'cmap', iColors(iStim,:));
                xlabel('Time (s)');
                ylabel('Dipole moment (z-scored)')
                xline(0, ':');
                yline(0, ':');
                xlim([-0.1 0.4]);
                xticks(-0.1 :0.1: 0.4);
                box off
            end
            
            plotpath = fullfile(obj.group_cfg.paths.result, ...
                'ERFs', 'ERF_per_stimtype');
            export_fig(plotpath, '-pdf', '-painters')
            export_fig(plotpath, '-png', '-q101', '-r600')
            
            %% ERF per stimulus type without error bars
            % compute where ERF differs by stimulus type
            tlim = [-0.1 0.4];
            timelim = time(time >= tlim(1) & time <= tlim(2));
            ERFlim = ERF(:,:,(time >= tlim(1) & time <= tlim(2)));
            
            % Run significance tests of ERFs against zero
            for iStim = 1:size(ERFlim,2)
                [~, tempStats] = permtest_t(squeeze(ERFlim(:,iStim,:)));
                tempStats = bwconncomp(tempStats.h); % select connected comps
                hVsZero{iStim} = tempStats.PixelIdxList;
            end
            
            ERFlim = ERFlim(:,whichStims,:);
            if size(ERFlim,2) > 1
                for iTime = 1:size(ERFlim,3)
                    p(iTime) = anova1(ERFlim(:, :, iTime), ...
                        1:size(ERFlim,2), 'off');
                end
            else
                p = ones(size(ERFlim,2));
            end
            hAnova = fdr_bh(p, 0.05, 'dep', 'yes');
            hAnova = bwconncomp(hAnova);
            hAnova = hAnova.PixelIdxList;
            
            disp('Plotting per-stimulus ERF - plot 2.')
            figure('units', 'centimeters','position', [1 1 5 4], ...
                'Visible', 'on', 'Color', 'w');
            hold on
            for iwStim = 1:length(whichStims)
                iStim = whichStims(iwStim);
                ERFgrandavg = mean(squeeze(ERFlim(:,iwStim,:)));
                plot(timelim, ERFgrandavg, 'Color', iColors(iStim,:));
                xlabel('Time (s)');
                ylabel('Dipole moment (z-scored)')
                xline(0, ':');
                yline(0, ':');
                xlim([-0.1 0.4]);
                xticks(-0.1 :0.1: 0.4);
                box off
            end
            
            sigline_y = ylim;
            maxYlim = max(sigline_y);
            sigline_y = sigline_y(1);
            linestep = -0.045;
            ylim([sigline_y + linestep, maxYlim]);
            for iSig = 1:length(hAnova)
                plot(timelim(hAnova{iSig}), ...
                    repmat(sigline_y, length(hAnova{iSig}),1),...
                    's', 'MarkerFaceColor', 'k', ...
                    'MarkerEdgeColor', 'none', 'MarkerSize', 1);
                disp('Significant differences at:')
                disp(timelim(hAnova{iSig}))
            end
            
            if plotAllSigbars
                % significance bars against zero
                for iLine = whichStims
                    thisH = hVsZero{iLine};
                    sigline_y = ylim;
                    maxYlim = max(sigline_y);
                    sigline_y = sigline_y(1);
                    ylim([sigline_y + linestep, maxYlim]);
                    for iSig = 1:length(thisH)
                        plot(timelim(thisH{iSig}), ...
                            repmat(sigline_y, length(thisH{iSig}),1),...
                            's', 'MarkerFaceColor', iColors(iLine,:), ...
                            'MarkerEdgeColor', 'none', 'MarkerSize', 1);
                    end
                end
            end
            
            plotpath = fullfile(obj.group_cfg.paths.result, ...
                'ERFs', 'ERF_per_stimtype_no_errorbars');
            export_fig(plotpath, '-pdf', '-painters')
            export_fig(plotpath, '-png', '-q101', '-r600')
            
            %% Bar plots of C1 ERF size
            % window with significant differences between colors
            tlower = obj.group_cfg.erfplot.ERFcomps(1);
            thigher = obj.group_cfg.erfplot.ERFcomps(2);
            timeselect = timelim >= tlower & timelim <= thigher;
            ERFheight = mean(ERFlim(:,:,timeselect),3);
            
            for iStim = 1:size(ERFheight,2)
                stimMean(iStim) = ...
                    nanmean(ERFheight(:,iStim));
                stimErr(iStim,:) = ...
                    bootci(500, {@nanmean, ERFheight(:,iStim)});
                stimErr(iStim,:) = ...
                    abs(stimErr(iStim,:) - ...
                    stimMean(iStim));
            end
            
            cBars = figure('renderer', 'Painters',...
                'Unit', 'centimeters', 'Position', [0 0, 6, 6]);
            b = bar(whichStims, stimMean, 'EdgeColor', 'k');
            b.FaceColor = 'flat';
            b.CData = iColors(whichStims,:);
            hold on
            errorbar(whichStims, stimMean, ...
                stimErr(:,1), stimErr(:,2),...
                'Color', 'k', 'LineStyle', 'none');
            box off
            
            export_fig(cBars, ...
                fullfile(obj.group_cfg.paths.result, 'ERFs', ...
                'C1_height_unscaled'), '-pdf', '-eps', '-painters');
            
            export_fig(cBars, ...
                fullfile(obj.group_cfg.paths.result, 'ERFs', ...
                'C1_height'), '-png', '-q101', '-r600');
        end
        
        function erPupils(obj, whichStims)
            % Compute color and grating pupil traces per subject.
            % whichStims determines which stimuli are plotted - chose 1:8
            % for colors, 9 for contrast, 10 for grating
            % plotAllSigbars determines if significances against zero are
            % plotted for all stimuli.
            
            disp('Running per-participant ERpupil analysis on SLURM.')
            out = slurmfun(@slurmStimPupil, obj.subjects,...
                'partition', '8GBS', 'stopOnError', true);
            
            disp('Collecting results...')
            nSubj = length(obj.subjects);
            [nStim, nTimes] = size(out{1}.avgPupil);
            if ~exist('whichStims', 'var'), whichStims = 1:nStim; end
            
            ERPupils = NaN(nSubj, nStim, nTimes);
            stimOrder = cell(nSubj, nStim);
            time = NaN(nSubj, nTimes);
            for iSubj = 1:length(obj.subjects)
                ERPupils(iSubj,:,:) = out{iSubj}.avgPupil;
                stimOrder(iSubj,:) = out{iSubj}.stimtypes';
                time(iSubj,:) = out{iSubj}.time;
            end
            assert(size(unique(...
                double(cell2mat(stimOrder)), 'rows'), 1) == 1, ...
                'stimOrder is mixed up over subjects!')
            stimtypes = stimOrder(1,:);
            assert(max(abs(diff(time)),[],'all') < 1e-15, ...
                'Time vector of subjects is different!');
            time = median(time); % get rid of floating point error
            
            %% Average ERF for Fig. 1
            disp('Plotting avg. ERPupil.')
            avgERPp = squeeze(mean(ERPupils(:,whichStims,:),2)); % avg. over stimuli
            ERPpgrandavg = mean(avgERPp);
            ERPpci = bootci(10000, @mean, avgERPp);
            ERPpci = abs((ERPpci - ERPpgrandavg)');
            
            figure('units', 'centimeters','position', [1 1 5 4], ...
                'Visible', 'on', 'Color', 'w');
            boundedline(time, ERPpgrandavg, ERPpci,...
                'cmap', myColors(1, 'bright'));
            xlabel('Time (s)');
            ylabel('Pupil size (a.u.)')
            xline(0, ':');
            yline(0, ':');
            %xlim([min(time)-0.001 max(time)]);
            xlim([-0.1 1.3]);
            box off
            
            plotpath = fullfile(obj.group_cfg.paths.result, ...
                'Pupil', 'avg_ERPupil');
            export_fig(plotpath, '-pdf', '-painters')
            export_fig(plotpath, '-png', '-q101', '-r600')
            
            
            %% ERPupil per stimulus type
            disp('Plotting per-stimulus ERPupil.')
            figure('units', 'centimeters','position', [1 1 5 4], ...
                'Visible', 'on', 'Color', 'w');
            hold on
            iColors = stimtype2colors(stimtypes);
            
            for iStim = whichStims
                ERPpgrandavg = mean(squeeze(ERPupils(:,iStim,:)));
                ERPpci = bootci(1000, @mean, ...
                    squeeze(ERPupils(:,iStim,:)));
                ERPpci = abs((ERPpci - ERPpgrandavg)');
                
                boundedline(time, ERPpgrandavg, ERPpci,...
                    'cmap', iColors(iStim,:));
                xlabel('Time (s)');
                ylabel('Pupil sisze (a.u.)')
                xline(0, ':');
                yline(0, ':');
                xlim([-0.1 1.3]);
                box off
            end
            
            plotpath = fullfile(obj.group_cfg.paths.result, ...
                'Pupil', 'ERPupil_per_stimtype');
            export_fig(plotpath, '-pdf', '-painters')
            export_fig(plotpath, '-png', '-q101', '-r600')
            
            %% ERPupil per stimulus type without error bars
            % compute where ERPupils differs by stimulus type
            tlim = [-0.1 1.3];
            timelim = time(time >= tlim(1) & time <= tlim(2));
            ERPpLim = ERPupils(:,:,(time >= tlim(1) & time <= tlim(2)));
            
            % Run significance test of ERPupils against each other
            ERPpLim = ERPpLim(:,whichStims,:);
            if size(ERPpLim,2) > 1
                for iTime = 1:size(ERPpLim,3)
                    p(iTime) = anova1(ERPpLim(:, :, iTime), ...
                        1:size(ERPpLim,2), 'off');
                end
            else
                p = ones(size(ERPpLim,2));
            end
            hAnova = fdr_bh(p, 0.05, 'dep', 'yes');
            hAnova = bwconncomp(hAnova);
            hAnova = hAnova.PixelIdxList;
            
            disp('Plotting per-stimulus ERPupils - plot 2.')
            figure('units', 'centimeters','position', [1 1 5 4], ...
                'Visible', 'on', 'Color', 'w');
            hold on
            for iwStim = 1:length(whichStims)
                iStim = whichStims(iwStim);
                ERPpgrandavg = mean(squeeze(ERPpLim(:,iwStim,:)));
                plot(timelim, ERPpgrandavg, 'Color', iColors(iStim,:));
                xlabel('Time (s)');
                ylabel('Pupil size (a.u.)')
                xline(0, ':');
                yline(0, ':');
                box off
            end
            
            sigline_y = ylim;
            maxYlim = max(sigline_y);
            sigline_y = sigline_y(1);
            linestep = -0.045;
            ylim([sigline_y + linestep, maxYlim]);
            for iSig = 1:length(hAnova)
                plot(timelim(hAnova{iSig}), ...
                    repmat(sigline_y, length(hAnova{iSig}),1),...
                    's', 'MarkerFaceColor', 'k', ...
                    'MarkerEdgeColor', 'none', 'MarkerSize', 1);
            end
            
            plotpath = fullfile(obj.group_cfg.paths.result, ...
                'Pupil', 'ERPp_per_stimtype_no_errorbars');
            export_fig(plotpath, '-pdf', '-painters')
            export_fig(plotpath, '-png', '-q101', '-r600')
        end
    end
end

