classdef Preprocessing < handle
    % PREPROCESSING Run preprocessing functions on RepUp-Data
    % Loads raw MEG data and behavioral data from disk, runs artefact
    % detection, creates Quality Control plots and saves them to disk.
    
    properties
        cfg
        subject
        data_cleaned
        behavior
        artifact_jump % artefact sample points
        artifact_muscle
        artifact_eog
        data_add
        data_eye
        data_meg
        trials = [];
        grad = [];
    end
    
    methods
        function obj = Preprocessing(subj_nr)
            %% Constructor. Add folders to path, load settings.
            [obj.cfg, obj.subject] = setup(subj_nr);
        end
        
        function obj = loadRaw(obj)
            %% Read raw data from disk
            fprintf('Reading raw data for subj. %s.\n', obj.subject.subj_nr);
            % Do the same for all recorded blocks, then merge.
            for iDataset = 1:length(obj.subject.megdata)
                % Define trials
                dataset = obj.subject.megdata{iDataset};
                obj.cfg.trials.dataset = dataset;
                obj.trials{iDataset} = ft_definetrial(obj.cfg.trials);
                obj.cfg.preprocessing.trl = obj.trials{iDataset}.trl;
                obj.cfg.otherpreproc.trl = obj.trials{iDataset}.trl;
                
                % Run preprocessing, seperately for eyetracker and
                % photodiode channels.
                obj.cfg.preprocessing.dataset = dataset;
                obj.cfg.otherpreproc.dataset = dataset;
                
                % Read continous data, filter line noise
                data_meg_trials{iDataset} = ft_preprocessing(...
                    obj.cfg.preprocessing);
                % Read eyetracking data + photodiode data
                data_add_trials{iDataset} = ft_preprocessing(...
                    obj.cfg.otherpreproc);
                
                if obj.cfg.general.control_headmovements
                    disp('Controlling for headmotion effects on grad coordinate system.')
                    warning(['This function still needs work to ' ...
                        ' combine grads over trials!'])
                    conf            = [];
                    conf.dataset    = dataset;
                    conf.trl        = obj.trials{iDataset}.trl;
                    conf.method     = 'avgoverrpt';
                    conf.feedback   = false;
                    controlled = ft_headmovement(conf);
                    controlled.grad.coordsys = 'ctf';
                    data_meg_trials{iDataset}.grad = controlled.grad;
                    data_add_trials{iDataset}.grad = controlled.grad;
                end
            end
            
            obj.grad = data_meg_trials{1}.grad;
            
            % Correct trial numbers, merge blocks into one dataset
            nTrialsBlock1 = ...
                obj.subject.stim_cfg{1}.experiment.nTrialsGrating + ...
                obj.subject.stim_cfg{1}.experiment.nTrialsColor;
            nTrialsBlock12 = nTrialsBlock1 +  ...
                obj.subject.stim_cfg{2}.experiment.nTrialsGrating + ...
                obj.subject.stim_cfg{2}.experiment.nTrialsColor;
            data_meg_trials{2}.trialinfo = ...
                data_meg_trials{2}.trialinfo + nTrialsBlock1;
            data_add_trials{2}.trialinfo = ...
                data_add_trials{2}.trialinfo + nTrialsBlock1;
            try
                data_meg_trials{3}.trialinfo = ...
                    data_meg_trials{3}.trialinfo + nTrialsBlock12;
                data_add_trials{3}.trialinfo = ...
                    data_add_trials{3}.trialinfo + nTrialsBlock12;
            catch
                missingThirdBlock = true;
                warning('No data for third block found!');
            end
            obj.behavior = obj.subject.behavior;
            
            conf = [];
            conf.keepsampleinfo = 'no';
            if ~exist('missingThirdBlock', 'var') || ~missingThirdBlock
                obj.data_meg = ft_appenddata(conf, data_meg_trials{1}, ...
                    data_meg_trials{2}, data_meg_trials{3});
                obj.data_add = ft_appenddata(conf, data_add_trials{1}, ...
                    data_add_trials{2}, data_add_trials{3});
            elseif missingThirdBlock
                obj.data_meg = ft_appenddata(conf, data_meg_trials{1}, ...
                    data_meg_trials{2});
                obj.data_add = ft_appenddata(conf, data_add_trials{1}, ...
                    data_add_trials{2});
            end
            
            % reconstruct sampleinfo
            obj.data_meg = ft_checkdata(obj.data_meg, 'hassampleinfo', 'yes');
            obj.data_add = ft_checkdata(obj.data_add, 'hassampleinfo', 'yes');
            
        end
        
        function detectArtefacts(obj)
            assert(~isempty(obj.data_meg), 'loadRaw() must be run first!');
            if obj.cfg.general.do_manual_artifact_rejection
                obj.manualArtefactDetection();
            elseif obj.cfg.general.do_saved_artifact_rejection
                obj.savedArtefactDetection();
            end
            
        end
        
        function selectCleanChans(obj)
            % Select clean channels
            channels.method      = 'summary';
            channels.keepchannel = 'no';
            channels.channel     = {'MEG'};
            obj.data_meg = ft_rejectvisual(channels, obj.data_meg);
            % Select active eyetracker channels
            channels.channel     = obj.cfg.eyechannels;
            obj.data_eye = ft_rejectvisual(channels, obj.data_add);
            obj.subject.cleanMEGChans = obj.data_meg.label;
            obj.subject.cleanEyeChans = obj.data_eye.label;
            
            if obj.cfg.general.save_partial_results
                disp('Saving clean channels to disk.');
                cleanMEGChans = obj.subject.cleanMEGChans;
                ESIsave(fullfile(obj.subject.preproc_data, ...
                    'cleanMEGChans.mat'), 'cleanMEGChans');
                cleanEyeChans = obj.subject.cleanEyeChans;
                ESIsave(fullfile(obj.subject.preproc_data, ...
                    'cleanEyeChans.mat'), 'cleanEyeChans');
            end
        end
        
        function obj = manualArtefactDetection(obj)
            %% Employ artefact detection on raw data.
            % Select clean channels
            obj.selectCleanChans();
            
            disp('Loading saved clean channel lists.')
            ESIload(fullfile(obj.subject.preproc_data, ...
                'cleanMEGChans.mat'));
            ESIload(fullfile(obj.subject.preproc_data, ...
                'cleanEyeChans.mat'));
            
            % Muscle artifacts
            disp('Showing muscle artefacts.');
            [obj.cfg.muscledetect.settings, artifact_muscle] = ...
                ft_artifact_zvalue(obj.cfg.muscledetect, obj.data_meg);
            
            % Jump artefacts
            disp('Showing jump artefacts.');
            [obj.cfg.jumpdetect.settings, artifact_jump] = ...
                ft_artifact_zvalue(obj.cfg.jumpdetect, obj.data_meg);
            
            % Blinks
            disp('Showing blink artefacts.');
            obj.cfg.blinkdetect.artfctdef.zvalue.channel = ...
                intersect(obj.cfg.blinkdetect.artfctdef.zvalue.channel, ...
                cleanEyeChans);
            [obj.cfg.blinkdetect.settings, artifact_eog] = ...
                ft_artifact_zvalue(obj.cfg.blinkdetect, obj.data_eye);
            
            % If requested: save artefact timecodes
            if obj.cfg.general.save_partial_results
                disp('Saving artefacts to subject struct.');
                ESIsave(fullfile(obj.subject.path, 'artifact_jump.mat'),...
                    'artifact_jump');
                ESIsave(fullfile(obj.subject.path, 'artifact_muscle.mat'),...
                    'artifact_muscle');
                ESIsave(fullfile(obj.subject.path, 'artifact_eog.mat'),...
                    'artifact_eog');
            end
            
            obj.artifact_jump = artifact_jump;
            obj.artifact_muscle = artifact_muscle;
            obj.artifact_eog = artifact_eog;
        end
        
        function obj = savedArtefactDetection(obj)
            disp('Using loaded artifact timeponts.');
            ESIload(fullfile(obj.subject.path, 'artifact_jump.mat'));
            ESIload(fullfile(obj.subject.path, 'artifact_muscle.mat'));
            ESIload(fullfile(obj.subject.path, 'artifact_eog.mat'));
            
            ESIload(fullfile(obj.subject.preproc_data, 'subject.mat'));
            obj.subject = subject;
            
            obj.artifact_jump = artifact_jump;
            obj.artifact_muscle = artifact_muscle;
            obj.artifact_eog = artifact_eog;
        end
        
        function obj = removeArtefacts(obj)
            % Remove manually selected or loaded in artefacts. Must be run
            % after obj.detectArtefacts().
            assert(~isempty(obj.artifact_muscle), ...
                'Load/create artefact definitions first.')
            assert(~isempty(obj.data_meg), 'loadRaw() must be run first!');
            
            % if requested: remove trials without response
            if obj.cfg.general.reject_nonresponses
                % Get nonresponded non-catch trials
                nonresponses = obj.behavior.wasCatch == 0 & ...
                    strcmp(obj.behavior.answer, 'none');
                % Find their trial number, convert to index in meg trial data
                nonresponses = obj.behavior.trialNr(nonresponses);
                nonresponses = ismember(obj.data_meg.trialinfo, nonresponses);
                rejecting.artfctdef.nonresponses.artifact = ...
                    obj.data_meg.sampleinfo(nonresponses,:);
            end
            
            % if requested: remove non-fixated trials
            if obj.cfg.general.reject_nonfixations
                nonfixations = obj.behavior.wasFixated == 0;
                nonfixations = obj.behavior.trialNr(nonfixations);
                nonfixations = ismember(obj.data_meg.trialinfo, nonfixations);
                rejecting.artfctdef.nonfixations.artifact = ...
                    obj.data_meg.sampleinfo(nonfixations,:);
            end
            
            rejecting.artfctdef.reject = obj.cfg.general.rejectTrials;
            rejecting.artfctdef.eog.artifact = obj.artifact_eog;
            rejecting.artfctdef.jump.artifact = obj.artifact_jump;
            rejecting.artfctdef.muscle.artifact = obj.artifact_muscle;
            obj.data_meg = ft_rejectartifact(rejecting, ...
                obj.data_meg);
            obj.data_add = ft_rejectartifact(rejecting, ...
                obj.data_add);
            
            % Put it back together
            obj.data_cleaned = ft_appenddata([], ...
                obj.data_meg, obj.data_add);
            obj.data_cleaned.grad = obj.grad;
            
            % Remove dirty channels, empty eyetracker channels
            if obj.cfg.general.reject_bad_channels
                disp('Loading saved clean channel lists.')
                ESIload(fullfile(obj.subject.preproc_data, ...
                    'cleanMEGChans.mat'));
                ESIload(fullfile(obj.subject.preproc_data, ...
                    'cleanEyeChans.mat'));
                
                to_remove = cat(1, ...
                    setdiff(obj.data_meg.label, cleanMEGChans),...
                    setdiff(obj.cfg.eyechannels', cleanEyeChans));
                to_remove = cellfun(@(x) ['-' x], to_remove, 'UniformOutput', 0);
                channelconf.channel = [{'all'}, to_remove'];
                obj.data_cleaned = ft_selectdata(channelconf, obj.data_cleaned);
            end
            
            if obj.cfg.general.save_partial_results % Save results
                if ~exist(obj.subject.preproc_data,'dir')
                    mkdir(obj.subject.preproc_data); end
                obj.data_cleaned.cfg.preprocessing_settings = obj.cfg;
                data_cleaned = obj.data_cleaned;
                behav_cleaned = obj.behavior;
                ESIsave(fullfile(obj.subject.preproc_data, 'data_cleaned'), 'data_cleaned');
                ESIsave(fullfile(obj.subject.preproc_data, 'behav_cleaned'), 'behav_cleaned');
            end
        end
        
        
        function obj = loadPreprocessed(obj)
            %% Load already cleaned data
            disp('Loading preprocessed data from disk.');
            ESIload(fullfile(obj.subject.preproc_data, 'data_cleaned'));
            ESIload(fullfile(obj.subject.preproc_data, 'behav_cleaned'));
            obj.data_cleaned = data_cleaned;
            obj.behavior = behav_cleaned;
        end
        
        function obj = QCplots(obj)
            %% Create QC plots
            % Movement plot
            qcMovement(obj.cfg.qc_movement, obj.subject, obj.data_cleaned);
            % Diode plot, one per block
            for iDataset = 1:length(obj.subject.megdata)
                dataset = obj.subject.megdata{iDataset};
                obj.cfg.otherpreproc.dataset = dataset;
                if isfield(obj.cfg.otherpreproc, 'trl')
                    obj.cfg.otherpreproc = rmfield(obj.cfg.otherpreproc, 'trl');
                end
                data_cont = ft_preprocessing(...
                    obj.cfg.otherpreproc);
                obj.cfg.trials.dataset = dataset;
                iTrials = ft_definetrial(obj.cfg.trials);
                qcDiode(obj.cfg.qc_diode, obj.subject, data_cont, ...
                    iTrials.event, iDataset);
            end
            % Empty room plot
            qcEmptyroom(obj.cfg.qc_emptyroom, obj.subject);
            qcFrequency(obj.data_cleaned, obj.cfg.qc_frequency, ...
                obj.subject, obj.behavior);
        end
    end
end

