classdef RepEffect < handle
    %REPEFFECT Tests the stimulus-specific gamma repetition enhancement on
    % previously beamformed MEG data.
    %
    % RepEffect Properties:
    %   cfg                 = A config struct as created by fillConfigs().
    %   subject             = A subject struct with subject-specific info and
    %                         already computed analysis results (eg beamformer
    %                         filters, gamma peaks, etc.).
    %   data                = Cleaned, preprocessed, sensorlevel MEG data.
    %   dipole_filters      = LCMV sensors->source filter matrix for each
    %                         dipole.
    %   dipole_timecourses  = Computed source dipole timecourses.
    %   results             = The gamma repetition effect result table.
    %
    % Beamformer Methods:
    %   Beamformer(cfg, subj_nr) = Class constructor.
    %   loadData()               = Load MEG and MRI data from disk.
    %   createSourcemodel()      = Create subject-specific dipole grid.
    properties
        cfg
        subject
        data
        dipole_filters
        dipole_timecourses
        results
        resultsSpect
    end
    
    methods
        function obj = RepEffect(cfg, subj_nr)
            obj.cfg = cfg;
            ESIload(fullfile(cfg.paths.data, num2str(subj_nr, '%03d'), ...
                'Preprocessed', 'subject.mat'));
            obj.subject = subject;
        end
        
        function doEverything(obj) % Run the whole effect pipeline
            computeSource(obj);
            quantifyGammaPower(obj);
            plotResults(obj);
            plotEffectSpectra(obj);
            plotBehavior(obj);
        end
        
        function computeSource(obj)
            %% Load data, apply filters
            fprintf('Computing sources for subject %s.\n', obj.subject.subj_nr);
            ESIload(fullfile(obj.subject.preproc_data, 'data_cleaned'));
            conf.channel = 'meg';
            obj.data = ft_selectdata(conf, data_cleaned);
            
            for iDOI = 1:length(obj.cfg.enhance.doi)
                % Get LCMV filters
                this_dipole  = obj.cfg.enhance.doi{1,iDOI};
                fprintf('Computing dipole %s.\n', this_dipole);
                dipole_pos   = obj.subject.sources.(this_dipole).pos;
                [~, iPos] = intersect(obj.subject.mni_pos, dipole_pos, 'rows');
                obj.subject.sources.(this_dipole).index = iPos;
                obj.dipole_filters{iDOI} = obj.subject.filters{iPos};
                
                % Multiply dipole weights with MEG data
                obj.dipole_timecourses{iDOI} = obj.data;
                weights_multiply = @(weights, trial) mtimes(weights, trial);
                %obj.dipole.Timecourses{iDOI} =
                obj.dipole_timecourses{iDOI}.trial = cellfun(...
                    @(x) weights_multiply(obj.dipole_filters{iDOI},x), ...
                    obj.data.trial, 'UniformOutput', false);
                
                % Name dipole channels
                obj.dipole_timecourses{iDOI}.label = {[this_dipole '_dir1'];...
                    [this_dipole '_dir2']; [this_dipole '_dir3']};
            end
        end
        
        function plotSpectra(obj) % Run an FFT on each source dipole, 
            % get per-color induced power spectra.
           for iDOI = 1:length(obj.dipole_timecourses)
              conf = [];
              conf.minlength = obj.cfg.enhance.minlength;
              dipole_whole = ft_redefinetrial(conf, ...
                   obj.dipole_timecourses{iDOI});
              
               conf = [];
               conf.minlength = obj.cfg.enhance.minlength;
               conf.toilim    = obj.cfg.enhance.base.toilim;
               dipole_base    = ft_redefinetrial(conf, ...
                   obj.dipole_timecourses{iDOI});
                
               conf.toilim    = obj.cfg.enhance.stim.toilim;
               dipole_stim    = ft_redefinetrial(conf, ...
                   obj.dipole_timecourses{iDOI});
                              
               % recut into sliding windows for Welch
               if obj.cfg.enhance.sliding_window
                   conf = [];
                   conf.length = obj.cfg.enhance.window_length;
                   conf.overlap = obj.cfg.enhance.window_overlap;
                   dipole_base = ft_redefinetrial(conf, dipole_base);
                   dipole_stim = ft_redefinetrial(conf, dipole_stim);
               end
               
               conf = [];
               conf.foilim         = [2 100];
               conf.method         = 'mtmfft';
               conf.taper          = obj.cfg.enhance.freq.taper;
               conf.tapsmofrq      = obj.cfg.enhance.freq.tapsmofrq;
               conf.keeptrials     = 'yes';
               conf.output         = 'pow';
               freq_base           = ft_freqanalysis(conf, dipole_base);
               freq_stim           = ft_freqanalysis(conf, dipole_stim);
               
               freq_base = mergeFFTSnippets(freq_base, freq_base.trialinfo, 3);
               freq_stim = mergeFFTSnippets(freq_stim, freq_stim.trialinfo, 3);
               
               [stimtypes, stimtype] = getStimtypes(obj.subject.colorBehav);
               figure('units', 'centimeters','position', [1 1 20 length(stimtypes)*8], ...
                'color', 'w', 'Visible', 'on');
               for iStimtype = 1:length(stimtypes)
                   thisStimtype = stimtypes{iStimtype};
                   stim_trials = find(strcmp(stimtype, thisStimtype));
                   stim_trialparts = freq_base.trialinfo(...
                       ismember(freq_base.trialinfo, stim_trials));
                   [~,~,iStim_trials] = intersect(stim_trialparts, freq_base.trialinfo);
                   
                   iFreq_base = freq_base.powspctrm(iStim_trials,:,:);
                   iFreq_stim = freq_stim.powspctrm(iStim_trials,:,:);
                   iFreq_base = squeeze(mean(iFreq_base,2));
                   iFreq_base = squeeze(mean(iFreq_base,1));
                   iFreq_stim = squeeze(mean(iFreq_stim,2));
                   iFreq_stim = squeeze(mean(iFreq_stim,1));

                   subplot(length(stimtypes)+1,2,iStimtype*2-1);
                   
                   pColor = cellfun(@str2num, strsplit(thisStimtype), 'un', 0);
                   pColor = cell2mat(pColor(3:5))./255;
                   if any(isnan(pColor)), pColor = [0 0 0]; end
                   
                   plot(freq_base.freq, iFreq_stim./iFreq_base, ...
                       'LineWidth', 3, 'Color', pColor);
                   title(thisStimtype);
                   
                   % Run TFR over whole trial
                   conf            = [];
                   conf.method     = 'mtmconvol';
                   conf.output     = 'pow';
                   conf.taper      = 'dpss';
                   conf.foi        = [2:2:100];
                   conf.t_ftimwin  = repmat(0.5, 1, length(conf.foi));
                   conf.tapsmofrq  = 8;
                   conf.toi        = '50%';
                   conf.pad        = 4;
                   conf.trials     = iStim_trials;
                   TFR = ft_freqanalysis(conf, dipole_whole);
                   subplot(length(stimtypes)+1,2,iStimtype*2);
                   conf = [];
                   conf.baseline = [-1 0-1/1200];
                   conf.baselinetype = 'relchange';
                   ft_singleplotTFR(conf, TFR);
                   
               end
           end
        end
        
        function quantifyGammaPower(obj) % Run an FFT on each source dipole, get
            % induced gamma power spectra.
            obj.results = cell(length(obj.cfg.enhance.doi),1);
            
            for iDOI = 1:length(obj.cfg.enhance.doi) % for all dipoles of interest
                
                conf.minlength = obj.cfg.enhance.minlength;
                conf.toilim    = obj.cfg.enhance.base.toilim;
                dipole_base    = ft_redefinetrial(conf, ...
                    obj.dipole_timecourses{iDOI});
                
                conf.toilim    = obj.cfg.enhance.stim.toilim;
                dipole_stim    = ft_redefinetrial(conf, ...
                    obj.dipole_timecourses{iDOI});
                
                % recut into sliding windows for Welch
                if obj.cfg.enhance.sliding_window
                    conf = [];
                    conf.length = obj.cfg.enhance.window_length;
                    conf.overlap = obj.cfg.enhance.window_overlap;
                    dipole_base = ft_redefinetrial(conf, dipole_base);
                    dipole_stim = ft_redefinetrial(conf, dipole_stim);
                end
                
                % Do frequency analysis
                band_lower  = round(obj.subject.gammapeak_sensors) - ...
                    round(obj.cfg.enhance.bandwidth/2);
                band_higher = round(obj.subject.gammapeak_sensors) + ...
                    round(obj.cfg.enhance.bandwidth/2);
                
                conf = [];
                conf.foilim         = [band_lower band_higher];
                conf.method         = 'mtmfft';
                conf.taper          = obj.cfg.enhance.freq.taper;
                conf.tapsmofrq      = obj.cfg.enhance.freq.tapsmofrq;
                conf.keeptrials     = 'yes';
                conf.output         = 'pow';
                freq_base           = ft_freqanalysis(conf, dipole_base);
                freq_stim           = ft_freqanalysis(conf, dipole_stim);
                
                freq_base = mergeFFTSnippets(freq_base, freq_base.trialinfo, 3);
                freq_stim = mergeFFTSnippets(freq_stim, freq_stim.trialinfo, 3);
                
                % calculate effectscore (step 6)
                meanfreq_baseInd  = squeeze(mean(freq_base.powspctrm,2));
                meanfreq_stim     = squeeze(mean(freq_stim.powspctrm,2));
                meanfreq_base     = squeeze(mean(meanfreq_baseInd,1));
                gamma_inc_spec    = bsxfun(@rdivide, meanfreq_stim, meanfreq_base);
                % gamma_inc is gamma power at individual gamma peak
                % frequency, gamma_inc_spec is the same +- bandwith
                gamma_inc = gamma_inc_spec(:, round(size(gamma_inc_spec, 2)/2));
                
                % fill results table (step 9
                trialNr     = freq_stim.trialinfo;
                
                obj.results{iDOI} = [table(trialNr, ...
                    gamma_inc), array2table(gamma_inc_spec)];
                
                % Merge with behavioral data
                this_behav_result = obj.subject.color_behav;
                this_behav_result.subjNr = repmat(...
                    {num2str(obj.subject.subj_nr, '%03d')}, ...
                    size(obj.subject.behav_results,1), 1);
                this_behav_result.block = floor((this_behav_result.trialNr-1)/...
                    obj.subject.stim_cfg.experiment.nTrialsBlock) + 1;
                
                obj.results{iDOI} = outerjoin(this_behav_result, ...
                    obj.results{iDOI}, 'Keys', 'trialNr', 'MergeKeys', true);
                
                % Merge with eyetracking data
                try
                    sacfile = fullfile(obj.subject.preproc_data, 'saccades.csv');
                    saccades = readtable(sacfile);
                catch
                    warning('No saccade data found.');
                    sac_per_trial = NaN(length(trialNr),1);
                    saccades = table(trialNr, sac_per_trial);
                end
                obj.results{iDOI} = outerjoin(saccades, ...
                    obj.results{iDOI}, 'Keys', 'trialNr', 'MergeKeys', true);
                
                
                % Merge with pupilsize data
                try
                    pupilfile = fullfile(obj.subject.preproc_data, 'pupilsize.csv');
                    pupil = readtable(pupilfile);
                catch
                    warning('No pupil data found.');
                    nope = NaN(length(trialNr),1);
                    pupil = table(trialNr, nope, nope, 'VariableNames', ...
                    {'trialNr', 'prestim_pupil', 'poststim_pupil'});
                end
                obj.results{iDOI} = outerjoin(pupil, ...
                    obj.results{iDOI}, 'Keys', 'trialNr', 'MergeKeys', true);
                
                % Add behavioral response variable
                response = repmat({'tbd'}, length(obj.results{iDOI}.answer), 1);
                response(obj.results{iDOI}.oriChange<0 & ...
                    strcmp(obj.results{iDOI}.answer,'1!')) = {'correct'};
                response(obj.results{iDOI}.oriChange>0 & ...
                    strcmp(obj.results{iDOI}.answer,'2@')) = {'correct'};
                response(obj.results{iDOI}.oriChange<0 & ...
                    strcmp(obj.results{iDOI}.answer,'2@')) = {'error'};
                response(obj.results{iDOI}.oriChange>0 & ...
                    strcmp(obj.results{iDOI}.answer,'1!')) = {'error'};
                response(obj.results{iDOI}.wasCatch & ...
                    strcmp(obj.results{iDOI}.answer, 'none')) = {'cr'};
                response(~obj.results{iDOI}.wasCatch & ...
                    strcmp(obj.results{iDOI}.answer, 'none')) = {'miss'};
                response(strcmp(obj.results{iDOI}.answer, 'LeftShift')) = ...
                    {'intervention'};
                assert(~any(strcmp(response, 'tbd')), 'Missing response cases!');
                obj.results{iDOI}.response = response;
                
                if obj.cfg.general.save_partial_results
                    results = obj.results{iDOI};
                    thisDOI = obj.cfg.enhance.doi{iDOI};
                    display('Saving result to disc.');
                    savepath = fullfile(obj.cfg.paths.data, ...
                        obj.subject.subj_nr, 'Preprocessed', ...
                        ['DOI' thisDOI '_results.csv']);
                    writetable(results, savepath);
                end
            end
        end
        
        function plotResults(obj)
            nBlocks = obj.subject.stim_cfg.experiment.nBlocksGratings + ...
                obj.subject.stim_cfg.experiment.nBlocksRepeat;
            
            effectplotTime = figure('units', 'centimeters',...
                'position', [1 1 length(obj.cfg.enhance.doi)*25 15], ...
                'Visible', 'off');
            set(effectplotTime,'color','w');
            
            for iDOI = 1:length(obj.cfg.enhance.doi)
                doi_results = obj.results{iDOI};
                subplot(length(obj.cfg.enhance.doi), 1, iDOI);
                
                iColors = lines(nBlocks);
                blocks = zeros(nBlocks, 2);
                for iBlock = 1:nBlocks
                    blocks(iBlock, 1) = min(...
                        doi_results.trialNr(doi_results.block == iBlock));
                    blocks(iBlock, 2) = max(...
                        doi_results.trialNr(doi_results.block == iBlock));
                end
                
                smoothedLinePlot(doi_results.trialNr, doi_results.gamma_inc, ...
                    'blocks', blocks, ...
                    'span', obj.cfg.enhance.binsize, ...
                    'color', iColors);
                title(sprintf(['Induced gamma at ' ...
                    'highest gamma power of: %s'], obj.cfg.enhance.doi{iDOI}));
                xlabel('Trial');
                ylabel('Mean induced gamma change');
            end
            
            export_fig(effectplotTime, obj.subject.saveplots.enhance, '-png',...
                '-q101', '-r400', '-nocrop');
        end
        
        function plotBehavior(obj) % Plot accuracy, RTs over bins
            analyseBehavior(obj);
        end
        
        function plotEffectSpectra(obj)
            one_Bin = obj.subject.stim_cfg.experiment.nTrialsBlock/...
                obj.cfg.enhance.bins;
            bin_Nr = repelem(1:obj.cfg.enhance.bins, one_Bin);
            n_Blocks = obj.subject.stim_cfg.experiment.nBlocksGratings + ...
                obj.subject.stim_cfg.experiment.nBlocksRepeat;
            
            band_lower  = round(obj.subject.gammapeak_sensors) - ...
                round(obj.cfg.enhance.bandwidth/2);
            band_higher = round(obj.subject.gammapeak_sensors) + ...
                round(obj.cfg.enhance.bandwidth/2);
            
            for i_DOI = 1:length(obj.cfg.enhance.doi)
                i_DOI_results = obj.results{i_DOI};
                
                % Find the columns which have spectrally resolved gamma power
                % data
                var_names = i_DOI_results.Properties.VariableNames;
                spect_cols = ~cellfun(@isempty, ...
                    regexp(var_names, 'gamma_inc_sp*'));
                
                effectplot_spect = figure('units', 'centimeters',...
                    'position', [1 1 n_Blocks*14 12], ...
                    'Visible', 'off');
                set(effectplot_spect,'color','w');
                title(sprintf(['Induced gamma at ' ...
                    'highest gamma power of: %s'], obj.cfg.enhance.doi{i_DOI}));
                i_Bin_Nr = bin_Nr(i_DOI_results.stimNr)' + ...
                    (i_DOI_results.block*obj.cfg.enhance.bins);
                hold on
                for i_Block = 1:n_Blocks
                    subplot(1, n_Blocks, i_Block);
                    these_Bins = unique(i_Bin_Nr(i_DOI_results.block==i_Block));
                    colors = lines(length(these_Bins));
                    set(gca, 'ColorOrder', colors);
                    hold on
                    for i_Bin = 1:length(these_Bins)
                        this_Bin = these_Bins(i_Bin);
                        i_Means = ESI_nanmean(i_DOI_results{...
                            i_Bin_Nr == this_Bin, spect_cols})*100;
                        nFreqs = length(i_Means);                    
                        plot(linspace(band_lower,band_higher, nFreqs), i_Means); 
                        
                        iLegend{i_Bin} = sprintf('Bin %i', i_Bin);
                    end
                    xlabel('Frequency (Hz)');
                    ylabel('Mean induced change (%)');
                    legend(iLegend, 'Location','southoutside',...
                        'Orientation','horizontal');
                end
                
                export_fig(effectplot_spect, ...
                    fullfile(obj.subject.saveplots.basepath, ...
                    'Change_spectra', [obj.cfg.enhance.doi{i_DOI} obj.subject.subj_nr]),...
                    '-png', '-q101', '-r400');
            end
            
        end
    end
end

