function analyseBehavior( obj )
%ANALYSEBEHAVIOR Subfunction of RepEffect - analyses behavioral
%performance, creates plots.

behavior = obj.subject.behav_results;

nBlocksGrating = obj.subject.stim_cfg.experiment.nBlocksGratings;
nTrialsBlock = obj.subject.stim_cfg.experiment.nTrialsBlock;

% Remove catch trials, nonfixated trials, missed trials
behavior = behavior(behavior.wasCatch == 0,:);
behavior = behavior(behavior.wasFixated == 1,:);
behavior = behavior(~strcmp(behavior.answer, 'none'),:);

% compute if trial is correct
behavior.correct = zeros(size(behavior,1),1);
behavior.correct(and(behavior.oriChange < 0, strcmp(behavior.answer,'1!'))) = 1;
behavior.correct(and(behavior.oriChange > 0, strcmp(behavior.answer,'2@'))) = 1;

if obj.cfg.enhance.fullPsychoCurve
    % use behavior.report to mean 'pressed right button'
    behavior.report =  strcmp(behavior.answer, '2@');
else
    % collapse difficulties over left/right changes
    behavior.report = behavior.correct;
    behavior.oriChange = abs(behavior.oriChange);
end

behavior.orientation = categorical(behavior.orientation);
behavior.answer = categorical(behavior.answer);

behavior.bin = floor((behavior.stimNr-1) / ...
    (nTrialsBlock / obj.cfg.enhance.bins)) +1;

behavior_rep = behavior(strcmp(behavior.stimType, 'grating_repeat'),:);
behavior = behavior(strcmp(behavior.stimType, 'grating'),:);

%% Accuracy
% Compare difficulties
psychoCurve = figure('units', 'centimeters','position', [1 1 15 12], ...
    'color', 'w', 'visible', 'off');
hold on
for iBin = 1:obj.cfg.enhance.bins
    dataBin = behavior(behavior.bin == iBin, :);
    if ~isempty(dataBin)
        diffi = unique(dataBin.oriChange)';
        for iDiffi = 1:length(diffi)
            psychometric.iMean(iDiffi) = mean(dataBin.report(dataBin.oriChange == diffi(iDiffi)));
            psychometric.iSD(iDiffi) = std(dataBin.report(dataBin.oriChange == diffi(iDiffi)));
            psychometric.iSEM(iDiffi) = psychometric.iSD(iDiffi) /...
                sqrt(size(dataBin.report(dataBin.oriChange == diffi(iDiffi)),1));
        end
        errorbar(diffi+iBin*0.008,psychometric.iMean,psychometric.iSEM )
        iLegend{iBin} = sprintf('Bin %i', iBin);
    end
end

% Add repeat block
for iDiffi = 1:length(diffi)
    psychometric.iMean(iDiffi) = mean(behavior_rep.report(behavior_rep.oriChange == diffi(iDiffi)));
    psychometric.iSD(iDiffi) = std(behavior_rep.report(behavior_rep.oriChange == diffi(iDiffi)));
    psychometric.iSEM(iDiffi) = psychometric.iSD(iDiffi) /...
        sqrt(size(behavior_rep.report(behavior_rep.oriChange == diffi(iDiffi)),1));
end
errorbar(diffi+(obj.cfg.enhance.bins+1)*0.008,...
    psychometric.iMean,psychometric.iSEM )
iLegend{obj.cfg.enhance.bins+1} = sprintf('Repetition');

xlim([min(diffi)-0.05, max(diffi)+0.05]);
ylim([0, 1]);
title('Performance at different difficulties')
xlabel('Rotation (degrees)');
if obj.cfg.enhance.fullPsychoCurve
    ylabel('Percent right button presses');
else
    ylabel('Percent correct');
end
legend(iLegend, 'Location', 'southoutside', 'Orientation', 'horizontal');
export_fig(psychoCurve, fullfile(obj.cfg.paths.plot, 'Accuracy', ...
    ['Accuracy' obj.subject.subj_nr]), '-png', '-q101', '-r300', '-nocrop');

%% Fit a psychometric function (if sensible)

psychoFit = figure('Visible', 'off', 'Color', 'w');

if mean(behavior.correct) < 0.55
    % Don't create a fit
    warning('Subject was buttonmashing - not fitting a psychometric function.');
    text(0.1,0.5,'Subject was buttonmashing - not fitting a psychometric function.');
    gca.Visible = 'off';
    
else
    [psychoResults] = fitPsychoFunction(obj, behavior);
end

export_fig(psychoFit, fullfile(obj.cfg.paths.plot, 'Accuracy_Fits', ...
    ['Accuracy_Fit' obj.subject.subj_nr]), '-png', '-q101', '-r300', '-nocrop');

%% Reaction times
rtPlot = figure('units', 'centimeters','position', [1 1 15 10], 'color', 'w',...
    'Visible', 'off');
hold on
plot(behavior.trialNr, behavior.rt);
plot(behavior_rep.trialNr, behavior_rep.rt);
% add vertical lines after each block
for iBlock = 1:nBlocksGrating
    plot([nTrialsBlock*iBlock+0.5, nTrialsBlock*iBlock+0.5], ...
        [0, max(behavior.rt)+0.2], 'r');
end
ylim([min(behavior.rt)-0.1, max(behavior.rt)+0.2]);
title('Reaction time over all trials');
xlabel('Trial number');
ylabel('RT in sec.');
export_fig(rtPlot, fullfile(obj.cfg.paths.plot, 'RT_all', ...
    ['RT_all' obj.subject.subj_nr]), '-png', '-q101', '-r300', '-nocrop');

% psychometric curve reaction times
psychoCurveRT = figure('units', 'centimeters','position', [1 1 15 12], ...
    'color', 'w', 'visible', 'off');
hold on
for iBin = 1:obj.cfg.enhance.bins
    dataBin = behavior(behavior.bin == iBin, :);
    if ~isempty(dataBin)
    diffi = unique(dataBin.oriChange)';
    for iDiffi = 1:length(diffi)
        psychometric.iMean(iDiffi) = mean(dataBin.rt(dataBin.oriChange == diffi(iDiffi)));
        psychometric.iSD(iDiffi) = std(dataBin.rt(dataBin.oriChange == diffi(iDiffi)));
        psychometric.iSEM(iDiffi) = psychometric.iSD(iDiffi) /...
            sqrt(size(dataBin.rt(dataBin.oriChange == diffi(iDiffi)),1));
    end
    errorbar(diffi+iBin*0.008,psychometric.iMean,psychometric.iSEM)
    iLegend{iBin} = sprintf('Bin %i', iBin);
    end
end

% Add repeat block
for iDiffi = 1:length(diffi)
    psychometric.iMean(iDiffi) = mean(behavior_rep.rt(behavior_rep.oriChange == diffi(iDiffi)));
    psychometric.iSD(iDiffi) = std(behavior_rep.rt(behavior_rep.oriChange == diffi(iDiffi)));
    psychometric.iSEM(iDiffi) = psychometric.iSD(iDiffi) /...
        sqrt(size(behavior_rep.rt(behavior_rep.oriChange == diffi(iDiffi)),1));
end
errorbar(diffi+(obj.cfg.enhance.bins+1)*0.008,...
    psychometric.iMean,psychometric.iSEM)
iLegend{obj.cfg.enhance.bins+1} = sprintf('Repetition');
xlim([min(diffi)-0.05, max(diffi)+0.05]);
title('Performance at different difficulties')
xlabel('Rotation (degrees)');
ylabel('Reaction time');
legend(iLegend, 'Location', 'southoutside', 'Orientation', 'horizontal');
export_fig(psychoCurveRT, fullfile(obj.cfg.paths.plot, 'RT', ...
    ['RT' obj.subject.subj_nr]), '-png', '-q101', '-r300', '-nocrop');

end

