function [subject] = beamform (cfg, subject, data, template)
%BEAMFORM Script for running a beamform analysis for visual gamma on MEG
%data. Can be separated into 5 major parts:
% 1) prepare head- and sourcemodel
% 2) run DICS beamformer
% 3) find location of peak gamma activity
% 4) run LCMV beamformer 
% 5) save plots and LCMV results
% 
% Inputs:
% cfg        = config structure as prepared with fillConfigs.m
% subject    = subject structure as prepared with fillSubjectInfo.m
% data       = preprocessed MEG data
% template   = template grid for creating sourcemodel. Can be prepared with
%              getTemplateSourcemodel.m. Should be in 'mni' coordsys for
%              compatibility.
% Outputs:
% subject    = subject structure is extended by field 'source', containing
%              information about the location of the gamma peak in
%              specified ROIs
 
%% Prepare head- and sourcemodel

% load data 
mri_raw = ft_read_mri(subject.mridata);
sensors = ft_read_sens(strcat(subject.megdata,filesep),'senstype','meg');

% realign ctf coordinates (step 1)
mri_realigned               = mri_raw;
mri_realigned.transform     = subject.mri_realigned.transform;
mri_realigned.inside        = subject.mri_realigned.inside;
mri_realigned.transformorig = subject.mri_realigned.transformorig;
mri_realigned.coordsys      = subject.mri_realigned.coordsys;
clear subject.mri_realigned

% segment into chosen output (step 2)
mri_segmented  = ft_volumesegment(cfg.segment, mri_realigned);

% create headmodel and convert to cm (step 3)
headmodel = ft_prepare_headmodel(cfg.headmodel, mri_segmented);
headmodel = ft_convert_units(headmodel, 'cm');

% create the subject specific grid using a previously created template
% (step 4)
cfg.sourcemodel.grid.template  = template;
cfg.sourcemodel.mri            = mri_realigned;
grid                           = ft_prepare_sourcemodel(cfg.sourcemodel);

%% DICS Beamformer

% redefine trials into baseline and stimulus (step 1)
data_whole  = ft_redefinetrial(cfg.defall, data);
data_base   = ft_redefinetrial(cfg.defbase, data);
data_stim   = ft_redefinetrial(cfg.defstim, data);

% do frequency analysis (step 2)
freq_whole  = ft_freqanalysis(cfg.freq, data_whole);
freq_base   = ft_freqanalysis(cfg.freq, data_base);
freq_stim   = ft_freqanalysis(cfg.freq, data_stim);

% run sourceanalysis on whole trial (step 3)
cfg.beamdics.grid         = grid;
cfg.beamdics.headmodel    = headmodel;
source_whole              = ft_sourceanalysis(cfg.beamdics, freq_whole);

% run sourceanalysis on baseline and stimulus presentation (step 4)
cfg.source.grid.filter    = source_whole.avg.filter;
source_base               = ft_sourceanalysis(cfg.beamdics, freq_base);
source_stim               = ft_sourceanalysis(cfg.beamdics, freq_stim);

% create difference variable
source_diff = source_stim;
source_diff.avg.pow = (source_stim.avg.pow - source_base.avg.pow) ./...
                       source_base.avg.pow;

% interpolate sources on mri (step 5)
source_diffInt = ft_sourceinterpolate(cfg.int, source_diff , mri_realigned);

%% Find source with highest gamma power

% load atlas (step 1)
atlas = ft_read_atlas(cfg.gammaloc.atlas);
atlas = ft_convert_units(atlas, 'cm');
cfg.mask.atlas = atlas;

%find maximum activity in identified roos (step 2)
for iROI = 1:length(cfg.mask.spec_roi)
    source_diff.pos = template.pos;
    cfg.mask.roi = atlas.tissuelabel(1, cfg.mask.spec_roi{iROI,2});
    
    mask = ft_volumelookup(cfg.mask, source_diff);
    [val, pos] = max(source_diff.avg.pow(mask));
    posMask    = source_diff.pos(mask,:);
    pos        = posMask(pos,:);
    
    cfg.lab.output     = 'label';
    cfg.lab.inputcoord = cfg.mask.inputcoord;
    cfg.lab.atlas      = cfg.mask.atlas;
    cfg.lab.roi        = pos;
    labels             = ft_volumelookup(cfg.lab, source_diff);

    lab = find(labels.count);
    lab = labels.name(lab);

    subject.sources.(cfg.mask.spec_roi{iROI,1}).val  = val;
    subject.sources.(cfg.mask.spec_roi{iROI,1}).pos  = pos;
    subject.sources.(cfg.mask.spec_roi{iROI,1}).lab  = lab;
    subject.sources.(cfg.mask.spec_roi{iROI,1}).mask = mask;
end

%% LCMV Beamformer

% cut out ERF (step 1)
data_base = ft_redefinetrial(cfg.erfbase, data);
data_stim = ft_redefinetrial(cfg.erfstim, data);
data_woERF = ft_appenddata([], data_base, data_stim);

%timelock trials (step 2)
cfg.timelock.dataset = data_woERF;
timelock             = ft_timelockanalysis(cfg.timelock, data_woERF);

cfg.timelock.dataset = data_woERF;
cfg.timelock.channel = cfg.timelock.coi;
timelockOCC = ft_timelockanalysis(cfg.timelock, data_woERF);

% create common filter using the whole timelocked trials (step 3)
cfg.beamlcmv.grid    = grid;
cfg.beamlcmv.vol     = headmodel;
source_lcmv          = ft_sourceanalysis(cfg.beamlcmv, timelock);

%% Save and Plot

% plot subject sourcemodel
figure('units', 'centimeters','position', [1 1 30 18]);
set(gcf, 'Visible', 'off')

subplot(1,2,1)
hold on;
ft_plot_vol(headmodel, 'facecolor', 'cortex', 'edgecolor',...
    'none');alpha 0.5; camlight;
ft_plot_mesh(grid.pos(grid.inside,:), 'vertexsize', 5);
ft_plot_sens(sensors, 'style', '*b');
view([-260 30])

subplot(1,2,2)
hold on;
ft_plot_vol(headmodel, 'facecolor', 'cortex', 'edgecolor',...
    'none');alpha 0.5; camlight;
ft_plot_mesh(grid.pos(grid.inside,:), 'vertexsize', 5);
ft_plot_sens(sensors, 'style', '*b');
view([0 0])

set(gcf, 'Color', 'w');
export_fig(subject.saveplots.sourcemodel,'-png','-q101','-r400');

% plot dics results
config = [];
config.method        = 'slice';
config.funparameter  = 'avg.pow';
config.maskparameter = config.funparameter;
config.visible       = 'off';
config.slicedim = 2;
ft_sourceplot(config, source_diffInt);

saveas(gcf, subject.saveplots.dics,'png')

% plot timelocked trial and deleted ERF
figure('units', 'centimeters','position', [1 1 30 18]);
set(gcf, 'Visible', 'off')

subplot(2,1,1)
plot(timelock.time, timelock.avg)
xlabel('time')
ylabel('power')
title('All MEG channels')

subplot(2,1,2)
plot(timelockOCC.time, mean(timelockOCC.avg))
xlabel('time')
ylabel('power')
title('Occipital channels')

saveas(gcf, subject.saveplots.timelocked,'png')

% save lcmv results
source_lcmv.mni_pos = template.pos;
subject.dipoles.pos         = source_lcmv.pos(source_lcmv.inside,:);
subject.dipoles.mni_pos     = source_lcmv.mni_pos(source_lcmv.inside,:);
subject.dipoles.filters     = source_lcmv.avg.filter(source_lcmv.inside,:);
ESIsave(fullfile(subject.prepoc_data, 'subject'), 'subject');

end

