function plot_in_DKL(DKLcoords, baseRGB, colors, fundamentals)
%PLOT_IN_DKL Plot an equiluminant slice through DKL with a set of color
%coordinates marked. baseRGB is the RGB coordinate of [0 0 0].
% colors    Measured spectral power distributions of the red, green and 
%           blue color channels of the display device at maximum output 
%           (e.g. RGB = [1, 0, 0] for red). Formatted as an n by 3 matrix
%           with row order [red; green; blue]. Default = the spectra I
%           measured from the ProPixx. Colors and fundamentals need to be
%           on the same wavelength axis.
% fundamentals Provided cone fundamenals normalized to unity. The Stockmann
%           & Sharpe (2010) data works well. Formatted as an n by 3 matrix
%           with row order [red; green; blue]. Default = Stockmann & Sharpe
%           (2010). Colors and fundamentals need to be on the same 
%           wavelength axis.

%% Load defaults.
if ~exist('baseRGB', 'var') || isempty(baseRGB)
    baseRGB = [102 102 102]./255;
end

if ~exist('colors', 'var') || isempty(colors)
    [colors, ~] = load_default_spectra('ProPixx', ...
        'StockMannSharpe2010_10deg');
end
if ~exist('fundamentals', 'var') || isempty(fundamentals)
    [~, fundamentals] = load_default_spectra('ProPixx', ...
        'StockMannSharpe2010_10deg');
end
assert(isequal(size(colors), size(fundamentals)));
assert(isequal(size(baseRGB), [1 3]), 'baseRGB needs to be in 1x3 vector.');

% get LMS coordinates of the base color
baseLMS = rgb2lms(baseRGB);
baseLAB = rgb2lab(baseRGB, 'ColorSpace', 'linear-rgb');

% set up DKL coordinate grid
resolut = 201;
LM = linspace(-1, 0.7, resolut);
SLM = linspace(-2, 1.4, resolut);
iCombinations = combvec(LM, SLM);
iDKL = [zeros(size(iCombinations,2),1), iCombinations'];

iLMS = NaN(size(iCombinations,2),3);
iRGB = NaN(size(iCombinations,2),3);
iLAB = NaN(size(iCombinations,2),3);
iAlpha = ones(size(iCombinations,2),1);
% transform DKL coordinates to LMS
n = size(iDKL,1);
fprintf('Completed: 0%%')
for i=1:n
    iperc = i/n*100;
    fprintf('%2.0f%%\n', round(iperc))
        iLMS(i,:) = dkl2lms(baseLMS, iDKL(i,:));
        iRGB(i,:) = lms2rgb(iLMS(i,:) , colors, fundamentals);
        iLAB(i,:) = rgb2lab(iRGB(i,:), 'ColorSpace', 'linear-rgb');
        if any(iRGB(i,:) >1) || any(iRGB(i,:) <0)
            iRGB(i,:) = [0 0 0];
            iAlpha(i) = 0;
        end
end

%transform the original stimulus coordinates into RGB and LAB space
for iStim = 1:size(DKLcoords,1)
    LMScoords = dkl2lms(baseLMS, DKLcoords(iStim,:));
    RGBCoords(iStim,:) = lms2rgb(LMScoords, colors, fundamentals);
    LABCoords(iStim,:) = rgb2lab(RGBCoords(iStim,:), 'ColorSpace', 'linear-rgb');
end

iAlpha = logical(iAlpha);
fig = figure('Unit', 'centimeter', 'Position', [0 0 12 12]);
scatter(iDKL(iAlpha,2), iDKL(iAlpha,3), 30, lin2rgb(iRGB(iAlpha,:)), 'filled')
hold on
h = scatter(DKLcoords(:,2), DKLcoords(:,3), 50, lin2rgb(RGBCoords), 'filled');
h.MarkerEdgeColor = 'k'; h.LineWidth = 2;
h = scatter(0, 0, 50, lin2rgb(baseRGB), 'filled');
h.MarkerEdgeColor = 'k'; h.LineWidth = 2;
axis equal
axis off
camroll(180)
exportgraphics(fig, 'DKLplot.pdf')


fig = figure('Unit', 'centimeter', 'Position', [0 0 12 12]);
scatter(iLAB(iAlpha,2), iLAB(iAlpha,3), 30, lin2rgb(iRGB(iAlpha,:)), 'filled')
hold on
h = scatter(LABCoords(:,2), LABCoords(:,3), 50, lin2rgb(RGBCoords), 'filled');
h.MarkerEdgeColor = 'k'; h.LineWidth = 2;
h = scatter(baseLAB(:,2), baseLAB(:,3), 50, lin2rgb(baseRGB), 'filled');
h.MarkerEdgeColor = 'k'; h.LineWidth = 2;

axis equal
axis off
exportgraphics(fig, 'LABplot.pdf')

end

