function [ms, cfg] = bhv_microsaccade_detection(cfg, eyedata)
% uses detection algorithim from Engbert, R. & Kliegl, R. (2002)
% computes a different threshold relative to noise for each trial
% seperate threshold for vertical and horizontal velocities

if ~exist('cfg','var'); cfg = []; end

if ~isfield(cfg, 'toilim')
    cfg.toilim = [-0.5 1.3];
end
if ~isfield(cfg, 'velThresh')
    cfg.velThresh = 5; %sd. 5 as in Engbert and Mergenthaler, 2006 (used 6/7 for noisey NIH data)
end
if ~isfield(cfg,  'MinDur')
    cfg.MinDur = 4; %samples. As in Engbert, R. & Kliegl, R. (2002) 
end
if ~isfield(cfg, 'amplMax')
    cfg.amplMax = 2; % fix window in degrees
end
if ~isfield(cfg, 'velMax')
    cfg.velMax = cfg.amplMax*110; % roughly *100 on main sequence
end
if~isfield(cfg, 'tOverShoot')
    cfg.tOverShoot = 20; %ms (20 is from Kleo 15/05/2018 as in Martinez-Conde et al 2000; McCamy et al, 2014)
end
if ~isfield(cfg, 'debugplot')
    cfg.debugplot = true;
end
if ~isfield(cfg, 'speedAlgo')
    cfg.speedAlgo = 2; %use 3 for high sample rates
end

%% prepare data

cfgIn = [];
cfgIn.toilim = cfg.toilim;
eyedata = ft_redefinetrial(cfgIn, eyedata);

%% median filter
% cfgIn = [];
% cfgIn.medianfilter = 'yes'; %boxcar preserving steps
% cfgIn.medianfiltord = 5;
% tmp = ft_preprocessing(cfgIn, eyedata);

%% halve sampling rate
% for iTrl = 1:length(eyedata.trial)
%     x = eyedata.trial{iTrl}(1,:);
%     y = eyedata.trial{iTrl}(2,:);
%     newSmp = floor(length(x)/2);
%     
%     x = mean(reshape(x(1:newSmp*2),2,newSmp),1);
%     y = mean(reshape(y(1:newSmp*2),2,newSmp),1);
%     eyedata.trial{iTrl} = cat(1,x,y);
%     eyedata.time{iTrl} = eyedata.time{iTrl}(2:2:end);
% end
% eyedata.fsample = eyedata.fsample/2;

%% calc
%   micrDetails(1:num,1)   onset of saccade
%   micrDetails(1:num,2)   end of saccade
%   micrDetails(1:num,3)   peak velocity of saccade
%   micrDetails(1:num,4)   saccade amplitude
%   micrDetails(1:num,5)   angular orientation
%   micrDetails(1:num,6)   horizontal component (delta x)
%   micrDetails(1:num,7)   vertical component (delta y)

ms = make_ms_struct(cfg, eyedata);

if cfg.debugplot
    micro_check_detection(ms, eyedata, 15, cfg.speedAlgo)
end

%% save
%savename = ['~/rew_analysis/', ms.session '_ms.mat'];
%disp(['Now saving ' savename])
%save(savename, 'ms', 'cfg');

end

%% subfunctions

function ms = make_ms_struct(cfg, eyedata)

% preallocate
nTrl = length(eyedata.trial);
micrDetails{nTrl} = [];
micrTimes{nTrl} = [];
nSac(nTrl) = 0;
sacRate(nTrl) = 0;

for iTrl = 1:nTrl
    xy = eyedata.trial{iTrl}';
    
    v = vecvel(xy,eyedata.fsample, cfg.speedAlgo);
    
%     figure('Position', [0 0 1000, 600])
%     subplot(1,2,1)
%     plot(eyedata.time{iTrl}, xy)
%     subplot(1,2,2)
%     plot(eyedata.time{iTrl}, v)
    
    micrDetTemp = micr_details(xy(:,1), xy(:,2), v(:,1), v(:,2), ...
        cfg.velThresh, cfg.MinDur, cfg.tOverShoot/1000, eyedata.fsample);
    if ~isempty(micrDetTemp)
        
        [micrDetails{iTrl}, nRej{iTrl}, nOver{iTrl}] = pick_ms(micrDetTemp, ...
            cfg.amplMax, cfg.velMax, cfg.tOverShoot, 1000/eyedata.fsample);
        
        if ~isempty(micrDetails{iTrl})
            % convert to time as in eyedata
            micrTimes{iTrl} = eyedata.time{iTrl}(micrDetails{iTrl}(:,1:2));
        end
    else
        micrDetails{iTrl} = [];
    end
    nSac(iTrl) = size(micrDetails{iTrl},1);
    sacRate(iTrl) = nSac(iTrl)/(eyedata.time{iTrl}(end) - eyedata.time{iTrl}(1));
end

ms.micrDetails = micrDetails;
ms.micrTimes = micrTimes;
ms.nSac = nSac;
ms.sacRate = nanmean(sacRate);
ms.sacRateTrl = sacRate;
ms.fsample = eyedata.fsample;
ms.trialinfo = eyedata.trialinfo;
ms.analWin = cfg.toilim;

%[~,ms.session] = fileparts(cfg.filename);
if cfg.debugplot
    figure('Position', [0 0 1000 500])
    subplot(1,2,1)
    hist([nRej{:}],max([nRej{:}])+1)
    title('Number of too large microsaccades per trial')
    
    subplot(1,2,2)
    hist([nOver{:}],max([nOver{:}])+1)
    title('Number of overshoot microsaccades per trial')
end
end


function [ micrDetails, nReject, nOvershoot ] = pick_ms(micrDetails, amplmax, velmax, ovshoot_time, time_steps)
%microsaccades were required to have a magnitude <1 and a temporal distance from the previous microsaccade of >20 ms
%remove microsaccades larger than ampl/vel threshold and those closer than
%overshoot time
nReject = 0;
n=0;
for iMicr = 1:size(micrDetails,1)
    n=n+1;
    if micrDetails(n,4)>=amplmax || micrDetails(n,3)>=velmax
        micrDetails(n,:) = [];
        n=n-1;
        nReject = nReject+1;
    end
end

% now remove the MS that happen within the next ovshoot_time ms after the previous one
% - overshoots-
nOvershoot = 0;

ovshoot_time = ovshoot_time/time_steps;

if ~isempty(micrDetails)
    n = 1;
    for iMicr = 2:size(micrDetails,1)
        n=n+1;
        if micrDetails(n,1) <= micrDetails(n-1,2) + ovshoot_time
            micrDetails(n,:) = [];
            n=n-1;
            nOvershoot = nOvershoot+1;
        end
    end
end

end