function micrDetails = micr_details(x,y,vx,vy,velThresh,MinDur,overshoot,SAMPLING)
%--------------------------------------------------------------------
%  FUNCTION micrDetails
%--------------------------------------------------------------------
%  PLEASE CITE THIS REFERENCE:
%  Engbert, R. & Kliegl, R. (2002) 
%  Microsaccades uncover the orientation of covert attention.
%  Vision Research 43, 1035-1045.
% modified last by K.K on 17/11/2014
% modified by KS on 23/05/2018
%--------------------------------------------------------------------
% INPUT
%   x(:,1)        position vector x
%   y(:,1)        position vector y
%   vx(:,1)       velocity vector  x
%   vy(:,1)       velocity vector  y
%   velThresh     relative velocity threshold 
%   MinDur        minimum saccade duration (number of samples) ***for
%   microsaccades it is ~8ms(Logothetis)// in their paper it is 12ms.
%   overshoot     maximum ms overshoot window (seconds) 0.020 in Martinez-Conde et al 2000

% OUTPUT
%   micrDetails(1:num,1)   onset of saccade
%   micrDetails(1:num,2)   end of saccade
%   micrDetails(1:num,3)   peak velocity of saccade
%   micrDetails(1:num,4)   saccade amplitude 
%   micrDetails(1:num,5)   angular orientation  
%   micrDetails(1:num,6)   horizontal component (delta x)
%   micrDetails(1:num,7)   vertical component (delta y)
%---------------------------------------------------------------------

% 1. Computing thresholds
msdx = sqrt( median(vx.^2) - (median(vx))^2 ); 
msdy = sqrt( median(vy.^2) - (median(vy))^2 );
radiusx = velThresh*msdx;
radiusy = velThresh*msdy;

% 2. Detecting microsaccades 
test = (vx/radiusx).^2 + (vy/radiusy).^2;
indx = find(test>1);

% vabs = sqrt( vx.^2 + vy.^2 );
% indx = find(vabs<velThresh);

% 3. Building sequence
N = length(indx); 
maxGap = round(overshoot*SAMPLING);
micrDetails = [];
nsac = 0;
dur = 1;
a = 1;
k = 1;
%durations
while k<N
    indGap = indx(k+1)-indx(k);
    if indGap<=maxGap
        dur = dur + indGap;
    else
        % duration > MinDur
        if dur>=MinDur 
            nsac = nsac + 1;
            b = k;
            micrDetails(nsac,:) = [indx(a) indx(b)];
        end
        a = k+1;
        dur = 1;
    end
    k = k + 1;
end
% last saccade: duration > MINDUR?
if dur>=MinDur 
    nsac = nsac + 1;
    b = k;
    micrDetails(nsac,:) = [indx(a) indx(b)];
end

% 4. Computing microsaccade parameters
%vabs = SAMPLING*sqrt([0; diff(x)].^2 + [0; diff(y)].^2); % smoothed vel is too small
vabs = sqrt(vx.^2 + vy.^2);
for s=1:nsac
    a = micrDetails(s,1);   % begin of saccade time stamps
    b = micrDetails(s,2);   % end of saccade time stamps
    vpeak = max(vabs(a:b)); 
    ampl = sqrt( (x(a)-x(b))^2 + (y(a)-y(b))^2 ); 
    micrDetails(s,3) = vpeak; % peak velocity 
    micrDetails(s,4) = ampl;  % amplitude
    delx = x(b)-x(a);   
    dely = y(b)-y(a); % was y(a)-y(b)  WRONG!!!
    phi = 180/pi*atan2(dely,delx);   
    micrDetails(s,5) = phi;  % angular orientation
    micrDetails(s,6) = delx; % horizontal component
    micrDetails(s,7) = dely; % vertical component
end