function [] = qcDiode(cfg, subject, data, events, iPlot)
% qcDiode   Quality control for delay of trigger information based on
% photodiode recordings in MEG experiments. Allows for two different
% levels. Creates mean data as well as raw data plots as .png output.
% Trigger values, channel labels, expected delay and plot titles
% can be set in fillConfigs.m
%
% data    = preprocessed fieldtrip data structure
% events  = events structure as created in preprocess.m
% subject = subject structure created with fillSubjectInfo.m
% cfg     = config structure created with fillConfigs.m
% iPlot   = number to be added to filename. Useful if several datasets per
%           subject were recorded. Optional.

% Get diode and trigger channel.
chan_diode      = data.trial{1,1}(strcmp(cfg.diodeChan, data.label),:);
chan_trigger    = data.trial{1,1}(strcmp(cfg.triggerChan, data.label),:);

if cfg.flip_axis
    chan_diode = chan_diode*-1;
end

%% Trigger 1 - Stimulus onset
% Get samples of the stimulus onset trigger.
samples_on = arrayfun(@(x) any(x.value == cfg.trigger_on), events);
samples_on = [events(samples_on).sample];

% Get indices of windows around on triggers.
winLength = cfg.plotwindow;
windows   = NaN(length(samples_on), winLength*2+1);

for iTrigger = 1:size(windows,1)
    startWin = samples_on(iTrigger) - winLength;
    endWin = samples_on(iTrigger) + winLength;
    windows(iTrigger,:) = startWin:1:endWin; 
end

% Extract photodiode and trigger data inside windows.
diode_trigger_on = NaN(length(samples_on),size(windows,2));
trig_trigger_on  = NaN(length(samples_on),size(windows,2));

for iTrial = 1:length(samples_on)
    diode_trigger_on(iTrial,:) = chan_diode(windows(iTrial,:));
    trig_trigger_on(iTrial,:) = chan_trigger(windows(iTrial,:));
end

mean_diode_trigger_on   = mean(diode_trigger_on);
mean_trigger_trigger_on = mean(trig_trigger_on);

%% Trigger 2 - Stimulus offset
% Get samples of the stimulus offset trigger
samples_off = arrayfun(@(x) any(x.value == cfg.trigger_off), events);
samples_off = [events(samples_off).sample];

% Get indices of windows around off triggers
winLength = cfg.plotwindow;
windows   = NaN(length(samples_off), winLength*2+1);

for iTrigger = 1:size(windows,1)
    startWin = samples_off(iTrigger) - winLength;
    endWin = samples_off(iTrigger) + winLength;
    windows(iTrigger,:) = startWin:1:endWin; 
end

% Extract photodiode and trigger data inside windows
diode_trigger_off = NaN(length(samples_off),size(windows,2));
trig_trigger_off  = NaN(length(samples_off),size(windows,2));

for iTrial = 1:length(samples_off)
    diode_trigger_off(iTrial,:) = chan_diode(windows(iTrial,:));
    trig_trigger_off(iTrial,:) = chan_trigger(windows(iTrial,:));
end

mean_diode_trigger_off   = mean(diode_trigger_off);
mean_trigger_trigger_off = mean(trig_trigger_off);

%% Plot and save

% X-axis definition
x_axis = 0-winLength:winLength;

% plot one figure with four subplots
diode_plot = figure('units','centimeter', 'position', [1 1 30 20], 'Visible', 'off');
set(gcf, 'Color', 'w');

% Mean trigger on
subplot(2,2,1)
[a,~,~] = plotyy(x_axis, mean_diode_trigger_on, x_axis, mean_trigger_trigger_on);
%set(a(1),'yLim',[0 0.3])
%set(a(2),'yLim',[0 45])
title('Stimulus On trigger');
xlabel('Samples')
ylabel(a(1),'Signal(Diode)') % left y-axis 
ylabel(a(2),'Signal(Trigger)') % right y-axis

% Mean trigger off
subplot(2,2,2)
[a,~,~] = plotyy(x_axis, mean_diode_trigger_off, x_axis, mean_trigger_trigger_off);
%set(a(1),'yLim',[0 0.3])
%set(a(2),'yLim',[0 125])
title('Stimulus Off trigger');
xlabel('Samples')
ylabel(a(1),'Signal(Diode)') % left y-axis 
ylabel(a(2),'Signal(Trigger)') % right y-axis

% All diode traces after the stim on trigger
subplot(2,2,3)
plot(x_axis, diode_trigger_on);
hold on
line([cfg.delay_on cfg.delay_on], get(gca, 'ylim'),'LineStyle','--','Color',...
     [0.7 0.7 0.7]);
xlim([cfg.delay_on-10 cfg.delay_on+10]);
xlabel('Samples');

% All diode traces after the stim off trigger
subplot(2,2,4)
plot(x_axis, diode_trigger_off);
hold on
line([cfg.delay_off cfg.delay_off], get(gca, 'ylim'),'LineStyle','--','Color',...
     [0.7 0.7 0.7]);
xlim([cfg.delay_off-10 cfg.delay_off+10]);
xlabel('Samples');

% Save as .png
if exist('iPlot', 'var')
    filename = [subject.saveplots.diode '_' num2str(iPlot)];
else
    filename = subject.saveplots.diode;
end

export_fig(diode_plot, filename, '-png', '-q101', '-r400');

end

