function [subject] = qcEmptyroom(cfg, subject)
% qcEmptyroom   creates an overview for quality control of an empty room
% recording.  The output is saved as a .png file, showing the power 
% spectrum and the log power spectrum of all channels and a selection of 
% channels of interest (here: occipital). In case no empty room recording 
% is found, a .png file is created stating that no empty room recording was
% found. Includes manual artifact rejection on channel level, where noisy
% channels are included in the subject structure.
%
% subject = subject structure created with fillSubjectInfo.m
% cfg     = config structure as created with fillConfigs.m
 
%check whether an empty room recording exists
try
data_emptyroom = ft_preprocessing(cfg);

if cfg.do_manual_ar 
    
    % check for bad channels in trial view
    cfg.manual_ar         = ft_definetrial(cfg.manual_ar);
    data_channels         = ft_preprocessing(cfg.manual_ar);
    cfg.manual_ar.method  = 'channel';
    data_clean            = ft_rejectvisual(cfg.manual_ar, data_channels);
    subject.ER_noisyChanView = setdiff(data_emptyroom.label,...
                               data_clean.label);
    
    % double check for bad channels in summary
    cfg.manual_ar.method  = 'summary';
    cfg.manual_ar.metric  = 'zvalue';
    data_clean            = ft_rejectvisual(cfg.manual_ar, data_emptyroom);
    subject.ER_noisySummView = setdiff(data_emptyroom.label,...
                               data_clean.label);
    
    % write bad channels into subject structure
    subject.ER_noisyCombined = intersect(subject.ER_noisyChanView,...
                               subject.ER_noisySummView);
end


%% PLOT 

% call figure and turn invisible
figure('units','centimeter', 'position', [1 1 30 20]);
set(gcf, 'Visible', 'off')

% plot power spectrum of all sensors
cfg.freqs.channel = cfg.freqs.allmeg;
powerspec         = ft_freqanalysis(cfg.freqs, data_emptyroom);

if cfg.do_manual_ar
     subplot(3,2,1);
else subplot(2,2,1);
end
    
plot(powerspec.freq, powerspec.powspctrm);
xlim([0 80]);
title('All Channels');
xlabel('Frequency');
ylabel('Power');
line([16.5  16.5], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
line([  50    50], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);

if cfg.do_manual_ar
     subplot(3,2,2);
else subplot(2,2,2);
end

logpower = log(powerspec.powspctrm);
semilogx(powerspec.freq, logpower);
xlim([0 500]);
title('All Channels');
xlabel('Frequency');
ylabel('Log(Power)');
line([16.5  16.5], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
line([  50    50], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
line([ 150   150], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
line([ 200   200], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
line([ 250   250], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);

% plot power spectrum for occiptial sensors only
cfg.freqs.channel = cfg.freqs.coi;
powerspec         = ft_freqanalysis(cfg.freqs, data_emptyroom);

if cfg.do_manual_ar
     subplot(3,2,3);
else subplot(2,2,3);
end

plot(powerspec.freq, powerspec.powspctrm);
xlim([0 80]);
title(cfg.freqs.coiLabel);
xlabel('Frequency');
ylabel('Power');
line([16.5  16.5], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
line([  50    50], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);

if cfg.do_manual_ar
     subplot(3,2,4);
else subplot(2,2,4);
end

logpower = log(powerspec.powspctrm);
semilogx(powerspec.freq, logpower);
xlim([0 500]);
title(cfg.freqs.coiLabel);
xlabel('Frequency');
ylabel('Log(Power)');
line([16.5  16.5], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
line([  50    50], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
line([ 150   150], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
line([ 200   200], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
line([ 250   250], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);

if cfg.do_manual_ar
    % plot power spectrum for noisy sensors
    cfg.freqs.channel = subject.ER_noisyCombined;
    powerspec         = ft_freqanalysis(cfg.freqs, data_emptyroom);

    subplot(3,2,5);
    plot(powerspec.freq, powerspec.powspctrm);
    xlim([0 80]);
    title('Noisy Channels');
    xlabel('Frequency');
    ylabel('Power');
    line([16.5  16.5], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
    line([  50    50], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
    % add textbox with noisy channel labels
    str = subject.ER_noisyCombined;
    y = get(gca,'ylim');
    y = y(2) - (y(2)/8);
    text(5,y,str);
    
    subplot(3,2,6);
    logpower = log(powerspec.powspctrm);
    semilogx(powerspec.freq, logpower);
    xlim([0 500]);
    title('Noisy Channels');
    xlabel('Frequency');
    ylabel('Log(Power)');
    line([16.5  16.5], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
    line([  50    50], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
    line([ 150   150], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
    line([ 200   200], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
    line([ 250   250], get(gca, 'ylim'),'LineStyle','--','Color',[0.7 0.7 0.7]);
end

% save power spectrum plot
set(gcf, 'Color', 'w');
export_fig(subject.saveplots.emptyroom,'-png');

catch
    % no empty room recording was made 
    figure;
    set(gcf, 'Visible', 'off')
    text(0.1,0.5,'No empty room recording found for this subject.');
    ax = gca;
    ax.Visible = 'off';

    set(gcf, 'Color', 'w');
    export_fig(subject.saveplots.emptyroom,'-png');
end

end