function qcFrequency(data_cleaned, cfg, subject, behavior)
% qcFrequency Create plot of mean occipital frequency response to stimuli
% for quick quality control.

% Find different stimulus types
[stimtypes, stimtype] = getStimtypes(behavior);

% Create big plot over all stimulus types
stimPlot = figure('units', 'centimeters','position', [1 1 50 50], ...
    'color', 'w', 'Visible', 'off');
for iStimtype = 1:length(stimtypes)
    try
    % Find trials or trial snippets of this stim type - trial number might
    % not be the same as trial number during presentation if trials with
    % artifacts have been partially rejected.
    thisStimtype = stimtypes{iStimtype};
    stim_trials = find(strcmp(stimtype, thisStimtype));
    stim_trialparts = data_cleaned.trialinfo(...
        ismember(data_cleaned.trialinfo, stim_trials));
    [~,~,iStim_trials] = intersect(stim_trialparts, data_cleaned.trialinfo);
    
    % Select occipital channels
    occipital = regexp(data_cleaned.label, regexptranslate('wildcard', 'M*O*'));
    occipital = find(~cellfun(@isempty,occipital));
    occipital_channels = data_cleaned.label(occipital);
    
    % Cut baselines out of trials, select only relevant trials
    conf = [];
    conf.minlength = cfg.minLength;
    conf.toilim = cfg.toi_baseline;
    conf.trials = iStim_trials;
    data_base = ft_redefinetrial(conf, data_cleaned);
    
    % Cut stimulation interval out of trials, select only relevant trials
    conf = [];
    conf.minlength = cfg.minLength;
    conf.toilim = cfg.toi_stim;
    conf.trials = iStim_trials;
    data_stim = ft_redefinetrial(conf, data_cleaned);
    
    % Select only relevant trials
    conf = [];
    conf.minlength = cfg.minLength;
    conf.trials     = iStim_trials;
    iData_cleaned = ft_redefinetrial(conf, data_cleaned);
        
    % Get data for ERFs
    conf = [];
    conf.toilim = [-0.5 1];
    conf.minlength = cfg.minLength;
    conf.trials = iStim_trials;
    iERF = ft_redefinetrial(conf, data_cleaned);
    
    % Filter data for ERFs
    conf = [];
    conf.bpfilter = 'yes';
    conf.bpfiltord = 3;
    conf.bpfreq = [0.1 40];
    %conf.demean = 'yes';
    %conf.detrend = 'yes';
    iERF = ft_preprocessing(conf, iERF);
    
    % Compute ERFs
    conf = [];
    conf.vartrllength = 1;
    conf.keeptrials = 'yes';
    iERF = ft_timelockanalysis(conf, iERF);
    
    % Run FFT for baseline and stimulus interval
    conf            = [];
    conf.method     = 'mtmfft';
    conf.output     = 'pow';
    conf.channel    = occipital_channels;
    conf.taper      = cfg.spect.taper;
    conf.foi        = cfg.spect.foi;
    conf.tapsmofrq  = cfg.spect.tapsmofrq;
    conf.pad        = cfg.pad;
    freq_base = ft_freqanalysis(conf, data_base);
    freq_stim = ft_freqanalysis(conf, data_stim);
    
    % Run TFR over whole trial
    conf            = [];
    conf.method     = 'mtmconvol';
    conf.output     = 'pow';
    conf.channel    = 'MEG';
    conf.taper      = cfg.tfr.taper;
    conf.foi        = cfg.tfr.foi;
    conf.t_ftimwin  = cfg.tfr.n_cycle_win ./ conf.foi;
    conf.tapsmofrq  = cfg.tfr.tapsmofrq;
    conf.toi        = cfg.tfr.toi;
    conf.pad        = cfg.pad;
    TFR = ft_freqanalysis(conf, iData_cleaned);
    
    % Computes coherence between photodiode and visual cortex
    conf            = [];
    conf.method     = 'mtmfft';
    conf.output     = 'fourier';
    conf.taper      = 'dpss';
    conf.tapsmofrq  = 2;
    conf.foi        = cfg.ssvep.foi;
    conf.pad        = cfg.pad;
    conf.channel    = [occipital_channels; {'UADC003'}];
    conf.channelcmb = {'UADC003' 'all'};
    freq_photo = ft_freqanalysis(conf, data_stim);
        
    conf            = [];
    conf.method     = 'ppc';
    conf.channelcmb = {'UADC003' 'all'};
    connect_spec = ft_connectivityanalysis(conf, freq_photo);
    connect_spec = mean(connect_spec.ppcspctrm,1);
    
    % Compute change spectrum
    baseline = mean(freq_base.powspctrm,1);
    changefreq = bsxfun(@rdivide, freq_stim.powspctrm, baseline);
    changefreq = mean(changefreq,1);
       
    %% Plots
    subplot(length(stimtypes),5,iStimtype*5-4);
    pColor = cellfun(@str2num, strsplit(thisStimtype), 'un', 0);
    pColor = cell2mat(pColor(3:5))./255;
    if any(isnan(pColor)), pColor = [0 0 0]; end
    plot(freq_stim.freq, changefreq', 'Color', pColor);
    %ylim([0.5 2.85]);
    title(thisStimtype);
    xlabel('Frequency (Hz)');
    ylabel('Power increase');
    
    % plot topoplot of gammaband
    conf = [];
    conf.baseline     = cfg.toi_baseline;
    conf.baselinetype = cfg.baselinetype;
    conf.xlim         = cfg.topoplot.toi;
    conf.ylim         = cfg.gammaband;
    conf.layout       = 'CTF275_helmet.mat';
    conf.comment      = 'xlim';
    conf.commentpos   = 'title';
    subplot(length(stimtypes),5,iStimtype*5-3);
    ft_topoplotTFR(conf, TFR);
    
    % plot TFR
    conf = [];
    conf.baseline     = cfg.toi_baseline;
    conf.baselinetype = cfg.baselinetype;
    conf.channel      = occipital_channels;
    conf.title = 'Occipital channels';
    subplot(length(stimtypes),5,iStimtype*5-2);
    ft_singleplotTFR(conf, TFR);
    
    % plot photodiode-occipital "connectivity"
    subplot(length(stimtypes),5,iStimtype*5-1);
    plot(freq_photo.freq, connect_spec);
    title('SSVEP');
    xlabel('Frequency (Hz)');
    ylabel('PPC Photodiode-Occipital');
    
    % plot ERFs
    femto = 1e-15;
    trialdata = squeeze(ESI_nanmean(iERF.trial(:,occipital,:),2)) ./femto;
    bounds = 1.96.*((ESI_nanstd(trialdata)) ./ sqrt(size(trialdata,1)));
    femtoERF = squeeze(ESI_nanmean(trialdata,1));
    subplot(length(stimtypes),5,iStimtype*5);
    boundedline(iERF.time, femtoERF, bounds, 'alpha');
    title('ERF');
    xlabel('Time (s)');
    ylabel('Magnetic gradient (fT)');
    line([0 0], [-100 300], 'color', 'black', 'linestyle', '--')
    line([-0.5 1.5], [0 0], 'color', 'black', 'linestyle', '--')
    catch
        warning('No data for one stimulus type found.');
    end

end

% Save QC plot into dedicated folder.
outputfile = fullfile(cfg.plotpath, num2str(subject.subj_nr, '%03d'));
export_fig(stimPlot, outputfile, '-png', '-q101', '-r400');
end
