function [maxchange,maxloc] = qcMovement(cfg, subject, data)
% qcMovement Quality control for movement of MEG data. Needs circumcenter
% function, attached at end of qcMovement script. Needs export_fig
% toolbox. Creates output in form of a multi-page .pdf file, which contains
% 2D and 3D plots on head center movement and coil movement.
%
% data    = preprocessed fieldtrip data structure
% subject = subject structure created with fillSubjectInfo.m
% cfg     = config structure created with fillConfigs.m

% calculate the mean coil position per trial
ntrials = length(data.trial);

%preallocate space
coil1 = NaN(3, ntrials);
coil2 = NaN(3, ntrials);
coil3 = NaN(3, ntrials);

%find channels in data.trial
labels  = {data.label};
CNX_idx = find(strcmp('HLC0011',labels{1,1}(:,1)));
CNY_idx = find(strcmp('HLC0012',labels{1,1}(:,1)));
CNZ_idx = find(strcmp('HLC0013',labels{1,1}(:,1)));
CLX_idx = find(strcmp('HLC0021',labels{1,1}(:,1)));
CLY_idx = find(strcmp('HLC0022',labels{1,1}(:,1)));
CLZ_idx = find(strcmp('HLC0023',labels{1,1}(:,1)));
CRX_idx = find(strcmp('HLC0031',labels{1,1}(:,1)));
CRY_idx = find(strcmp('HLC0032',labels{1,1}(:,1)));
CRZ_idx = find(strcmp('HLC0033',labels{1,1}(:,1)));

%calculates mean x,y,z position of each of the three headcoils during each
%trial
for t = 1:ntrials
coil1(:,t) = [mean(data.trial{1,t}(CNX_idx,:)); ...
              mean(data.trial{1,t}(CNY_idx,:)); ...
              mean(data.trial{1,t}(CNZ_idx,:))];
coil2(:,t) = [mean(data.trial{1,t}(CLX_idx,:)); ...
              mean(data.trial{1,t}(CLY_idx,:)); ...
              mean(data.trial{1,t}(CLZ_idx,:))];
coil3(:,t) = [mean(data.trial{1,t}(CRX_idx,:)); ...
              mean(data.trial{1,t}(CRY_idx,:)); ...
              mean(data.trial{1,t}(CRZ_idx,:))];
end

%calculate the headposition and orientation per trial
%needs function circumcenter.m
cc = circumcenter(coil1, coil2, coil3);

%creates matrix with change relative to first head position
cc_rel = (cc - repmat(cc(:,1),1,size(cc,2)))';

%creates matrix with change relative to first coil position
coil1_rel = (coil1 - repmat(coil1(:,1),1,size(coil1,2)))';
coil2_rel = (coil2 - repmat(coil2(:,1),1,size(coil2,2)))';
coil3_rel = (coil3 - repmat(coil3(:,1),1,size(coil3,2)))';

%Maximum Location Changes in Numbers 
[HC_maxchange, HC_maxloc] = max(abs(cc_rel(:,1:3)*1000)); % in mm
[CN_maxchange, CN_maxloc] = max(abs(coil1_rel(:,1:3)*1000)); % in mm
[CL_maxchange, CL_maxloc] = max(abs(coil2_rel(:,1:3)*1000)); % in mm
[CR_maxchange, CR_maxloc] = max(abs(coil3_rel(:,1:3)*1000)); % in mm

maxchange = cat(1,HC_maxchange,CN_maxchange,CL_maxchange,CR_maxchange);
maxloc = cat(1,HC_maxloc,CN_maxloc,CL_maxloc,CR_maxloc);

%time variable for color
time = 1:1:ntrials;

%% PLOT

switch cfg.plotType
    case 'minimal'
        % plot head center movements
        HC_mvmt = figure('units','centimeter', 'position', [1 1 30 20]);
        set(HC_mvmt, 'Visible', 'off')
        HC_mvmt = plot(cc_rel(:,1:3)*1000);% in mm
        title('Head Movements along the x,y,z Axes');
        xlabel('Number of Trials');
        ylabel('Head Movement in mm');
        legend ('x Coordinates','y Coordinates','z Coordinates',...
            'Location', 'southwest');
        legend ('boxoff');
        HC_mvmt(1).Color = [0.2 0.8 0.8];
        HC_mvmt(2).Color = [0 0.5 1];
        HC_mvmt(3).Color = [1 0.2 0.2];
        ylim([-10 5]); xlim([0 ntrials]);
        hold on;
        dim = [.15 .5 .3 .4];
        format longg;
        str = sprintf('Max X = %d\nMax Y = %d\nMax Z = %d',maxchange(1,1),...
            maxchange(1,2), maxchange(1,3));
        annotation('textbox',dim,'String',str,'FitBoxToText','on');
        
        set(gcf, 'Color', 'w');
        export_fig(subject.saveplots.movement,'-png');
        
    case 'full'
        % Plot 2D Head Center Location
        
        % plot head center movements
        HC_mvmt = figure('units','centimeter', 'position', [1 1 30 20]);
        set(HC_mvmt, 'Visible', 'off')
        HC_mvmt = plot(cc_rel(:,1:3)*1000);% in mm
        title('Head Movements along the x,y,z Axes');
        xlabel('Number of Trials');
        ylabel('Head Movement in mm');
        legend ('x Coordinates','y Coordinates','z Coordinates',...
            'Location', 'southwest');
        legend ('boxoff');
        HC_mvmt(1).Color = [0.2 0.8 0.8];
        HC_mvmt(2).Color = [0 0.5 1];
        HC_mvmt(3).Color = [1 0.2 0.2];
        ylim([-10 5]); xlim([0 ntrials]);
        hold on;
        dim = [.15 .5 .3 .4];
        format longg;
        str = sprintf('Max X = %d\nMax Y = %d\nMax Z = %d',maxchange(1,1),...
            maxchange(1,2), maxchange(1,3));
        annotation('textbox',dim,'String',str,'FitBoxToText','on');
        
        set(gcf, 'Color', 'w');
        export_fig(subject.saveplots.movement,'-pdf');
        
        % plot head center rotations
        HC_rot = figure('units','centimeter', 'position', [1 1 30 20]);
        set(HC_rot, 'Visible', 'off')
        HC_rot = plot(cc_rel(:,4:6));
        title('Orientations to the x,y,z Axes');
        xlabel('Number of Trials')
        ylabel('Head Orientation in Degree')
        legend ('x Coordinates','y Coordinates','z Coordinates',...
            'Location', 'southwest');
        legend ('boxoff');
        HC_rot(1).Color = [0.2 0.8 0.8];
        HC_rot(2).Color = [0 0.5 1];
        HC_rot(3).Color = [1 0.2 0.2];
        ylim([-1 1]); xlim([0 ntrials]);
        hold on;
        
        set(gcf, 'Color', 'w');
        export_fig(subject.saveplots.movement, '-pdf', '-append')
        
        % Plot 2D Coil Locations
        
        %plot with subplots
        coil_mvmt = figure('units','centimeter', 'position', [1 1 30 20]);
        set(coil_mvmt, 'Visible', 'off')
        
        subplot (3,1,1)
        CN_mvmt = plot(coil1_rel*1000);% in mm
        title('Nasion');
        xlabel('Number of Trials')
        ylabel('Movement in mm')
        legend ('x coordinates','y coordinates','z coordinates',...
            'Location', 'southwest');
        legend ('boxoff');
        ylim([-10 5]); xlim([0 ntrials]);
        CN_mvmt(1).Color = [0.2 0.8 0.8];
        CN_mvmt(2).Color = [0 0.5 1];
        CN_mvmt(3).Color = [1 0.2 0.2];
        
        subplot (3,1,2)
        CL_mvmt = plot(coil2_rel*1000);% in mm
        title('Left Ear');
        xlabel('Number of Trials')
        ylabel('Movement in mm')
        legend ('x coordinates','y coordinates','z coordinates',...
            'Location', 'southwest');
        legend ('boxoff');
        ylim([-10 5]); xlim([0 ntrials]);
        CL_mvmt(1).Color = [0.2 0.8 0.8];
        CL_mvmt(2).Color = [0 0.5 1];
        CL_mvmt(3).Color = [1 0.2 0.2];
        
        subplot (3,1,3)
        hold on
        CR_mvmt = plot(coil3_rel*1000);% in mm
        title('Right Ear');
        xlabel('Number of Trials')
        ylabel('Movement in mm')
        legend ('x coordinates','y coordinates','z coordinates',...
            'Location', 'southwest');
        legend ('boxoff');
        ylim([-10 5]); xlim([0 ntrials]);
        CR_mvmt(1).Color = [0.2 0.8 0.8];
        CR_mvmt(2).Color = [0 0.5 1];
        CR_mvmt(3).Color = [1 0.2 0.2];
        hold on;
        
        set(gcf, 'Color', 'w');
        export_fig(subject.saveplots.movement, '-pdf', '-append')
        
        % Plot 3D Locations
        
        %plot coordinates without time dimension
        mvmt = figure('units','centimeter', 'position', [1 1 30 20]);
        set(mvmt, 'Visible', 'off')
        
        subplot(2,2,1);
        scatter3(coil2_rel(:,1)*1000,coil2_rel(:,2)*1000,coil2_rel(:,3)*1000,5,time);
        colormap (jet(7));
        colorbar('Direction','reverse');
        title('Left Ear'); xlabel('x'); ylabel('y'); zlabel('z movements in mm');
        xlim([-3 3]); ylim([-3 3]); zlim([-10 3]);
        az = 160; el = 15; view(az, el);
        hold on
        
        subplot(2,2,2);
        scatter3(coil3_rel(:,1)*1000,coil3_rel(:,2)*1000,coil3_rel(:,3)*1000,5,time);
        colormap (jet(7));
        colorbar('Direction','reverse');
        title('Right Ear'); xlabel('x'); ylabel('y'); zlabel('z movements in mm')
        xlim([-3 3]); ylim([-3 3]); zlim([-10 3]);
        az = 160; el = 15; view(az, el);
        hold on
        
        subplot(2,2,3);
        scatter3(coil1_rel(:,1)*1000,coil1_rel(:,2)*1000,coil1_rel(:,3)*1000,5,time);
        colormap (jet(7));
        colorbar('Direction','reverse');
        title('Nasion'); xlabel('x'); ylabel('y'); zlabel('z movements in mm')
        xlim([-3 3]); ylim([-3 3]); zlim([-10 3])
        az = 160; el = 15; view(az, el);
        hold on
        
        subplot(2,2,4)
        scatter3(cc_rel(:,1)*1000,cc_rel(:,2)*1000,cc_rel(:,3)*1000,5,time);
        colormap (jet(7));
        colorbar('Direction','reverse');
        title('Head Center'); xlabel('x'); ylabel('y'); zlabel('z movements in mm')
        xlim([-3 3]); ylim([-3 3]); zlim([-10 3])
        az = 160; el = 15; view(az, el);
        hold on
        
        set(gcf, 'Color', 'w');
        export_fig(subject.saveplots.movement, '-pdf', '-append')
        
    otherwise
        warning('Unexpected plot type. No plot created.')     
end

%% Circumcenter function

function [cc] = circumcenter(coil1,coil2,coil3)
 
% CIRCUMCENTER determines the position and orientation of the circumcenter
% of the three fiducial markers (MEG headposition coils). 
%
% Input: X,y,z-coordinates of the 3 coils [3 X N],[3 X N],[3 X N] where N
% is timesamples/trials.
%
% Output: X,y,z-coordinates of the circumcenter [1-3 X N], and the 
% orientations to the x,y,z-axes [4-6 X N].
%
% A. Stolk, 2012
 
% number of timesamples/trials
N = size(coil1,2);
 
%% x-, y-, and z-coordinates of the circumcenter
% use coordinates relative to point `a' of the triangle
xba = coil2(1,:) - coil1(1,:);
yba = coil2(2,:) - coil1(2,:);
zba = coil2(3,:) - coil1(3,:);
xca = coil3(1,:) - coil1(1,:);
yca = coil3(2,:) - coil1(2,:);
zca = coil3(3,:) - coil1(3,:);
 
% squares of lengths of the edges incident to `a'
balength = xba .* xba + yba .* yba + zba .* zba;
calength = xca .* xca + yca .* yca + zca .* zca;
 
% cross product of these edges
xcrossbc = yba .* zca - yca .* zba;
ycrossbc = zba .* xca - zca .* xba;
zcrossbc = xba .* yca - xca .* yba;
 
% calculate the denominator of the formulae
denominator = 0.5 ./ (xcrossbc .* xcrossbc + ycrossbc .* ycrossbc + zcrossbc .* zcrossbc);
 
% calculate offset (from `a') of circumcenter
xcirca = ((balength .* yca - calength .* yba) .* zcrossbc - (balength .* zca - calength .* zba) .* ycrossbc) .* denominator;
ycirca = ((balength .* zca - calength .* zba) .* xcrossbc - (balength .* xca - calength .* xba) .* zcrossbc) .* denominator;
zcirca = ((balength .* xca - calength .* xba) .* ycrossbc - (balength .* yca - calength .* yba) .* xcrossbc) .* denominator;
 
cc(1,:) = xcirca + coil1(1,:);
cc(2,:) = ycirca + coil1(2,:);
cc(3,:) = zcirca + coil1(3,:);
 
%% orientation of the circumcenter with respect to the x-, y-, and z-axis
% coordinates
v = [cc(1,:)', cc(2,:)', cc(3,:)'];
vx = [zeros(1,N)', cc(2,:)', cc(3,:)']; % on the x-axis
vy = [cc(1,:)', zeros(1,N)', cc(3,:)']; % on the y-axis
vz = [cc(1,:)', cc(2,:)', zeros(1,N)']; % on the z-axis
 
for j = 1:N
  % find the angles of two vectors opposing the axes
  thetax(j) = acos(dot(v(j,:),vx(j,:))/(norm(v(j,:))*norm(vx(j,:))));
  thetay(j) = acos(dot(v(j,:),vy(j,:))/(norm(v(j,:))*norm(vy(j,:))));
  thetaz(j) = acos(dot(v(j,:),vz(j,:))/(norm(v(j,:))*norm(vz(j,:))));
 
  % convert to degrees
  cc(4,j) = (thetax(j) * (180/pi));
  cc(5,j) = (thetay(j) * (180/pi));
  cc(6,j) = (thetaz(j) * (180/pi));
end
end

end