function [ cfg ] = fillConfigs(subject, basepath)
% fillConfigs Set all your general settings here. Returns a cfg struct.
% Input: A subject info struct as returned by fillSubjectInfo()

%% General settings
% Use github version of fieldtrip instead of the ESI internal one?
cfg.general.useGithub = true;

% preprocess raw data or load preprocessed data from disk?
cfg.general.run_preprocessing = false; 
cfg.general.run_peakdetection = false;
cfg.general.doQC = true;
cfg.general.do_frequency_QC = false;
cfg.general.save_partial_results = true;
cfg.general.do_manual_artifact_rejection = false;
cfg.general.do_saved_artifact_rejection = ~cfg.general.do_manual_artifact_rejection;
cfg.general.rejectTrials = 'partial'; % partial, none, or complete
cfg.general.reject_bad_channels = true; % should meg-/eye-channels markes as bad/empty be removed?
cfg.general.reject_nonresponses = true; % reject trials without responses?
cfg.general.reject_nonfixations = true; % reject trials without fixation at baseline?
cfg.general.reject_interventions = true; % reject trials which were paused by experimenter?
cfg.general.reject_invalid_RTs  = true; % reject trials in which subject responded too early?
cfg.general.writeLog = false; % write matlab output into file for overnight runs
cfg.general.control_headmovements = true; % Use avg head position as basis for CTF coordinates
cfg.general.HCP_or_grid_sources = 'HCP'; % can be 'HCP' or 'grid' - which sourcemodel do we want?
cfg.general.fsample = 1200;
cfg.general.wholetrial = [-1 1.3-(1/cfg.general.fsample)];
cfg.general.areaROIs =  [1 4 5 13 19 158 6 156 20 21 22 23 163 2 ...
    157 118 16 17 48 49 95 42 45 10 84]; % the Michalareas 2016 set
cfg.general.ERFcomps = [0.057 0.094];
cfg.general.earlyLate = [11:50; 81:120];
cfg.general.subsampleV1 = 0.0625; % How many of the dipoles in V1 do we want?
cfg.general.plotTrialBinsize = 5;

cfg.paths.base = basepath;
cfg.paths.analysis = fullfile(basepath, 'Analysis');
cfg.paths.data = fullfile(basepath, 'Data');
cfg.paths.code = fullfile(cfg.paths.analysis, 'Code');
cfg.paths.plot = fullfile(cfg.paths.analysis, 'Plots');
cfg.paths.util = fullfile(cfg.paths.code, 'Utility');

cfg.channels.photodiode     = 'UADC003';  
cfg.channels.responsebox    = 'UPPT002';  
cfg.channels.eyemovement    = {'UADC005', 'UADC006', 'UADC008', 'UADC009'};
cfg.channels.eyemovementL   = {'UADC005', 'UADC006'};
cfg.channels.eyemovementR   = {'UADC008', 'UADC009'};
cfg.channels.pupilsize      = {'UADC007', 'UADC010'};
cfg.channels.pupilsizeL     = 'UADC007';
cfg.channels.pupilsizeR     = 'UADC010';

% Eyetracking setuo
cfg.eyetracker.distanceCm = 55; % setting in ini file of eyetracking pc
cfg.eyetracker.screenwidthCm = 42; % setting in ini file of eyetracking pc
cfg.eyetracker.screenheightCm = 23.9; % setting in ini file of eyetracking pc
cfg.eyetracker.screenwidthPx = 1920; % setting in ini file of eyetracking pc
cfg.eyetracker.screenheightPx = 1080; % setting in ini file of eyetracking pc
cfg.eyetracker.convFactorX = 264.1316; % Conversion factor of V_x -> pixels_x of the DAC
cfg.eyetracker.convFactorY = 152.9514; % Conversion factor of V_y -> pixels_y of the DAC


%% Gamma peak detection
cfg.gammapeak.computeSourceLevel = true; % set to true once LCMV has been run
cfg.gammapeak.range = [25 95]; % Range in which gamma peak is searched for.
cfg.gammapeak.filepath = cfg.paths.util; % Path for temporary files.
cfg.gammapeak.plotpath = fullfile(cfg.paths.plot, 'Gamma_Peak_Fits'); 
cfg.gammapeak.minLength = 0.5; % min length of data snippets for freq analysis
cfg.gammapeak.toi_baseline = [-1 -0.1-1/1200];
cfg.gammapeak.toi_stim = [0.3 1.3-1/1200];
cfg.gammapeak.fallback = 50; % What to use if we don't have a clear gamma peak?

%% Preprocessing
% Done during trial cutting
cfg.preprocessing.dataset       = subject.megdata;
cfg.preprocessing.channel       = {'meg'};% 'EEG057', 'EEG058'};
cfg.preprocessing.demean        = 'no';
cfg.preprocessing.detrend       = 'no';
cfg.preprocessing.continuous    = 'yes';
cfg.preprocessing.dftfreq       = [49.8 :0.1: 50.2 ...
                                   99.8 :0.1: 100.2...
                                   149.8 :0.1: 150.2...
                                   199.8 :0.1: 200.2...
                                   249.8 :0.1: 250.2...
                                   299.8 :0.1: 300.2...
                                   349.8 :0.1: 350.2...
                                   399.8 :0.1: 400.2];
%59.8:0.1:60.2
cfg.preprocessing.dftfilter     = 'yes';
cfg.preprocessing.padding       = 5; % in seconds
cfg.preprocessing.padtype       = 'data';

% Eyetracker, photodiode, HLCs
cfg.otherpreproc.dataset        = cfg.preprocessing.dataset;
cfg.otherpreproc.channel        = {'UADC005', 'UADC006', 'UADC007'...
                                   'UADC008', 'UADC009', 'UADC010',...
                                   'UADC003', ... %photodiode
                                   'UPPT001', ... %trigger
                                   'HLC0011','HLC0012','HLC0013', ...
                                   'HLC0021','HLC0022','HLC0023', ...
                                   'HLC0031','HLC0032','HLC0033'};
cfg.otherpreproc.continuous     = 'yes';

cfg.trials.dataset              = subject.megdata;
cfg.trials.trialfun             = 'trialfun_full';

if strcmp(subject.subj_nr, '050') % no triggers recorded for subject 50
    cfg.trials.trialfun = 'trialfun_photo'; 
    warning('Using photodiode trialfun!');
elseif strcmp(subject.subj_nr, '019')
    cfg.trials.trialfun = 'trialfun_19'; 
    warning('Using 019-specific trialfun!');
end
cfg.trials.trialdef.min_length  = 1.3; % in seconds
cfg.trials.stim_on_delay = 13; % how many samples does the stimulus take to come on?
cfg.trials.stim_off_delay = 13; % how many samples does it take to change?
cfg.trials.baseline = subject.stim_cfg{1}.experiment.baselinelength;
% 24/29 for old projector

%% Quality control plots
cfg.qc_diode.triggerChan = 'UPPT001'; %label of trigger channel
cfg.qc_diode.diodeChan   = cfg.channels.photodiode;
cfg.qc_diode.trigger_on  = 40; 
cfg.qc_diode.trigger_off = 120; 
cfg.qc_diode.plotwindow  = 150; % how many samples are plotted around triggers?
cfg.qc_diode.delay_on    = cfg.trials.stim_on_delay; %expected delay; draws vertical line at this value
cfg.qc_diode.delay_off   = cfg.trials.stim_off_delay; 
cfg.qc_diode.flip_axis   = true;

cfg.qc_emptyroom.dataset        = subject.emptyroomdata; %preprocess raw ER data
cfg.qc_emptyroom.channel        = {'meg'};
cfg.qc_emptyroom.continuous     = 'yes';
cfg.qc_emptyroom.demean         = 'yes';
cfg.qc_emptyroom.detrend        = 'yes';

cfg.qc_emptyroom.do_manual_ar   = false;  %false, if no manual artifact
                                         %rejection is wanted
                                          
cfg.qc_emptyroom.manual_ar.dataset              = cfg.qc_emptyroom.dataset;
cfg.qc_emptyroom.manual_ar.trialdef.triallength = 9;
cfg.qc_emptyroom.manual_ar.demean               = 'yes';
cfg.qc_emptyroom.manual_ar.detrend              = 'yes';
cfg.qc_emptyroom.manual_ar.channel              = {'meg'};
                                          
cfg.qc_emptyroom.freqs.method   = 'mtmfft';
cfg.qc_emptyroom.freqs.output   = 'pow';
cfg.qc_emptyroom.freqs.foi      = 1:1:600;
cfg.qc_emptyroom.freqs.taper    = 'hanning';
cfg.qc_emptyroom.freqs.allmeg   = {'meg'};
cfg.qc_emptyroom.freqs.coi      = {'MLO11';'MLO12';'MLO13';'MLO14';...
   'MLO21';'MLO22';'MLO23';'MLO24';'MLO31';'MLO32';'MLO33';'MLO34';...
   'MLO41';'MLO42';'MLO43';'MLO44';'MLO51';'MLO52';'MLO53';'MRO11';...
   'MRO12';'MRO13';'MRO14';'MRO22';'MRO23';'MRO24';'MRO31';'MRO32';...
   'MRO33';'MRO34';'MRO41';'MRO42';'MRO43';'MRO44';'MRO51';'MRO52';...
   'MRO53'};  %sensors of channels of interest
cfg.qc_emptyroom.freqs.coiLabel = 'Occipital Sensors'; %label of COIs
cfg.qc_emptyroom.freqs.trials   = 'all';
                                     
cfg.qc_movement.plotType = 'full'; %can be 'minimal' or 'full'

cfg.qc_frequency.minLength = 0.5; % min length of data snippets for freq analysis
cfg.qc_frequency.toi_baseline = [-1 -0-1/1200];
cfg.qc_frequency.toi_stim = [0.3 1.3-1/1200];
cfg.qc_frequency.gammaband = [40 70]; % gamma band for topoplot
cfg.qc_frequency.baselinetype = 'relchange'; %relative or relchange, for all plots
cfg.qc_frequency.pad = 4; % number / nextpow2 / maxperlen
cfg.qc_frequency.spect.taper = 'hann'; % taper for change spectrum plot
cfg.qc_frequency.spect.tapsmofrq = 4; % spectral smoothing if taper = 'dpss'
cfg.qc_frequency.spect.foi = 2:0.5:150;
cfg.qc_frequency.tfr.taper = 'hann'; % taper for TFR plot
cfg.qc_frequency.tfr.tapsmofrq = round(0.3 *cfg.qc_frequency.spect.foi);
cfg.qc_frequency.tfr.n_cycle_win = 5; % window length in cycles pre frequency
cfg.qc_frequency.tfr.foi = 2:2:100;
cfg.qc_frequency.tfr.toi = [min(cfg.qc_frequency.toi_baseline) :0.05:...
                            max(cfg.qc_frequency.toi_stim)];
cfg.qc_frequency.topoplot.toi = [0.4 0.9];
cfg.qc_frequency.plotpath = fullfile(cfg.paths.plot, 'Frequency_response');

%% Artifact rejection
cfg.eyechannels = {'UADC005', 'UADC006', 'UADC007', 'UADC008', 'UADC009', 'UADC010'};

cfg.jumpdetect.datafile                       = subject.megdata;
cfg.jumpdetect.headerfile                     = subject.megdata;
cfg.jumpdetect.continuous                     = 'yes';
cfg.jumpdetect.artfctdef.zvalue.channel       = 'MEG';
cfg.jumpdetect.artfctdef.zvalue.cutoff        = 20;
cfg.jumpdetect.artfctdef.zvalue.trlpadding    = 0;
cfg.jumpdetect.artfctdef.zvalue.artpadding    = 0;
cfg.jumpdetect.artfctdef.zvalue.fltpadding    = 0;
cfg.jumpdetect.artfctdef.zvalue.cumulative    = 'yes';
cfg.jumpdetect.artfctdef.zvalue.medianfilter  = 'yes';
cfg.jumpdetect.artfctdef.zvalue.medianfiltord = 9;
cfg.jumpdetect.artfctdef.zvalue.absdiff       = 'yes';
cfg.jumpdetect.artfctdef.zvalue.interactive   = 'yes';
cfg.jumpdetect.artfctdef.zvalue.detrend       = 'yes';
cfg.jumpdetect.artfctdef.zvalue.demean        = 'yes';

cfg.muscledetect.datafile                     = subject.megdata;
cfg.muscledetect.headerfile                   = subject.megdata;
cfg.muscledetect.continuous                   = 'yes';
cfg.muscledetect.artfctdef.zvalue.channel     = 'MEG';
cfg.muscledetect.artfctdef.zvalue.cutoff      = 9;
cfg.muscledetect.artfctdef.zvalue.trlpadding  = 0;
cfg.muscledetect.artfctdef.zvalue.fltpadding  = 0;
cfg.muscledetect.artfctdef.zvalue.artpadding  = 0.1;
cfg.muscledetect.artfctdef.zvalue.bpfilter    = 'yes';
cfg.muscledetect.artfctdef.zvalue.bpfreq      = [110 140];
cfg.muscledetect.artfctdef.zvalue.bpfiltord   = 9;
cfg.muscledetect.artfctdef.zvalue.bpfilttype  = 'but';
cfg.muscledetect.artfctdef.zvalue.hilbert     = 'yes';
cfg.muscledetect.artfctdef.zvalue.boxcar      = 0.2;
cfg.muscledetect.artfctdef.zvalue.interactive = 'yes';
cfg.muscledetect.artfctdef.zvalue.detrend     = 'yes';
cfg.muscledetect.artfctdef.zvalue.demean      = 'yes';

cfg.blinkdetect.datafile                      = subject.megdata;
cfg.blinkdetect.headerfile                    = subject.megdata;
cfg.blinkdetect.continuous                    = 'yes';
cfg.blinkdetect.artfctdef.zvalue.channel      = cfg.channels.eyemovement;
cfg.blinkdetect.artfctdef.zvalue.cutoff       = 5;
cfg.blinkdetect.artfctdef.zvalue.trlpadding   = 0;
cfg.blinkdetect.artfctdef.zvalue.artpadding   = 0.1;
cfg.blinkdetect.artfctdef.zvalue.fltpadding   = 0.2;
cfg.blinkdetect.artfctdef.zvalue.interactive  = 'yes';
cfg.blinkdetect.artfctdef.zvalue.rectify       = 'yes';

%% Beamform
cfg.beam.template.file = '/opt/fieldtrip_github/external/spm8/templates/T1.nii';
cfg.beam.template.coordsystem                      = 'spm';
cfg.beam.template.headmodel.method                 = 'singleshell';
cfg.beam.template.sourcemodel.grid.resolution      = 0.5;
cfg.beam.template.sourcemodel.inwardshift          = -2;

%BEAMFORM
%prepare models
%step 1: realign MRI (options for ft_volumerealign)
% Realign by hand or load existing realignment?
cfg.beam.mri_realign_by_hand            = false; 
cfg.beam.realign.method                 = 'interactive';
cfg.beam.realign.coordsys               = 'ctf';
%step 2: segment MRI (options for ft_volumesegment)
cfg.beam.segment.output                 = 'brain';
%step 3: prepare headmodel (options for ft_prepare_headmodel)
cfg.beam.headmodel.method               = 'singleshell';
%step 4: prepare sourcemodel (options for ft_prepare_sourcemodel)
cfg.beam.sourcemodel.grid.warpmni       = 'yes';
cfg.beam.sourcemodel.grid.nonlinear     = 'yes';
cfg.beam.sourcemodel.grid.units         = 'cm';
%DICS beamformer
%step 1: redefine trials for freq analysis (options for ft_redefinetrial)
cfg.beam.slidingwindow           = true;
cfg.beam.window_length           = 1; % for gamma
cfg.beam.window_length_short     = 0.5; % for gamma
cfg.beam.window_overlap          = 0.5;
cfg.beam.defall.minlength        = 1; %the whole trial
cfg.beam.defbase.minlength       = 1; %baseline
cfg.beam.defbase.toilim          = [-1 -0-1/cfg.general.fsample];
cfg.beam.defstim.minlength       = 1; %actual trial, stimulus shown
cfg.beam.defstim.toilim          = [0.3 1.3-1/cfg.general.fsample];

%step 2: do frequency analysis (options for ft_freqanalysis)
cfg.beam.freq.channel            = 'meg';
cfg.beam.freq.method             = 'mtmfft';
cfg.beam.freq.taper              = 'dpss';
cfg.beam.freq.output             = 'powandcsd';
cfg.beam.freq.tapsmofrq          = 3;
cfg.beam.freq.foilim             = [1 100];
cfg.beam.freq.pad                = 1;
%step 3: sourceanalysis (options for ft_sourceanalysis)
cfg.beam.beamdics.method           = 'dics';
cfg.beam.lambda                    = '0%'; % for DICS and LCMV
cfg.beam.beamdics.dics.keepfilter  = 'yes';
cfg.beam.beamdics.dics.realfilter  = 'yes';
cfg.beam.beamdics.dics.fixedori    = 'yes';
%step 4: interpolate (options for ft_sourceinterpolate and ft_sourceplot)
cfg.beam.int.parameter             = 'avg.pow';
cfg.beam.plot.method               = 'ortho'; %or 'slice'
%find gamma peak location
%step 1: load atlas
cfg.beam.mask.atlasname = fullfile...
    ('/opt/fieldtrip_github/template/atlas/aal/ROI_MNI_V4.nii');
%step 2: create masks for rois (options for ft_volumelookup)
cfg.beam.mask.inputcoord = 'mni';
cfg.beam.mask.spec_roi{1,1} = 'left_hem';
cfg.beam.mask.spec_roi{1,2} = (1:2:107); %mask for left hemisphere
cfg.beam.mask.spec_roi{2,1} = 'right_hem';
cfg.beam.mask.spec_roi{2,2} = (2:2:108); %mask for right hemisphere
cfg.beam.mask.spec_roi{3,1} = 'left_occ';
cfg.beam.mask.spec_roi{3,2} = (43:2:53); %mask for left OCC cortex
cfg.beam.mask.spec_roi{4,1} = 'right_occ';
cfg.beam.mask.spec_roi{4,2} = (44:2:54); %mask for right OCC cortex
cfg.beam.mask.spec_roi{5,1} = 'all';
cfg.beam.mask.spec_roi{5,2} = (1:1:116); %the whole brain
%LCMV beamformer
%step 1: cut out ERF (options for ft_redefinetrial)
cfg.beam.lcmv.toilim            = [-1 1.3-1/1200];
%step 2: timelocked analysis (options for ft_timelockanalysis)
cfg.beam.lcmv.channel          = 'meg';
%step 3: lcmvb beamformer (options for ft_sourceanalysis)
cfg.beam.lcmv.method           = 'lcmv';
cfg.beam.lcmv.keepfilter       = 'yes';

% Freesurfer -> HCP -> Sourcemodel pipeline
cfg.beam.hcp.redo_realign    = false; % Do the realignment to APC by hand?
hcp_folder = fullfile(basepath, 'Data', 'Freesurfed', 'HCP');
cfg.beam.hcp.atlas = fullfile(hcp_folder, ['Q1-Q6_RelatedValidation210.'...
    'CorticalAreas_dil_Final_Final_Areas_Group_Colors.32k_fs_LR.dlabel.nii']);
% The plot brain template can be midthickness, inflated or very_inflated
plot_template = 'very_inflated';
cfg.beam.hcp.plot_template = {...
    [hcp_folder '/Q1-Q6_RelatedValidation210.L.' plot_template ...
    '_MSMAll_2_d41_WRN_DeDrift.32k_fs_LR.surf.gii'],...
    [hcp_folder '/Q1-Q6_RelatedValidation210.R.' plot_template ...
    '_MSMAll_2_d41_WRN_DeDrift.32k_fs_LR.surf.gii']};
cfg.beam.hcp.midthickness = {...
    [hcp_folder '/Q1-Q6_RelatedValidation210.L.' 'midthickness' ...
    '_MSMAll_2_d41_WRN_DeDrift.32k_fs_LR.surf.gii'],...
    [hcp_folder '/Q1-Q6_RelatedValidation210.R.' 'midthickness' ...
    '_MSMAll_2_d41_WRN_DeDrift.32k_fs_LR.surf.gii']};
cfg.beam.hcp.sphere = {...
    [hcp_folder '/Q1-Q6_RelatedValidation210.L.sphere.32k_fs_LR.surf.gii'],...
    [hcp_folder '/Q1-Q6_RelatedValidation210.R.sphere.32k_fs_LR.surf.gii']};
%% Compute gamma freqsment
%GENERAL: 
% Should gamma peaks be determined for each unique stimulus type?
cfg.freqs.stimspec_gamma            = false;
% Which spectrum bandwidth and frequency resolution do we want for the change 
% spectra?
cfg.freqs.bandwidth                 = 60; 
cfg.freqs.resolution                = 0.5; %in Hz

%step 1: which dipoles are of interest? see select_HCP_gamma_peak for an
% explanation
cfg.freqs.doi_restriction        = 'highestPower V1V2';
%step 2: cut out ERF (options for ft_redefinetrial)
cfg.freqs.minimum_trial_period   = true; % kick out trials with artifacts in early stim period
cfg.freqs.minimum_trial_time     = [-0.2 0.5-1/1200]; % early stim period
cfg.freqs.base.toilim            = [-1  0-1/1200];
cfg.freqs.stim.toilim            = [0.3 1.3-1/1200];
cfg.freqs.minlength              = 0.5;
% step 2b: recut data into welch windows
cfg.freqs.sliding_window         = true; 
cfg.freqs.window_length_long     = 0.5; % for theta, alpha 0.5
cfg.freqs.window_length_short    = 1/3; % for beta, gamma 0.2
cfg.freqs.window_overlap         = 0.5;
% alternative version for spectra (windows need to be longer)
cfg.freqs.window_length_long     = 0.5;
%step 5: run frequency analysis (options for ft_freqanalysis)
cfg.freqs.freq.taper             = 'dpss';
cfg.freqs.freq.tapsmofrq         = 6;
cfg.freqs.freq.polyremoval       = 1;
%plot
cfg.freqs.bins                   = 4; % only for behavioral data
cfg.freqs.binsize                = 31; 
% behavior
% should we plot all difficulties or should we collapse symmetric difficulties?
cfg.freqs.fullPsychoCurve           = false; 

%% Compute general spectra
cfg.spectra.minlength = 1;
cfg.spectra.all.toilim             = [-1.1 1.3-1/1200];
cfg.spectra.base.toilim            = cfg.freqs.base.toilim;
cfg.spectra.stim.toilim            = cfg.freqs.stim.toilim;
% cut data into welch windows
cfg.spectra.sliding_window         = true; 
cfg.spectra.window_length_long     = 0.5; 
cfg.spectra.window_length_short    = 1/3; 
cfg.spectra.window_overlap         = 0.5;
cfg.spectra.pad                    = 4;
cfg.spectra.foi_hanning            = [2 20];
cfg.spectra.foi_dpss               = [21 120];
cfg.spectra.tapsmofrq              = 6;
cfg.spectra.polyremoval            = 1;

% Saccade detection
cfg.ms.debugplot = false;
cfg.ms.speedAlgo = 3; % 1/2/3, higher numbers for higher sampling rate eye data
cfg.ms.downsampleFs = 1200; % should be 600 or 1200. speedAlgo needs to be adapted to this.
cfg.ms.toilim = cfg.freqs.stim.toilim;
cfg.ms.amplMax = 6; % maximum amplitude of detected saccades in degree
cfg.ms.MinDur = 6; % in samples
cfg.ms.velThresh = 6.5; % 5 for clean, 6/7 for more noisy data

% Theta-based MTL analysis
cfg.theta.thetaareas = {'STSda', 'STSva', 'PH'};

% Decoding and distance analyses
cfg.decoder.useTrialAverages = true;
cfg.decoder.nAverages = 2; % how many average trials per trial average?

%% If requested: turn on logfile
if cfg.general.writeLog 
    logpath = fullfile(cfg.paths.analysis, 'log');
    diary(logpath); 
end
end

