function subject = fillSubjectInfo( subj_nr, basepath )
%fillSubjectInfo Look up and generate all subject-specific infos and paths.

subject.basepath = basepath;
subject.subj_nr = subj_nr;

if isnumeric(subj_nr)
    subject.path = fullfile(subject.basepath, 'Data', num2str(subj_nr, '%03d'));
    subject.subj_nr = num2str(subj_nr, '%03d');
elseif ischar(subj_nr)
    warning(['SubjNr should normally be numeric. Are you sure you are '...
        'not doing something stupid?']);
    subject.path = fullfile(subject.basepath, 'Data', subj_nr);
end

% Set path to MEG data
datadir = dir([subject.path filesep '*ds']);
if size(datadir,1) ~= 3, warning('I haven''t found three datasets.'); end
for iDataset = 1:size(datadir,1)
    subject.megdata{iDataset} = fullfile(subject.path, datadir(iDataset).name);
end
subject.preproc_data = fullfile(subject.path, 'Preprocessed');

% Set path to MRI T1 data
mripath = strcat(subject.path, filesep, 'MRI');
datadir = dir([mripath filesep '*MPRAGE*nii.gz']);
if size(datadir,1) == 1
    subject.mridata = fullfile(mripath, datadir.name);
else
    warning(' I haven''t found one T1 MRI.')
end

% Set path to MRI T2 data
mripath = strcat(subject.path, filesep, 'MRI');
datadir = dir([mripath filesep '*TSE*nii.gz']);
if size(datadir,1) == 1
    subject.mridataT2 = fullfile(mripath, datadir.name);
else
    warning(' I haven''t found one T2 MRI.')
end

% Set path to the freesurfer/HCP cortical source grids
surfnameL = sprintf('%s.L.midthickness.32k_fs_LR.surf.gii', ...
    subject.subj_nr);
surfnameR = sprintf('%s.R.midthickness.32k_fs_LR.surf.gii', ...
    subject.subj_nr);

subject.HCPLsurfpath = fullfile(subject.basepath, 'Data', 'Freesurfed',...
    subject.subj_nr, 'surf', surfnameL);
subject.HCPRsurfpath = fullfile(subject.basepath, 'Data', 'Freesurfed',...
    subject.subj_nr, 'surf', surfnameR);

% Set path to emptyroom data
try
    emptyroompath = strcat(subject.path,filesep,'Empty_room');
    assert((exist(emptyroompath,'file')==7), ...
        'No empty room recording found for this subject');
    datadir = dir([emptyroompath filesep '*ds']);
    subject.emptyroomdata = fullfile(emptyroompath, datadir.name);
catch
    subject.emptyroompath = 'Error: Not Existent';
    subject.emptyroomdata = 'Error: Not Existent';
    warning('No empty room recording found.');
end

% Add the data saved on the stimulus PC
datadir = dir([subject.path filesep 'Behavior*']);
if size(datadir,1) ~= 3
    warndlg('There should be three behavioral datasets!');
end

for iDataset = 1:size(datadir,1)
    behavPath = fullfile(subject.path, datadir(iDataset).name);
    ESIload(behavPath);
    behavior{iDataset} = out.results;
    subject.stim_cfg{iDataset} = out.cfg;
    subject.meg_info{iDataset} = out.sessionInfo;
end

stim1 = subject.stim_cfg{1};
stim1.experiment = rmfield(subject.stim_cfg{1}.experiment, 'trialOrder');
stim2 = subject.stim_cfg{2};
stim2.experiment = rmfield(subject.stim_cfg{2}.experiment, 'trialOrder');
assert(isequal(stim1, stim2), 'Stimulus config changed between blocks!');

behavior{1}.block = ones(size(behavior{1}, 1), 1);
behavior{2}.block = 2 * ones(size(behavior{2}, 1), 1);
subject.colorBehav = vertcat(behavior{1}, behavior{2});
% fix trial numbers over blocks
try
    behavior{3}.block = 3 * ones(size(behavior{3}, 1), 1);
    subject.contrastBehav = behavior{3};
    subject.behavior = vertcat(subject.colorBehav, subject.contrastBehav);
    subject.behavior.trialNr = [1:length(subject.behavior.trialNr)]';
catch
    warning('No contrast behavioral dataset found!');
    subject.behavior = subject.colorBehav;
    subject.behavior.trialNr = [1:length(subject.behavior.trialNr)]';
end

% Add paths for saving the plots
subject.saveplots.basepath  = fullfile(basepath, 'Analysis', 'Plots');
subject.saveplots.emptyroom = fullfile(subject.saveplots.basepath,...
    'EmptyRoom', ['EmptyRoom' subject.subj_nr]);
subject.saveplots.movement = fullfile(subject.saveplots.basepath,...
    'Movement', ['Movement' subject.subj_nr]);
subject.saveplots.diode = fullfile(subject.saveplots.basepath,...
    'Diode', ['Diode' subject.subj_nr]);
subject.saveplots.sourcemodel = fullfile(subject.saveplots.basepath,...
    'QC_Sourcemodels', ['Sourcemodel' subject.subj_nr]);
subject.saveplots.HCPsourcemodel = fullfile(subject.saveplots.basepath,...
    'Sourcemodels', ['HCP_Sourcemodel' subject.subj_nr]);
subject.saveplots.templatesourcemodel = fullfile(subject.saveplots.basepath,...
    'QC_Sourcemodels', ['TemplateSourcemodel' subject.subj_nr]);
subject.saveplots.dics = fullfile(subject.saveplots.basepath,...
    'DICS', ['DICS' subject.subj_nr]);
subject.saveplots.HCPdics = fullfile(subject.saveplots.basepath,...
    'DICS', ['HCP_DICS' subject.subj_nr]);
subject.saveplots.timelocked = fullfile(subject.saveplots.basepath,...
    'Timelocked', ['Timelocked' subject.subj_nr]);

end

