function [handles] = HCP_source_plot(cfg,surf,sourcevals)
% function [handles] = HCP_source_plot(cfg,surf,source)
%
% allows to plot source results obtained from a cortical mesh
%
% cfg.opacity: specifies the opacities of the vertices e.g. [0,1]
% cfg.colormapping: defines the colorrange of the vertices e.g. [0,1]
% cfg.mask matrix specifying opacity, format: [n,1] for n vertices
% cfg.braincolor: value between zero and one that sets the color of the 
%                 non-labeled brain areas. Default = 0.85.
% cfg.labelalpha: value between zero and one that sets the opacity of the 
%                 labels. Default = 1.

if nargin < 3
  cfg = struct();
end

if ~isfield(cfg, 'braincolor')
    cfg.braincolor = 0.85;
end

if ~isfield(cfg, 'labelalpha')
    cfg.labelalpha = 1;
end

if ~isfield(surf, 'labels')
    surf.labels = repmat({'none'}, length(surf.pos), 1);
end

allLabels = unique(surf.labels);
validLabels = allLabels(cellfun(@(x) ~isequal(x, 'none'), unique(surf.labels)));
[~, surflabels] = ismember(surf.labels, validLabels);

labelcolors = gray(length(validLabels)+1);
%labelcolors = [0 0 0; labelcolors];
surflabelColored = labelcolors(surflabels+1,:);
surflabelColored(~any(surflabelColored,2),:) = cfg.braincolor;

% plot brain

h0 = patch('Vertices', surf.pos, 'Faces', surf.tri, ...
    'FaceColor', repmat(cfg.braincolor,1,3), 'Edgecolor', 'none', ...
    'FaceAlpha', 1);

if ~exist('sourcevals', 'var')||isempty(sourcevals)
    % only plot brain
    
    % Brain and area labels
    h1 = patch('Vertices', surf.pos, 'Faces', surf.tri, ...
        'FaceVertexCData', surflabelColored, 'Edgecolor', 'none', ...
        'FaceColor', 'interp', 'FaceAlpha', cfg.labelalpha);
    handles.h1 = h1;
    %set(h1, 'FaceVertexAlphaData', double(~any(surflabels,2)));
    axis   off;
    axis vis3d;
    axis equal;
    lighting gouraud
    material dull
    return
end

if ~isfield(cfg,'opacity')
    if mean(sourcevals(:)) <0
        opacvals = sourcevals .*-1;
    else
        opacvals = sourcevals;
    end
        opacmin = min(opacvals)-0.01;
        opacmax = max(opacvals)+0.01;
else
  opacmin = cfg.opacity(1);
  opacmax = cfg.opacity(2);
  opacvals = cfg.opacity;
end
if ~isfield(cfg,'colormapping')
  fcolmin = min(sourcevals);
  fcolmax = max(sourcevals);
else
  fcolmin = cfg.opacity(1);
  fcolmax = cfg.opacity(2);
end
if ~isfield(cfg,'mask')
    opacvals(abs(opacvals)<0.000001) = min(opacvals); % make non-significant values transparent
    cfg.mask = opacvals;
    masking = 'values';
elseif strcmpi(cfg.mask, 'no')
    %cfg.mask = ((51*double(any(abs(sourcevals-100),2))));
    cfg.mask = double(opacvals ~=0)*0.75;
    masking = 'significance';
end

% Brain and area labels
h1 = patch('Vertices', surf.pos, 'Faces', surf.tri, ...
    'FaceVertexCData', surflabelColored, 'Edgecolor', 'none', ...
    'FaceColor', 'interp', 'FaceAlpha', cfg.labelalpha);
%set(h1, 'FaceVertexAlphaData', double(~any(surflabels,2)));
material dull
axis   off;
axis vis3d;
axis equal;

% Activation levels
switch masking
    case 'values'
h2 = patch('Vertices', surf.pos, 'Faces', surf.tri, ...
    'FaceVertexCData', sourcevals , 'FaceColor', 'interp',...
    'Edgecolor', 'none', 'FaceVertexAlphaData', cfg.mask,...
    'FaceAlpha', 'interp', 'AlphaDataMapping',   'scaled');
alim(gca, [opacmin opacmax]);

    case 'significance'
h2 = patch('Vertices', surf.pos, 'Faces', surf.tri, ...
    'FaceVertexCData', sourcevals , 'FaceColor', 'interp',...
    'Edgecolor', 'none', 'FaceVertexAlphaData', cfg.mask,...
    'FaceAlpha', 'flat', 'AlphaDataMapping',   'scaled');
alim(gca, [0 1]);
%set(h2, 'FaceVertexAlphaData', cfg.mask);

end
caxis(gca,[fcolmin fcolmax]);
lighting gouraud
material dull
%htemp = ylabel(hbar, 'Induced gamma power change');
%set(htemp,'rotation',270, 'VerticalAlignment','middle', 'HorizontalAlignment','right')

handles.h1 = h1;
handles.h2 = h2;
%handles.hbar = hbar;
