function [ peak ] = findGammaPeak( cfg, subject, data )
% findGammaPeak Detects gamma peak frequency of a spectrum using FOOOF. 
% Spectrum should be smooth, i.e. have a value every 0.5Hz or better.
%
% Inputs
% cfg   A cfg struct subpart, as created usint fillConfigs()
% data  The cleaned MEG data.
%
% Outputs
% peak  The fitted gamma frequency peak

subj_string = num2str(subject.subj_nr, '''%03d''');

% Select occipital channels
occipital = regexp(data.label, 'M.*O.*');
occipital = find(~cellfun(@isempty,occipital));
occipital_channels = data.label(occipital);

% Cut baselines and stim periods out of trials
conf = [];
conf.minlength = cfg.gammapeak.minLength;
conf.toilim = cfg.gammapeak.toi_baseline;
data_base = ft_redefinetrial(conf, data);
conf.toilim = cfg.gammapeak.toi_stim;
data_stim = ft_redefinetrial(conf, data);

% Find gamma peak
%% Run FFT for baseline and stimulus interval
conf            = [];
conf.method     = 'mtmfft';
conf.output     = 'pow';
conf.channel    = occipital_channels;
conf.taper      = 'dpss';
conf.foi        = 25 :0.5: 120;
conf.tapsmofrq  = 5;
conf.pad        = 4;
freq_base = ft_freqanalysis(conf, data_base);
freq_stim = ft_freqanalysis(conf, data_stim);

baseline = mean(freq_base.powspctrm,1);
changefreq = bsxfun(@rdivide, freq_stim.powspctrm, baseline);
psd = mean(changefreq,1);
psd = movmean(psd, 5); % apply slight frequency smoothing;
freq = freq_base.freq;

%% Fit gaussians
modeltypes = {'gauss1' 'gauss2' 'gauss3' 'gauss4' 'gauss5' 'gauss6' 'gauss7'};
for iModel = 1:length(modeltypes)
    [f{iModel}, gof, output] = fit(freq', psd', modeltypes{iModel});
    iAic(iModel) = output.numobs*log(gof.sse/output.numobs)+output.numparam*log(output.numobs);
    %AIC(gof.sse, output.numobs, output.numparam);
end
figure
plot(f,freq,psd)

% exclude too wide peaks, peaks outside the range, etc
coeffvalues(f)

end
