function [ peak ] = findGammaPeakFOOOF( cfg, subject, data, filenamepart )
% findGammaPeak Detects gamma peak frequency of a spectrum using FOOOF. 
% Spectrum should be smooth, i.e. have a value every 0.5Hz or better.
%
% Inputs
% cfg   A cfg struct subpart, as created usint fillConfigs()
% data  The cleaned MEG data.
% filenamepart A string that will be added to the plot paths. Helpful if we're
% doing multiple fits per subject.
%
% Outputs
% peak  The fitted gamma frequency peak(s)

subj_string = num2str(subject.subj_nr, '''%03d''');
if exist('filenamepart', 'var') && ~isempty(filenamepart)
    subj_string = [filenamepart, subj_string];
end

% Select occipital channels
occipital = regexp(data.label, 'M.*O.*');
occipital = find(~cellfun(@isempty,occipital));
occipital_channels = data.label(occipital);

% Cut baselines and stim periods out of trials
conf = [];
conf.minlength = cfg.gammapeak.minLength;
conf.toilim = cfg.gammapeak.toi_baseline;
data_base = ft_redefinetrial(conf, data);
conf.toilim = cfg.gammapeak.toi_stim;
data_stim = ft_redefinetrial(conf, data);

% Find gamma peak
% Run FFT for baseline and stimulus interval
conf            = [];
conf.method     = 'mtmfft';
conf.output     = 'pow';
conf.channel    = occipital_channels;
conf.taper      = 'dpss';
conf.foi        = 2 :0.5: 200;
conf.tapsmofrq  = 5;
conf.pad        = 4;
freq_base = ft_freqanalysis(conf, data_base);
freq_stim = ft_freqanalysis(conf, data_stim);

baseline = mean(freq_base.powspctrm,1);
changefreq = bsxfun(@rdivide, freq_stim.powspctrm, baseline);
psd = mean(changefreq,1);
psd = ESI_movmean(psd, 5); % apply slight frequency smoothing;
freq = freq_base.freq;
relevant = freq >= cfg.gammapeak.range(1) & freq <= cfg.gammapeak.range(2);

% Write out freq and psd for the python gamma peak fitting function to
% load.
freq_file = fullfile(cfg.paths.util, 'temp_freq.csv');
psd_file = fullfile(cfg.paths.util, 'temp_psd.csv');
writetable(table(freq(:)), freq_file, 'WriteVariableNames', false);
writetable(table(psd(:)), psd_file, 'WriteVariableNames', false);

%% Make three system calls in one: 
% - Change folder to utility functions
% - Activate the conda fooof environment (needs to be installed, and contain
%   fooof, dependencies, and matplotlib).
% - Call the Gamma peak fitting function written in python.
command1 = sprintf('cd %s;', cfg.paths.util);
command3 = sprintf('./fitGammaPeak.py %s %i %i', subj_string, ...
                      cfg.gammapeak.range(1), cfg.gammapeak.range(2));
command = strcat(command1, command3);
[status, peak] = system(command, '-echo');
assert(status==0, 'System call to FOOOF failed.');

peak = str2double(peak);

if isequal(peak, 999) % if no peak was found
    warning('No gamma peak was found.');
    peak = NaN;
end


%% Clean up potential peaks returned from FOOOF
if ~isnan(peak)
fPeaks = readtable(fullfile(cfg.paths.util, ['gamma_' subj_string '.csv']),...
    'ReadVariableNames', false);
fPeaks.Properties.VariableNames = {'Freq', 'AmpOverF', 'Bandwidth'};
fPeaks.PeakNum = [1:size(fPeaks,1)]';
fPeaks.AmpRaw = pow2pChange(interp1(freq, psd, fPeaks.Freq));
fPeaks = fPeaks(fPeaks.AmpRaw > 0,:); % remove negative change peaks
fPeaks = fPeaks(fPeaks.Bandwidth < 50,:); % remove broadband peaks
fPeaks = fPeaks(fPeaks.Freq > cfg.gammapeak.range(1) +3 & ...
    fPeaks.Freq < cfg.gammapeak.range(2) -3,:); % remove edge peaks

% remove peaks that have no actual peak in the data in their area
[~, rawPeaks] = findpeaks(psd(relevant), freq(relevant), ...
    'MinPeakDistance', 5, 'MinPeakProminence', 0.01);
valid = [];
for iPeak = 1:length(fPeaks.Freq)
    peakDist = min(abs(fPeaks.Freq(iPeak) - rawPeaks));
    valid(iPeak) = peakDist <= 5;
end

valid = find(valid);
fPeaks = fPeaks(valid,:);
% remove the smaller peak of very close peakpairs
minDistance = 8; % in Hz
peakdist = abs(bsxfun(@minus, fPeaks.Freq, fPeaks.Freq.'));
peakdist(logical(triu(ones(size(peakdist)),0))) = NaN;
if any(peakdist(:) < minDistance)
   [peak2, peak1] = find(peakdist < minDistance);
   for iPeakpair = 1:length(peak1)
       twoPeaks = fPeaks([peak1(iPeakpair) peak2(iPeakpair)],:);
       [~, iSmaller] = min(twoPeaks.AmpRaw);
       remove(iPeakpair) = twoPeaks.PeakNum(iSmaller);
   end
   fPeaks(ismember(fPeaks.PeakNum, remove),:) = [];
end

if size(fPeaks,1) > 1
    % if the first peak is >150% of all other peaks, just pick this one
    fPeaks = sortrows(fPeaks, 'AmpOverF', 'descend');
    if fPeaks.AmpOverF(1) > 1.5*fPeaks.AmpOverF(2)
        fPeaks = fPeaks(1,:);
    end
end

if size(fPeaks, 1) == 0 % if no peak remains
    warning('No gamma peak was found.');
    peak = NaN;
elseif size(fPeaks,1) < 3
    peak = sort(fPeaks.Freq);
else % return NaN as a marker that there were more then is sensible
    peak = NaN;
    warning('Too many peaks!')
end
end

%% Create QC plot
relevant = freq >= cfg.gammapeak.range(1) & freq <= cfg.gammapeak.range(2);
fitfig = figure('Visible', 'off');
hold on
plot(freq(relevant), psd(relevant))
xlabel('Frequency (Hz)')
ylabel('Power change (fold)')
box off
for iPeak = 1:length(peak)
    line([peak(iPeak) peak(iPeak)], [min(psd(relevant)) max(psd(relevant))], ...
        'Color', 'k', 'LineStyle', '--');
end

plotpath = fullfile(cfg.paths.plot, 'Gamma_Peak_Fits', subj_string);
export_fig(plotpath, '-png')


%% Delete temporary files.
delete(freq_file, psd_file);

end
