function labels = label_HCP_areas( cfg, atlas, varargin )
%label_HCP_areas This function takes a list of any number of requested cortical 
% areas (for the naming scheme, see https://doi.org/10.1038/nature18933) and
% returns a vector of all vertices in the 32k fs_LR. The vector will be set
% to 'none' for all vertices not belonging to one of the requested areas, and
% to the requested area name for all vertices belonging to each area.
%
% The pre-loaded atlas (I  use Q1-Q6_RelatedValidation210.
% CorticalAreas_dil_Final_Final_Areas_Group_Colors.32k_fs_LR.dlabel.nii)
% can be provided as atlas to prevent reloading in multiple
% calls. Otherwise, it can be left empty.
%
% Example:
% labels = label_HCP_areas(cfg, [], 'V1', 'FEF', 'V4');

if ~exist('cfg', 'var')
    cfg = [];
    cfg.hemisphere = [];
end

% Check if the atlas has been provided. If not, load it.
if exist('atlas', 'var') && ~isempty(atlas)
    disp('Using provided atlas.');
    assert(isstruct(atlas),...
        'atlas should be the actual atlas (not the path).');
else
    disp('Loading HCP atlas from disk.');
    atlaspath = fullfile(cfg.paths.data, 'Freesurfed', 'HCP', ...
        ['Q1-Q6_RelatedValidation210.CorticalAreas_dil_Final_Final_Areas_'...
        'Group_Colors.32k_fs_LR.dlabel.nii']);
    atlas = ft_read_cifti(atlaspath, 'mapname', 'array', 'readdata', true);
end

if isfield(cfg, 'hemisphere') && ~isempty(cfg.hemisphere)
    theseOnes = atlas.brainstructure == cfg.hemisphere;
    atlas.corticalareas_dil_final_final_areas_group_colors = ...
        atlas.corticalareas_dil_final_final_areas_group_colors(theseOnes);
end

% Go through atlas, find all vertices of the requested labels.
labels = repmat({'none'}, ...
    size(atlas.corticalareas_dil_final_final_areas_group_colors,1), 1);
for iLabel = 1:nargin-2
    thisLabel = ['_' varargin{iLabel} '_'];
    iL = find(contains(...
        atlas.corticalareas_dil_final_final_areas_group_colorslabel,...
        thisLabel, 'IgnoreCase', true));
    iArea = ismember(...
        atlas.corticalareas_dil_final_final_areas_group_colors, iL);
    labels(iArea) = {varargin{iLabel}};
    fprintf('%i %s vertices found.\n', sum(iArea), varargin{iLabel});
end

end