function [ freq_merged ] = mergeFFTSnippets( freq_results, trial_nr, n_dims )
% mergeFFTSnippets Average ft_freqanalysis results according to their
% orginial trial. Run this on results from data that have several data
% snippets per trial due to window-cutting or artefact exclusion.
%
% Inputs:
% STRUCT freq_results   Output of ft_freqanalysis.
% VECT trial_nr         Vector of the number of the trial the "trials" in
%                       freq_results originally came from. If, for example,
%                       trial 1 has been cut into three snippets and trial
%                       2 into two, the "trials" in freq_results will be [1
%                       2 3 4 5], while trial_nr should be [1 1 1 2 2].
% INT n_dims            How many dimensions do the results have? Currently
%                       only implemented for trials*channels*freqs or
%                       trials*channels. I require this as an explicit
%                       argument to prevent the code from breaking
%                       silently by producing means over the wrong
%                       dimensions.
%
% Output:
% STRUCT freq_merged    Same as freq_results, except that trial snippet
%                       from the same trial have been averaged into one
%                       trial value.

ft_checkdata(freq_results, 'datatype', 'freq');
assert(regexpi(freq_results.dimord, 'rpt*') ==1, ...
    'Trials need to be on first dimension of freq_result.');
assert(n_dims == ndims(freq_results.powspctrm));

freq_merged = freq_results;
freq_merged.old = freq_results; % For later consistency checking
trials = unique(trial_nr);

switch n_dims
    case 3
        % Average the powspctrm.
        new_powspctrm = NaN(length(trials),...
            size(freq_results.powspctrm, 2),...
            size(freq_results.powspctrm, 3));
        new_trial_nr = NaN(length(trials) ,1);
        for iTrial = 1:length(trials)
            this_trial = trials(iTrial);
            snippets = freq_results.powspctrm(trial_nr == this_trial, :,:);
            new_powspctrm(iTrial,:,:) = squeeze(mean(snippets, 1));
            new_trial_nr(iTrial) = this_trial;
        end
        freq_merged.powspctrm = new_powspctrm;
        
        % Remove / edit info variables
        freq_merged.trialinfo = new_trial_nr;
        if isfield(freq_merged, 'cumsumcnt') 
            freq_merged = rmfield(freq_merged, 'cumsumcnt');
        end
        if isfield(freq_merged, 'cumtapcnt')
            freq_merged = rmfield(freq_merged, 'cumtapcnt');
        end
        
    case 2
        % Average the powspctrm.
        new_powspctrm = NaN(length(trials), size(freq_results.powspctrm, 2));
        new_trial_nr = NaN(length(trials) ,1);
        for iTrial = 1:length(trials)
            this_trial = trials(iTrial);
            snippets = freq_results.powspctrm(trial_nr == this_trial, :);
            new_powspctrm(iTrial,:) = squeeze(mean(snippets, 1));
            new_trial_nr(iTrial) = this_trial;
        end
        freq_merged.powspctrm = new_powspctrm;
        
        % Remove / edit info variables
        freq_merged.trialinfo = new_trial_nr;
        if isfield(freq_merged, 'cumsumcnt') 
            freq_merged = rmfield(freq_merged, 'cumsumcnt');
        end
        if isfield(freq_merged, 'cumtapcnt')
            freq_merged = rmfield(freq_merged, 'cumtapcnt');
        end
        
    otherwise
        error(['Currently only implemented for trials*channels*freqs '...
            'or trials*channel data.']);
end

end

