function smoothedGroupLinePlot(X, Y, varargin )
% SMOOTHEDLINEPLOT Convolves data in Y with a binning kernel, computes
% per-bin confidence intervals, plots result as a line over X with shaded
% confidence interval ares. Averages over different series of observations,
% e.g. data from several participants, days, etc. The series are provided
% as matching cells in Y and X.
%
% Required inputs:
% CELL Y        Data on the Y axis. Will be smoothed. Usually something like
%               spike rate, stock price, etc. Different cells contain one
%               series of observations each (e.g. one participant, one day,
%               etc).
% CELL X        Data on the X axis. Usually something like time, trial,
%               etc. Different cells contain one series of observations each
%               (e.g. one participant, one day, etc).
%
% Optional inputs (Provided as name-value-pairs, for example: 'span', 11):
% INT 'span'    What should the per-bin binsize be? Default =
%               0.2*length(X). Must be unequal.
% NUM 'cover'   What should the coverage of the confidence intervals be,
%               between zwero and one? Default = 0.95.
% VECT 'fullX'  Full data on the X axis - Necessary if X has
%               missing data points. Must be the same for all series of
%               data points.
% BOOL 'demean' Should Y be demeaned per series? Sensible if the different
%               participants/days/etcs have different means. Default =
%               false.
% BOOL 'zscore' Should Y be z-scored per series? Sensible if the different
%               participants/days/etcs have different means and variances.
%               Default = false.
% BOOL 'bootci' Should the confidence intervals be computed using
%               bootstrapping instead of parametrically? Can be time
%               consuming. Default = false.
% BOOL 'logtransform' Should Y be logtransformed?
%               Default = false.
% MAT 'blocks'  In general, smoothing should not be run over block borders.
%               To prevent the smoothing to smooth over block borders and to add
%               per-block line colors as well as a legend, provide block
%               borders as
%               [1stTrial1stBlock lastTrial1stBlock;
%                1stTrial2ndBlock lastTrial2ndBlock; ... ]
% VECT 'color'  Color of the line + shading as RGB triplet. Default = lines
%               matlab color scheme.
% BOOL 'showLegend' Should a legend be plotted? Default = true.
% CHAR 'legendloc' Position of the legend (as per Matlab standard syntax).
%               Default = 'southoutside'.
% Char 'legendori' Orientation if the legend (as per Matlab standard
%               syntax). Default = 'horizontal'.
%
% Requires ESI_stats and boundedline-pkg

%% Set up defaults, validate and parse inputs
parser = inputParser;
parser.PartialMatching = false;
parser.addRequired('X', @(x) all(cellfun(@isnumeric, x)) && ...
    all(cellfun(@isvector, x)) && isvector(x));
parser.addRequired('data', @(x) all(cellfun(@isnumeric, x)) && ...
    all(cellfun(@isvector, x)) &&...
    isequal( cellfun(@length, x), cellfun(@length, X) ) &&...
    isvector(x));
parser.addParameter('span', 2*floor((0.2*length(X))/2)+1, ...
    @(x) x==floor(x) && ...
    x<min(cellfun(@length, X)) && mod(x,2) == 1);
parser.addParameter('cover', 0.95, @(x) isnumeric(x) && x>0 && x<1);
parser.addParameter('fullX', X, @(x) isnumeric(x) && isvector(x));
parser.addParameter('zscore', false, @(x) isBoolean(x) && isscalar(x));
parser.addParameter('bootci', false, @(x) isBoolean(x) && isscalar(x));
parser.addParameter('demean', false, @(x) isBoolean(x) && isscalar(x));
parser.addParameter('logtransform', false, @(x) isBoolean(x) && isscalar(x));
parser.addParameter('blocks', [min(cellfun(@min,X)) max(cellfun(@max,X))], ...
    @(x) isnumeric(x) && size(x,2) == 2)
parser.addParameter('color', [0 0 0], @(x) isnumeric(x));
parser.addParameter('showLegend', true, @(x) isBoolean(x) && isscalar(x));
parser.addParameter('legendloc', 'southoutside', @(x) ischar(x));
parser.addParameter('legendori', 'horizontal', @(x) ischar(x));

parser.parse(X, Y, varargin{:});

if isrow(X) % make sure input cells are column data to simplify indexing
    X = X.';
    Y = Y.';
end

if isrow(parser.Results.fullX)
    fullX = parser.Results.fullX.';
else
    fullX = parser.Results.fullX;
end
blocks = parser.Results.blocks;

%% If requested: z-score/demean per series
if parser.Results.logtransform
    for iSeries = 1:length(Y)
        Y{iSeries} =  log(Y{iSeries});
    end
end

if parser.Results.demean
    for iSeries = 1:length(Y)
        Y{iSeries} =  Y{iSeries} - ESI_nanmean(Y{iSeries});
    end
end

if parser.Results.zscore
    for iSeries = 1:length(Y)
        % work around NaNs
        dat = Y{iSeries};
        nonan = ~isnan(dat);
        dat = zscore(dat(nonan));
        full = NaN(1,length(Y{iSeries}));
        full(nonan) = dat;
        Y{iSeries} =  full;
    end
end

%% If needed: refill Y and X up tp their full size, set missing values to NaN
for iSeries = 1:length(Y)
    if ~isequal(X{iSeries}, fullX)
        offset = min(fullX) -1;
        newY = NaN(length(fullX), 1);
        newX = X{iSeries} - offset;
        newY(newX) = Y{iSeries};
        Y{iSeries} = newY;
    end
end

%% Set up kernels
sum_kernel = ones(parser.Results.span,1);

%% Compute sliding means and CIs
% This was originally done with convolution and nanconv, but sadly NaNs
% still lead to wrong results. Is now done using a loop, for which I will
% hang my head in shame.
crit = norminv(1- (1 - parser.Results.cover) /2);
hold on

for iBlock = 1:size(blocks,1)
    first_index = find(blocks(iBlock,1) == fullX);
    last_index  = find(blocks(iBlock,2) == fullX);
    
    fullX_block = fullX(first_index:last_index);
    for iSeries = 1:length(Y)
        this_series = Y{iSeries};
        Y_block{iSeries} = this_series(first_index:last_index);
    end
    
    n_elements = zeros(length(Y), length(fullX_block));
    for iSeries = 1:length(Y)
        n_elements(iSeries,:) = double(~isnan(Y_block{iSeries}));
    end
    n_elements = sum(n_elements, 1);
    n_trials_per_bin = conv(n_elements, sum_kernel, 'same');
    
    means = NaN(length(fullX_block), 1);
    
    CIs = nan(length(fullX_block), 2);
    for i = 1:length(fullX_block)
        % Get data from this bin from all series
        lower = max(1, i-floor(parser.Results.span/2));
        upper = min(length(fullX_block), i+floor(parser.Results.span/2));
        
        thisData = cell(1,length(Y_block));
        for iSeries = 1:length(Y_block)
            thisSet = Y_block{iSeries};
            thisData{iSeries} = thisSet(lower:upper);
        end
        thisData = cell2mat(thisData);
        % Compute per-bin mean and CI
        means(i) = ESI_nanmean(thisData(:));
        if parser.Results.bootci
            ci = bootci(1000, {@ESI_nanmean, thisData(:)}, ...
                'alpha', 1-parser.Results.cover);
            CIs(i,:) = abs((ci-means(i))');
        else
            st_dev = ESI_nanstd(thisData(:));
            SE = st_dev / sqrt(n_trials_per_bin(i));
            CIs(i) = SE * crit;
        end
    end
    
    %% Plot it (requires https://github.com/kakearney/boundedline-pkg)
    boundedline(fullX_block, means, CIs, 'nan', ...
        'fill', 'cmap', parser.Results.color(iBlock,:));
    iLegend{iBlock} = sprintf('Block %i', iBlock);
end

if parser.Results.showLegend
    legend(iLegend, 'Location', parser.Results.legendloc ,...
        'Orientation', parser.Results.legendori);
end

end
