function smoothedLinePlot(X, Y, varargin )
%SMOOTHEDLINEPLOT Convolves data in Y with a binning kernel, computes
%per-bin confidence intervals, plots result as a line over X with shaded
%confidence interval ares.
%
% Required inputs:
% VECT Y        Data on the Y axis. Will be smoothed. Usually something like
%               spike rate, stock price, etc.
% VECT X        Data on the X axis. Usually something like time, trial, etc.
%
% Optional inputs (Provided as name-value-pairs, for example: 'span', 10):
% INT 'span'    What should the per-bin binsize be? Default = 0.2*length(X)
% NUM 'cover'   What should the coverage of the confidence intervals be,
%               between zwero and one? Default = 0.95.
% VECT 'fullX'  Full data on the X axis - Necessary if X has
%               missing data points.
% BOOL 'demean' Should Y be demeaned? Default = false.
% BOOL 'zscore' Should Y be z-scored? Default = false.
% VECT 'color'  Color of the lines + shadings as RGB triplets. Default =
%               lines color scheme.
% MAT 'blocks'  In general, smoothing should not be run over block borders.
%               To prevent the smoothing to smooth over block borders and to add
%               per-block line colors as well as a legend, provide block
%               borders as
%               [1stTrial1stBlock lastTrial1stBlock;
%                1stTrial2ndBlock lastTrial2ndBlock; ... ]
% CHAR 'legendloc' Position of the legend (as per Matlab standard syntax).
%               Default = 'southoutside'.
% Char 'legendori' Orientation if the legend (as per Matlab standard
%               syntax). Default = 'horizontal'.
%
% Requires ESI_stats and boundedline-pkg

%% Set up defaults, validate and parse inputs
parser = inputParser;
parser.PartialMatching = false;
parser.addRequired('X', @(x) isnumeric(x) && isvector(x) && length(x) == length(Y));
parser.addRequired('data', @(x) isnumeric(x) && isvector(x));
parser.addParameter('span', 2*floor((0.2*length(X))/2)+1, ...
    @(x) x==floor(x) && x<length(X) && mod(x,2) == 1);
parser.addParameter('cover', 0.95, @(x) isnumeric(x) && x>0 && x<1);
parser.addParameter('fullX', X, @(x) isnumeric(x) && isvector(x));
parser.addParameter('demean', false, @(x) isBoolean(x) && isscalar(x) &&...
    ~(x && zscore));
parser.addParameter('zscore', false, @(x) isBoolean(x) && isscalar(x));
parser.addParameter('blocks', [min(X) max(X)], @(x) isnumeric(x) &&...
    size(x,2) == 2)
parser.addParameter('color', lines(100), @(x) isnumeric(x));
parser.addParameter('legendloc', 'southoutside', @(x) ischar(x));
parser.addParameter('legendori', 'horizontal', @(x) ischar(x));
parser.parse(X, Y, varargin{:});

fullX = parser.Results.fullX;
blocks = parser.Results.blocks;
%% If necessary: refill Y and X up tp their full size, set missing values to NaN
if ~isequal(X, fullX)
    offset = min(fullX) -1;
    newY = NaN(length(fullX), 1);
    X = X - offset;
    newY(X) = Y;
    Y = newY;
end

%% If requested: demean/z-score Y
if parser.Results.demean, Y =  Y - ESI_nanmean(Y); end
if parser.Results.zscore, Y = ESI_zscore(Y); end

%% Set up kernels
sum_kernel = ones(parser.Results.span,1);

%% Compute sliding means and CIs
% This was originally done with convolution and nanconv, but sadly NaNs
% still lead to wrong results. Is now done using a loop, for which I shall
% hang my head in shame.
crit = norminv(1- (1 - parser.Results.cover) /2);
hold on

for iBlock = 1:size(blocks,1)
    first_index = find(blocks(iBlock,1) == fullX);
    last_index  = find(blocks(iBlock,2) == fullX);
    Y_block     = Y(first_index:last_index);
    fullX_block = fullX(first_index:last_index);
    
    n_elements = double(~isnan(Y_block));
    n_trials_per_bin = conv(n_elements, sum_kernel, 'same');
    
    means = NaN(length(Y_block), 1);
    CIs = NaN(length(Y_block), 1);
    
    for i = 1:length(fullX_block)
        lower = max(1, i-floor(parser.Results.span/2));
        upper = min(length(Y_block), i+floor(parser.Results.span/2));
        % Compute per-bin mean and CI
        means(i) = ESI_nanmean(Y_block(lower:upper));
        st_dev = ESI_nanstd(Y_block(lower:upper));
        SE = st_dev / sqrt(n_trials_per_bin(i));
        CIs(i) = SE * crit;
    end
    
    %% Plot it (requires https://github.com/kakearney/boundedline-pkg)
    boundedline(fullX_block, means, CIs, 'alpha', 'nan', ...
        'fill', 'cmap', parser.Results.color(iBlock,:));
    iLegend{iBlock} = sprintf('Block %i', iBlock);
end
legend(iLegend, 'Location', parser.Results.legendloc ,...
    'Orientation', parser.Results.legendori);

end
