function [ behav_summed, new_trial_nr ] = sumTrialSnippets( behav_results, trial_nr )
% sumTrialSnippets Sum any behavioral results (eg number of microsaccades)
% according to their orginial trial. Run this on results from data that have
% several data snippets per trial due to window-cutting or artefact exclusion.
%
% Inputs:
% VECT behav_results   	Per-trial measure that should be summed per original 
%                       trial, e.g. number of microsaccades.
% VECT trial_nr         Vector of the number of the trial the "trials" in
%                       behav_results originally came from. If, for example,
%                       trial 1 has been cut into three snippets and trial
%                       2 into two, the "trials" in behav_results will be [1
%                       2 3 4 5], while trial_nr should be [1 1 1 2 2].
%
% Output:
% VECT behav_summed     Same as behav_results, except that trial snippet
%                       from the same trial have been summed into one
%                       trial value.
% VECT new_trial_nr     Trial number of the merged behav_results.

assert(isvector(behav_results), 'Currently only vectors can be summed.');
assert(isequal(size(behav_results), size(trial_nr)), ['behav_results and '...
    'trial_nr must be of equal size.']);

trials = unique(trial_nr);
behav_summed = NaN(length(trials), 1);
new_trial_nr = NaN(length(trials) ,1);

for iTrial = 1:length(trials)
    this_trial = trials(iTrial);
    snippets = behav_results(trial_nr == this_trial);
    behav_summed(iTrial) = sum(snippets);
    new_trial_nr(iTrial) = this_trial;
end

end

