function [ eyetraceD ] = voltToDegrees( cfg, eyetraceV )
% VOLTTODEGREES This function transforms eyetracker traces as recorded in
% the MEG (which are recorded in volt - output of the DAC) into optical
% degrees. The transform highly depends on physical setup, screen size, and
% the physical.ini and final.ini files on the eyetracking PC, so those
% values need to be reset by hand if anything changes. Can be applied as a
% cellfun over trials in fieldtrip data struct.
%
% Input
% STRCT cfg     The cfg.eyetracker substruct of the general cfg struct of
%               the analysis. Needs fields distanceCm, screenwidthCm,
%               screenheightCm, sreenwidthPx, screenheightPx, convFactorX, 
%               convFactorY. The convFactor can be found by regressing the
%               voltage values onto the pixel values in the edf file,
%               everything else comes from the ini files on the eyetracking
%               pc.
% MAT eyetraceV The voltage values of the eyetrace. Needs to be a 2-row
%               matrix, with X-values on the first and Y-values on the
%               second row.

assert(size(eyetraceV,1) == 2, 'X and Y values need to be on two rows of the input.');

% Find necessary constants
pixCm = mean([cfg.screenheightCm/cfg.screenheightPx,...
              cfg.screenwidthCm/cfg.screenwidthPx]); % pixel size in cm

% Transform volt to pixel distance from middle of screen.
eyetracePx = [eyetraceV(1,:)*cfg.convFactorX; eyetraceV(2,:)*cfg.convFactorY];
% Transform this to cm distance from middle of screen
eyetraceCm = eyetracePx * pixCm;
% Transform this to degrees from middle of screen
eyetraceD = 2*atand(eyetraceCm./(2*cfg.distanceCm));

assert(isreal(eyetraceD), 'I''m getting imaginary eyetrace outputs.');
end

